/***************************************************************************
                           element.cpp
                           -------------------------------
    Copyright            : (C) 1999-2003 by Stefan Preis
                         : (C) 2004-2010 Guido Scholz
    email                : guido.scholz@bayernline.de
    last modified        : $Date: 2009-11-01 20:40:38 $
                           $Revision: 1.198 $
***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/

/***************************************************************************
   this file sets up the outfit of one element only and handles the sending
   of the switch command to the erddcd daemon
 ***************************************************************************/

#include <QApplication>
#include <QAction>
#include <QMenu>
#include <QPixmap>
#include <QPolygon>
#include <QDrag>
#include <QMimeData>

#include "drivedialog.h"
#include "dualdrivedialog.h"
#include "element.h"
#include "elementlabeldialog.h"
#include "feedbacktriggerdialog.h"
#include "preferences.h"
#include "resources.h"
#include "triggeraspectdialog.h"
#include "variantdialog.h"
#include "virtualaddressdialog.h"

/* track buttons */
#include "pixmaps/button-red.xpm"
#include "pixmaps/button-gray.xpm"
#include "pixmaps/button-black.xpm"
#include "pixmaps/button-yellow.xpm"

/* labels for external group buttons */
#include "pixmaps/signal-w.xpm"
#include "pixmaps/signal-wr.xpm"

/*pixmaps for variant dialog*/
#include "pixmaps/signal_hss_st1.xpm"
#include "pixmaps/signal_hss_st2.xpm"
#include "pixmaps/signal_hss_st3.xpm"
#include "pixmaps/signal_hs_st1.xpm"
#include "pixmaps/signal_hs_st2.xpm"
#include "pixmaps/signal_hs_st3.xpm"
#include "pixmaps/signal_vs_st1.xpm"
#include "pixmaps/signal_vs_st2.xpm"
#include "pixmaps/signal_vs_st3.xpm"
#include "pixmaps/entkoppler_st1.xpm"
#include "pixmaps/entkoppler_st2.xpm"
#include "pixmaps/entkoppler_st3.xpm"
#include "pixmaps/dkw_links_st2.xpm"
#include "pixmaps/dkw_links_st3.xpm"
#include "pixmaps/dkw_rechts_st2.xpm"
#include "pixmaps/dkw_rechts_st3.xpm"


// Element directions
enum {
   DIR_HP0 = 0,
   DIR_HP1,
   DIR_HP2,
   DIR_SH1
};

// signal light size parameter
// delay for edit mode after element locating
enum {
    LIGHTSIZE = 6,
    LOCATE_TIMER = 5000
};



element::element(QWidget* parent, SpdrItemClassId cid)
    : GrayPanel(parent, cid), TableLight(true)
{
    setObjectName("gbsitem");
    initVariables();
    updateProperties();
    setupElementIcon();
}


element::element(QTextStream& ts, QWidget* parent, SpdrItemClassId cid)
    : GrayPanel(parent, cid), TableLight(true)
{
    setObjectName("gbsitem");
    initVariables();
    visualMode = kvmNormal;
    readFileTextFromStream(ts);
    updateProperties();
    setupElementIcon();
}


/*set all variables which are not read from file*/
void element::initVariables()
{
    if (classid == siciAdr)
        iSoldInvert = 1;
    else
        iSoldInvert = -1;
    protocol1 = GaMessage::proNone;
    protocol2 = GaMessage::proNone;
    address1 = -1;
    address2 = -1;
    xchangeport1 = -1;
    xchangeport2 = -1;
    state = -1;
    iSoldSubType = -1;
    sSoldText = "";
    activetime1 = -1;
    activetime2 = -1;
    iFBContact = 1;
    trackindicatoroff = true;
    /*button 1 trigger*/
    enable1fbtrigger = false;
    button1fbbus = 1;
    button1fbcontact = 1;
    /*button 2 trigger*/
    enable2fbtrigger = false;
    button2fbbus = 1;
    button2fbcontact = 1;
    /*halt aspect trigger*/
    octriggerhalt = false;
    retriggerclear = false;
    fbtriggerhaltbus = 1;
    fbtriggerhaltcontact = 1;
    /*clear aspect trigger*/
    octriggerclear = false;
    retriggerhalt = false;
    fbtriggerclearbus = 1;
    fbtriggerclearcontact = 1;

    editsAddress = 0;
    ffmactive = false;
    ffm = false;
    occupied = false;
    routable = false;
    routed = false;
    routemark = false;
    signal = false;
    state2dkw = false;
    switchable = false;
    switched = false;
    simplega = false;
    turnout = false;
    lightson = true;
    lastdir = 0;
    newdir = 0;
    bus1 = bus2 = iFBBusNo = 1;
    port1 = 1;
    port2 = 1;

    // this is only used for crossings to choose the routed track
    routedtrack = 0;
    //parameter to remember direction of last route, used for detection
    //of route direction changes
    penultimateroutedir = lastrouteentrydir = entryDir();;

    lockCounter = 0;
    blinkcounter = 0;

    commander = NULL;
    setAcceptDrops(true);
}

/* Read the layout element data from stream.
 * Lines starting with # are recognized as comments, lines starting with
 * % are recognized as end of dataset marker (new style file format)*/
void element::readFileTextFromStream(QTextStream& ats)
{
    QString s, key, value;

    while (!ats.atEnd()) {
        s = ats.readLine();
        if (!s.startsWith("#")) {
            key = s.section(DS, 0, 0);
            value = s.section(DS, 1, 1).trimmed();

            /* key/value pairs are read sequence independent */
            if (key.compare(GF_CLASSID) == 0) {
                  classid = (SpdrItemClassId)value.toInt();
            }
            else if (key.compare(GF_INDEX) == 0) {
                  iSoldIndex = value.trimmed().toUInt();
            }
            else if (key.startsWith("%")) {
                /*end of dataset, exit while loop*/
                  break;
            }
            else if (key.compare(GF_INVERSTO) == 0) {
                iSoldInvert = value.toInt();
            }
            else if (key.compare(GF_PROTOCOL1) == 0) {
                if (value == "M")
                    protocol1 = GaMessage::proMM;
                else if (value == "N")
                    protocol1 = GaMessage::proDCC;
                else if (value == "P")
                    protocol1 = GaMessage::proServer;
                else if (value == "S")
                    protocol1 = GaMessage::proSelectrix;
                else
                    protocol1 = GaMessage::proNone;
            }
            else if (key.compare(GF_PROTOCOL2) == 0) {
                if (value == "M")
                    protocol2 = GaMessage::proMM;
                else if (value == "N")
                    protocol2 = GaMessage::proDCC;
                else if (value == "P")
                    protocol2 = GaMessage::proServer;
                else if (value == "S")
                    protocol2 = GaMessage::proSelectrix;
                else
                    protocol2 = GaMessage::proNone;
            }
            /*old style compatibility*/
            else if (key.compare(GF_PROTOCOL) == 0) {
                if (value == "M")
                    protocol1 = GaMessage::proMM;
                else if (value == "N")
                    protocol1 = GaMessage::proDCC;
                else if (value == "P")
                    protocol1 = GaMessage::proServer;
                else if (value == "S")
                    protocol1 = GaMessage::proSelectrix;
                else
                    protocol1 = GaMessage::proNone;
                protocol2 = protocol1;
            }
            else if (key.compare(GF_ADDRESS1) == 0) {
                bus1 = value.toInt();
                value = s.section(DS, 2, 2).trimmed();
                address1 = value.toInt();
                value = s.section(DS, 3, 3).trimmed();
                port1 = value.toUInt();
            }
            else if (key.compare(GF_ADDRESS2) == 0) {
                bus2 = value.toInt();
                value = s.section(DS, 2, 2).trimmed();
                address2 = value.toInt();
                value = s.section(DS, 3, 3).trimmed();
                port2 = value.toUInt();
            }
            else if (key.compare(GF_XCHCONN1) == 0) {
                xchangeport1 = value.toInt();
            }
            else if (key.compare(GF_XCHCONN2) == 0) {
                xchangeport2 = value.toInt();
            }
            else if (key.compare(GF_DIRECTION) == 0) {
                state = value.toInt();
            }
            else if (key.compare(GF_SUBTYPE) == 0) {
                iSoldSubType = value.toInt();
            }
            else if (key.compare(GF_TEXT) == 0) {
                sSoldText = s.section(DS, 1).trimmed();
                // filter old flag for "no label text"
                if (sSoldText == "-1")
                    sSoldText = "";
            }
            else if (key.compare(GF_ACTTIME1) == 0) {
                activetime1 = value.toInt();
            }
            else if (key.compare(GF_ACTTIME2) == 0) {
                activetime2 = value.toInt();
            }
            /*old style compatibility*/
            else if (key.compare(GF_ACTTIME) == 0) {
                activetime1 = value.toInt();
                activetime2 = activetime1;
            }
            else if (key.compare(GF_FBPORT) == 0) {
                iFBBusNo = value.toUInt();
                value = s.section(DS, 2, 2).trimmed();
                iFBContact = value.toInt();
                if (iFBContact <= 0)
                    iFBContact = 1;
            }
            else if (key.compare(GF_BUTTON1FB) == 0) {
                enable1fbtrigger = value.toUInt();
                value = s.section(DS, 2, 2).trimmed();
                button1fbbus = value.toUInt();
                value = s.section(DS, 3, 3).trimmed();
                button1fbcontact = value.toUInt();
            }
            else if (key.compare(GF_BUTTON2FB) == 0) {
                enable2fbtrigger = value.toUInt();
                value = s.section(DS, 2, 2).trimmed();
                button2fbbus = value.toUInt();
                value = s.section(DS, 3, 3).trimmed();
                button2fbcontact = value.toUInt();
            }
            else if (key.compare(GF_TRIGGERHALT) == 0) {
                octriggerhalt = value.toUInt();
                value = s.section(DS, 2, 2).trimmed();
                retriggerclear = value.toUInt();
                value = s.section(DS, 3, 3).trimmed();
                fbtriggerhaltbus = value.toUInt();
                value = s.section(DS, 4, 4).trimmed();
                fbtriggerhaltcontact = value.toUInt();
            }
            else if (key.compare(GF_TRIGGERCLEAR) == 0) {
                octriggerclear = value.toUInt();
                value = s.section(DS, 2, 2).trimmed();
                retriggerhalt = value.toUInt();
                value = s.section(DS, 3, 3).trimmed();
                fbtriggerclearbus = value.toUInt();
                value = s.section(DS, 4, 4).trimmed();
                fbtriggerclearcontact = value.toUInt();
            }
            else if (key.compare(GF_HIDELEDS) == 0) {
                trackindicatoroff = (value.toInt() == 1);
                /*this is the last parameter for old style format;
                 * now exit while loop*/
                break;
            }
        }
    }
}

/**
 * update element type dependent property values
 *
 * initialize standard properties of this special symbol to avoid
 * recalculation in several procedures by expensive string
 * comparisons
 * */
void element::updateProperties()
{
    // couplers and crossings get the non-active direction on setup
    if (classid == siciEnk || classid == siciKrh ||
        classid == siciKr1 || classid == siciKl1)
        state = 0;

    // route marks are entry or exit points of routes (limits)
    routemark = classid == siciHs1 || classid == siciHs3
        || classid == siciHss1 || classid == siciHss3
        || classid == siciSs1 || classid == siciSs3
        || classid == siciSd1 || classid == siciSd3
        || classid == siciSh1 || classid == siciSh3
        || classid == siciWs1 || classid == siciWs3
        || classid == siciZt1 || classid == siciZt3
        || classid == siciRt1 || classid == siciRt3;

    // signals include also not route mark type signals
    signal = routemark
        || classid == siciVs1 || classid == siciVs3
        || classid == siciZp1 || classid == siciZp3;


    // init signals as they were saved in layout file or with red state
    if (pref.initsignalsred) {
        if (signal || classid == siciRel || classid == siciBld)
            state = 0;
    }

    if (signal)
        ffm = (classid == siciHs1 || classid == siciHs3
                || classid == siciHss1 || classid == siciHss3
                || classid == siciSh1 || classid == siciSh3);

    else if (classid == siciTl1 || classid == siciTl3 ||
            classid == siciTr1 || classid == siciTr3 ||
            classid == siciIl1 || classid == siciIr3 ||
            classid == siciIl3 || classid == siciIr1 ||
            classid == siciSy1 || classid == siciSy3 ||
            classid == siciTw3 || classid == siciTw1 ||
            classid == siciSl1 || classid == siciSl3 ||
            classid == siciSr1 || classid == siciSr3 ||
            classid == siciDl1 || classid == siciDr1)
        turnout = true;

    else if (classid == siciBld || classid == siciEnk ||
            classid == siciRel || classid == siciMdc)
        simplega = true;

    /*element can be a part of a route*/
    routable = signal || turnout ||
        classid == siciSt1 || classid == siciSt2 ||
        classid == siciSt3 || classid == siciSt4 ||
        classid == siciCr1 || classid == siciCr2 ||
        classid == siciCr3 || classid == siciCr4 ||
        classid == siciCl1 || classid == siciCl2 ||
        classid == siciCl3 || classid == siciCl4 ||
        classid == siciTdr || classid == siciTdl ||
        classid == siciTul || classid == siciTur ||
        classid == siciTuh || classid == siciTuv ||
        classid == siciTdb || classid == siciKrh ||
        classid == siciKr1 || classid == siciKl1 ||
        classid == siciBue || classid == siciAdr ||
        classid == siciBld || classid == siciEnk;

    /*element has decoder connected*/
    switchable = (signal || turnout || simplega) &&
        classid != siciZt3 && classid != siciZt1 &&
        classid != siciRt3 && classid != siciRt1;

    state2dkw = (classid == siciDl1 || classid == siciDr1)
        && iSoldSubType == 0;
}


bool element::is2StateDKW()
{
    return state2dkw;
}


bool element::isRouteMark()
{
    return routemark;
}


bool element::isSignal()
{
    return signal;
}


bool element::isRoutable()
{
    return routable;
}


bool element::isSwitchable()
{
    return switchable;
}


bool element::isSimpleGA()
{
    return simplega;
}


bool element::isTurnout()
{
    return turnout;
}


/**
 * switch element to new direction
 */
void element::switchToDir(int newdir)
{
    // quit if element contains no solenoid
    if (!switchable)
        return;

    if (turnout)
        switchToDirBlinking(newdir);
    else {
        // Repainting the element is only done when new direction differs
        // from the old one. Momentary couplers are exceptional: they
        // only use one connector e.g. one direction which is activated
        // or deactivated, these are treated the same
        // TODO: option to repaint only when INFO messages came back from
        // srcp server
        if (newdir != state || classid == siciEnk) {
            state = newdir;
            setupElementIcon();
            sendSrcpState();
        }
        else {
            // Switch command is send to SRCP server if forced.
            if (pref.sendstate)
                sendSrcpState();
        }
    }
}

/**
 * react to mouse press events
 */
void element::mousePressEvent(QMouseEvent* e)
{
    /*normal mode*/
    if (visualMode == kvmNormal) {
        if (e->button() == Qt::LeftButton) {
            GbsButtonState ctrlButton = kNoneClicked;
            QPoint CursorPos = mapFromGlobal(QCursor::pos());

            if (classid == siciMdc) {
                commander = new elementCommander(this, classid);
                connect(commander, SIGNAL(sendTtCommand(int, int)),
                        this, SLOT(slotUpdateCommanderData(int, int)));

                commander->exec();        // parent window NOT usable
                commander->move(QCursor::pos());
            }

            /* determine what type of button was pressed an send the
             * corresponding value to GBSArea to change cursor shape etc.*/
            // if element contains a solenoid or is a external button
            // TODO: if (hasButton())
            else if ((address1 != -1) && (classid != siciAdr)) {

                if (classid == siciHs1 || classid == siciHs3
                        || classid == siciZt1 || classid == siciZt3)
                    ctrlButton = kZfsClicked;

                else if (classid == siciSs1 || classid == siciSs3 ||
                        classid == siciSd1 || classid == siciSd3 ||
                        classid == siciWs1 || classid == siciWs3 ||
                        classid == siciRt1 || classid == siciRt3)
                    ctrlButton = kRfsClicked;

                else if (classid == siciHss1) {
                    /* two different buttons on this panel */
                    if (CursorPos.x() > (width() >> 1))
                        ctrlButton = kZfsClicked;
                    else
                        ctrlButton = kRfsClicked;
                }

                else if (classid == siciHss3) {
                    /* two different buttons on this panel */
                    if (CursorPos.x() > (width() >> 1))
                        ctrlButton = kRfsClicked;
                    else
                        ctrlButton = kZfsClicked;
                }

                else if (classid == siciSh1 || classid == siciSh3)
                    ctrlButton = kZhsClicked;

                else
                    ctrlButton = kTurnoutClicked;

                emit elementClicked(this, ctrlButton);
            }

            e->accept();
        }
    }
    /*layout edit mode*/
    else if (visualMode == kvmEditLayout) {
        if (e->button() == Qt::LeftButton) {
            /*handled by gbsarea*/
            e->ignore();
        }
    }
    /*route edit mode*/
    else if (visualMode == kvmEditRoute) {
        // do nothing
        e->accept();
    }
}

/**
 * react to mouse release events
 */
void element::mouseReleaseEvent(QMouseEvent* e)
{
    /*normal mode*/
    if (visualMode == kvmNormal) {
        if (e->button() == Qt::RightButton) {
            if (isSwitchable()) {
                QMenu menu;
                QAction* toggleAction = new QAction(tr("&Toggle"), &menu);
                toggleAction->setEnabled(ctxCanSwitch());
                menu.addAction(toggleAction);
                if (menu.exec(QCursor::pos()) != NULL)
                    toggle();
            }
            e->accept();
        }
    }

    /*layout edit mode*/
    else if (visualMode == kvmEditLayout) {
        if (e->button() == Qt::RightButton) {
            runPropertyMenue(e->pos());
            e->accept();
        }
        else
            e->ignore();
    }

    /*route edit mode*/
    else if (visualMode == kvmEditRoute) {
        if (e->button() == Qt::LeftButton) {
            /*select/deselect start or stop signal*/
            if (routemark) {
                /*send record signal to router*/
                if (ksmNormal == selectionMode)
                    emit recordElement(this, krecStartStop);
                else
                    emit recordElement(this, krecClear);
                e->accept();
            }
        }
        else if (e->button() == Qt::MiddleButton) {

            /*select/deselect switchable element*/
            /*and send record signal to router*/
            if (turnout || classid == siciMdc ||
                classid == siciRel || classid == siciBld ||
                classid == siciZp1 || classid == siciZp3 ||
                classid == siciHs1 || classid == siciHs3 ||
                classid == siciHss1 || classid == siciHss3 ||
                classid == siciSs1 || classid == siciSs3 ||
                classid == siciSd1 || classid == siciSd3 ||
                classid == siciSh1 || classid == siciSh3 ||
                classid == siciVs1 || classid == siciVs3 ||
                classid == siciWs1 || classid == siciWs3 ||
                classid == siciKr1 || classid == siciKl1 ||
                classid == siciKrh) {
                if (ksmNormal == selectionMode)
                    emit recordElement(this, krecNormal);
                else
                    emit recordElement(this, krecClear);
                e->accept();
            }

            /*select/deselect train number display*/
            /*and send record signal to router*/
            else if (classid == siciAdr) {
                if (ksmNormal == selectionMode)
                    emit recordElement(this, krecDisplay);
                else
                    emit recordElement(this, krecClear);
                e->accept();
            }
        }
        /*FIXME: same as in normal mode*/
        else if (e->button() == Qt::RightButton) {
            if (isSwitchable()) {
                QMenu menu;
                QAction* toggleAction = new QAction(tr("&Toggle"), &menu);
                toggleAction->setEnabled(ctxCanSwitch());
                menu.addAction(toggleAction);
                if (menu.exec(QCursor::pos()) != NULL)
                    toggle();
            }
            e->accept();
        }
    }

    /*track clear detection / track occupancy detection*/
    else if (visualMode == kvmEditClearance) {
        if (e->button() == Qt::LeftButton) {

            /*select/deselect track indicator elements*/
            /*and send record signal to element recorder*/
            if (routable) {
                if (ksmNormal == selectionMode)
                    emit recordElement(this, krecTrackIndicator);
                else
                    emit recordElement(this, krecClear);
            }
            e->accept();
        }
    }
    else
        e->ignore();
}


void element::slotShowElement(int address, int ns, elemSelectionMode sm)
{
    if (address == address1) {
        selectionMode = sm;
        if (ns != -1)
            state = ns;
        setupElementIcon();
    }
}


void element::showElementState(int ns, elemSelectionMode sm)
{
    selectionMode = sm;
    if (ns != -1)
        state = ns;
    setupElementIcon();
}


void element::locateMe()
{
    // activate edit mode for LOCATE_TIMER secs
    // if this is the searched element, show element in found mode
    selectionMode = ksmFoundEl;
    update();

    locateTimer = new QTimer();
    locateTimer->setSingleShot(true);
    locateTimer->start(LOCATE_TIMER);
    connect(locateTimer, SIGNAL(timeout()),
            this, SLOT(slotLocateTimerTimeout()));
}


void element::slotLocateTimerTimeout()
{
    delete locateTimer;
    // show element in normal mode
    selectionMode = ksmNormal;
    update();
}


void element::switchAddress(bool secondone)
{
    // default copy of direction + address
    int realstate = state;
    GaMessage::Protocol realprotocol = protocol1;
    unsigned int realactivetime = activetime1;
    unsigned int iRealBus = bus1;
    unsigned int iRealAddress = address1;


    // element contains a momentarily activated coupler
    if (classid == siciEnk && iSoldSubType != -1)
        realstate = iSoldSubType;  // copy subtype as realDirection

    // element contains a main signal with direction >= 2
    else if ((classid == siciHs1 || classid == siciHs3 ||
                classid == siciHss1 || classid == siciHss3 ||
              classid == siciVs1 || classid == siciVs3)
            && state >= 2) {

        // Hp0+Hp1 not considered, is done by default copy
        switch (iSoldSubType) {
            case 6:            // Hp0+Hp2,     --> state = 2
            case 7:            // Hp0+Hp2+Sh1, --> state = 2 or 3
                realstate = 1;
                // Sh1, case 7, HSS
                if (state == 3) {
                    iRealAddress = address2;
                    iRealBus = bus2;
                }
                break;
            case 4:            // Hp0+Hp1+Hp2, --> state = 2
                realstate = 0;
                iRealAddress = address2;
                iRealBus = bus2;
                break;
            case 1:            // Hp0+Hp1+Sh1, --> state = 3
            case 5:            // Hp0+Hp1+Hp2+Sh1, --> state = 2 or 3
                realstate = state - 2;
                iRealAddress = address2;
                iRealBus = bus2;
                break;
        }
    }

    // element contains a 3-way-turnout
    else if (classid == siciTw1 || classid == siciTw3) {

        // send second address data
        if (secondone) {
            realstate = (state > 1);
            realprotocol = protocol2;
            realactivetime = activetime2;
            iRealAddress = address2;
            iRealBus = bus2;
        }
        // send first address data
        else {
            realstate = state % 2;
        }
    }

    // element contains a 4-state-DKW or EKW
    else if ((classid == siciSl1 || classid == siciSl3 ||
                classid == siciSr1 || classid == siciSr3) ||
            ((classid == siciDl1 || classid == siciDr1) &&
             iSoldSubType == 1)) {

        // send second address data
        if (secondone) {
            realstate = (state == 1 || state == 2);
            realprotocol = protocol2;
            realactivetime = activetime2;
            iRealAddress = address2;
            iRealBus = bus2;
        }
        // send first address data
        else {
            realstate = (state >= 2);
        }
    }

    if (realprotocol == GaMessage::proNone)
        return;

    /*
     * The calculated real direction to be sent is modified
     * again if you electronically changed your decoder
     * outputs of one address.
     */
    if (classid != siciDre) {
        if (iRealAddress == (unsigned int) address1)
            realstate = realstate ^ xchangeport1;
        if (iRealAddress == (unsigned int) address2)
            realstate = realstate ^ xchangeport2;
    }

    unsigned int port = 0;

    switch (realprotocol) {

        // only two ports for MM
        case GaMessage::proMM:
            port = realstate;
            break;

        // only two ports for DCC
        case GaMessage::proDCC:
            port = !realstate;
            break;

        // multiple (paired) ports for Selectrix
        case GaMessage::proSelectrix:
            if (iRealAddress == (unsigned int) address1)
                port = port1 + realstate;
            else
                port = port2 + realstate;
            break;

        // multiple (paired) ports for Server
        case GaMessage::proServer:
            if (iRealAddress == (unsigned int) address1)
                port = port1 + realstate;
            else
                port = port2 + realstate;
            break;

        default:
            break;
    }

    GaMessage sm = GaMessage(SrcpMessage::mtGaSet, iRealBus,
            iRealAddress, realprotocol, port, 1, realactivetime);
    //sm.setGaData(realprotocol, iRealBus, iRealAddress, port, 1,
    //        realactivetime);
    emit sendSrcpMessage(&sm);

    /*qWarning("Class: %d, Stype: %d, Dir: %d, A1: %d, A2: %d, RA: %d, P: %d",
            classid, iSoldSubType, state, address1,
            address2, iRealAddress, port);*/
    qApp->processEvents();
}


void element::sendSrcpState()
{
    /* do not send anything for rail buttons without signals */
    if (classid == siciRt1 || classid == siciRt3 ||
            classid == siciZt1 || classid == siciZt3)
        return;

    // switch only first address
    switchAddress(false);

    // switch also second address if there is one
    if ((classid == siciTw1 || classid == siciTw3 ||
                classid == siciSl1 || classid == siciSl3 ||
                classid == siciSr1 || classid == siciSr3 ||
                ((classid == siciDl1 || classid == siciDr1) &&
                 iSoldSubType == 1))) {
        switchAddress(true);
    }

    /*
     * if momentary coupler: activate it, wait for a short time
     * and deactivate it graphically
     */
    else if (classid == siciEnk && iSoldSubType != -1)
        QTimer::singleShot(activetime1, this,
                SLOT(repaintTimeOutEnk()));
}


void element::repaintTimeOutEnk()
{
    state = !state;
    setupElementIcon();
}

/*handle info messages for items with two decoders*/
void element::switch2AddressItem(unsigned int addr, unsigned int port)
{
    int realport;

    if (addr == (unsigned int)address1) {
        realport = port ^ xchangeport1;
        if (protocol1 == GaMessage::proDCC)
            realport = realport == 0 ? 1 : 0;

        if (classid == siciHss1 || classid == siciHss3) {
            // Subtype 7
            if (iSoldSubType == 7) {
                if (realport == 0 && state != 0) {
                    state = 0;
                    setupElementIcon();
                }
                else if (realport == 1 && state == 0) {
                    state = 2;
                    setupElementIcon();
                }
            }
            // Subtype 1 + 5
            else if (state != realport) {
                state = realport;
                setupElementIcon();
            }
        }

        else if (classid == siciHs1 || classid == siciHs3 ||
                classid == siciVs1 || classid == siciVs3 ||
                classid == siciTw1 || classid == siciTw3) {
            if (state != realport) {
                state = realport;
                setupElementIcon();
            }
        }

        // Crossing Subtype 1, decoder 1
        else if (classid == siciDl1 || classid == siciDr1) {
            if (state == 0 && realport == 1) {
                state = 3;
                setupElementIcon();
            }
            else if (state == 1 && realport == 1) {
                state = 2;
                setupElementIcon();
            }
            else if (state == 2 && realport == 0) {
                state = 1;
                setupElementIcon();
            }
            else if (state == 3 && realport == 0) {
                state = 0;
                setupElementIcon();
            }
        }

        // simple crossings, decoder 2
        else if (classid == siciSl1 || classid == siciSl3 ||
                classid == siciSr1 || classid == siciSr3) {
            if (state == 1 && realport == 1) {
                state = 2;
                setupElementIcon();
            }
            else if (state == 2 && realport == 0) {
                state = 1;
                setupElementIcon();
            }
        }

        //qWarning("A1, State: %d, RealPort: %d\n", state, realport);

    }
    else if (addr == (unsigned int)address2) {
        realport = port ^ xchangeport2;
        if (protocol2 == GaMessage::proDCC)
            realport = realport == 0 ? 1 : 0;

        if (classid == siciHss1 || classid == siciHss3) {
            // Subtype 1 + 7
            if (iSoldSubType == 1 || iSoldSubType == 7) {
                // TODO: Check, realport should be compared to 0!
                if (realport == 1 && state != 3) {
                    state = 3;
                    setupElementIcon();
                }
            }
            // Subtype 5
            else {
                if (realport == 0 && state != 2) {
                    state = 2;
                    setupElementIcon();
                }
                else if (realport == 1 && state != 3) {
                    state = 3;
                    setupElementIcon();
                }
            }
        }
        // Subtype 6
        else if (classid == siciHs1 || classid == siciHs3 ||
                classid == siciVs1 || classid == siciVs3) {
            if (state != 2 && realport == 0) {
                state = 2;
                setupElementIcon();
            }
        }

        // Crossing Subtype 1, decoder 2
        else if (classid == siciDl1 || classid == siciDr1) {
            if (state == 0 && realport == 1) {
                state = 1;
                setupElementIcon();
            }
            else if (state == 1 && realport == 0) {
                state = 0;
                setupElementIcon();
            }
            else if (state == 2 && realport == 0) {
                state = 3;
                setupElementIcon();
            }
            else if (state == 3 && realport == 1) {
                state = 2;
                setupElementIcon();
            }
        }

        // Tree way turnout, decoder 2
        else if (classid == siciTw1 || classid == siciTw3) {
            if (state != 2 && realport == 1) {
                state = 2;
                setupElementIcon();
            }
        }

        // simple crossings, decoder 1
        else if (classid == siciSl1 || classid == siciSl3 ||
                classid == siciSr1 || classid == siciSr3) {
            if (state == 0 && realport == 1) {
                state = 1;
                setupElementIcon();
            }
            else if (state == 1 && realport == 0) {
                state = 0;
                setupElementIcon();
            }
        }
        //qWarning("A2, State: %d, RealPort: %d\n", state, realport);
    }
    else
        return;
}

/**
 * this is the reverse case of "sendSrcpState()"
 */
void element::processInfoPortMessage(unsigned int bus,
        unsigned int addr, unsigned int port, unsigned int value)
{
    // ignore port reset messages
    if (value == 0)
        return;

    if (!switchable)
        return;

    if (blinkcounter != 0)
        return;

    if (classid == siciEnk && iSoldSubType != -1)
        return;

    if ((bus == (unsigned int)bus1 &&
                addr == (unsigned int)address1) ||
            (bus == (unsigned int)bus2 &&
             addr == (unsigned int)address2)) {

        if (address2 != -1) {
            //qWarning("InfoPass, Bus: %d, Addr: %d, Port: %d, Value: %d",
            //    bus, addr, port, value);
            switch2AddressItem(addr, port);
            return;
        }

        int realstate = state;
        if (realstate == 2) //Hp0-Hp2-Type (iSoldSubType == 6)
            realstate = 1;

        bool isDCC = (protocol1 == GaMessage::proDCC);
        if (isDCC)
            realstate = !realstate;

        /*invert direction if connectors are exchanged*/
        realstate = realstate ^ xchangeport1;

        /* modulo 2 if there are more than 2 ports per address*/
        if ((port % 2) != (unsigned int)realstate) {
            realstate = port % 2;

            /*again invert direction if connectors are exchanged*/
            realstate = realstate ^ xchangeport1;

            if (isDCC)
                realstate = !realstate;

            //Hp0-Hp2-Type (iSoldSubType == 6)
            if (iSoldSubType == 6 && realstate == 1)
                realstate = 2;

            state = realstate;
            setupElementIcon();
            // TODO: show warning message when element is locked
        }
    }
}

/**
 * toggle through element direction states
 */
void element::toggle()
{
    // toggles direction for 3-state-solenoids
    // 0 -> 1 -> 2 -> 0
    if (classid == siciTw1 ||classid == siciTw3 ||
        classid == siciSl1 || classid == siciSl3 ||
        classid == siciSr1 || classid == siciSr3) {
        if (state < 2)
            switchToDir(state + 1);
        else
            switchToDir(0);
    }

    // 0: Hp0, 1: Hp1, 2: Hp2 (resp. Vr...)
    // SubType 0: 0 -> 1 -> 0
    // SubType 4: 0 -> 1 -> 2 -> 0
    // SubType 6: 0 -> 2 -> 0
    else if (classid == siciHs1 || classid == siciHs3 ||
            classid == siciVs1 || classid == siciVs3) {
        switch (state) {
            case 0:
                if (iSoldSubType == 6)
                    switchToDir(2);
                else
                    switchToDir(1);
                break;
            case 1:
                if (iSoldSubType == 0)
                    switchToDir(0);
                else
                    switchToDir(2);
                break;
            case 2:
                switchToDir(0);
                break;
        }
    }

    // 0: Hp0, 1: Hp1, 2: Hp2, 3: Sh1
    // SubType 1: 0 -> 1 -> 3 -> 0
    // SubType 5: 0 -> 1 -> 2 -> 3 -> 0
    // SubType 7: 0 -> 2 -> 3 -> 0
    else if (classid == siciHss1 || classid == siciHss3) {
        switch (state) {
            case 0:
                if (iSoldSubType == 7)
                    switchToDir(2);
                else
                    switchToDir(1);
                break;
            case 1:
                if (iSoldSubType == 1)
                    switchToDir(3);
                else
                    switchToDir(2);
                break;
            case 2:
                switchToDir(3);
                break;
            case 3:
                switchToDir(0);
                break;
        }
    }

    // toggles cyclic for 4-state-solenoids
    // 0 -> 1 -> 2 -> 3 -> 0
    else if ((classid == siciDl1 || classid == siciDr1)
             && iSoldSubType == 1) {
        if (state < 3)
            switchToDir(state + 1);
        else
            switchToDir(0);
    }

    // toggles cyclic for 2-state-solenoids
    else
        switchToDir(state == 1 ? 0 : 1);
}

/**
 * get conditions for context menu in normal visual mode and route edit
 * mode
 */
bool element::ctxCanSwitch()
{
    return (!isLocked()) &&
        (!isOccupied() || simplega || signal) &&
        (visualMode == kvmNormal ||
         (visualMode == kvmEditRoute && ksmNormal == selectionMode));
}

/**
 * paint element icon
 */
void element::setupElementIcon()
{
    // empty symbol
    if (classid == siciTxt) {
        QRect br;

        if (iSoldInvert == 1)
            background.fill(QColor(Qt::darkGray));

        // paint text label
        QPainter p(&background);

        // text and white rectangle for background
        if (sSoldText != "-1" && !sSoldText.isEmpty()) {
            QFont f(QApplication::font());
            f.setPointSize(QApplication::font().pointSize() - 1);
            p.setFont(f);
            QFontMetrics fm(f);
            br = fm.boundingRect(sSoldText);
            br.setWidth(br.width() + 4);
            br.setHeight(br.height() - 2);
            br.moveLeft((width() - br.width())/2);
            br.moveTop((height() - br.height())/2);
            p.fillRect(br, QBrush(Qt::white));
            p.drawText(rect(), Qt::AlignCenter | Qt::TextSingleLine |
                    Qt::TextDontClip, sSoldText);
        }
        // no text available -> show white rectangle only
        else {
            br = QRect(13, 11, 30, 13);
            p.fillRect(br, QBrush(Qt::white));
        }
    }

    // direction arrows
    else if (classid == siciTdr || classid == siciTdb) {
        QPainter p(&background);
        p.setRenderHint(QPainter::Antialiasing);

        int w = background.width();
        int h = background.height();

        // paint arrows
        /**
         * isri2  rotated  rightarrow  leftarrow
         * -------------------------------------
         *  0       0         1            0
         *  0       1         0            1
         *  1       0         1            1
         *  1       1         1            1
         * -------------------------------------
         **/

        p.setBrush(QBrush(Qt::black));

        // paint left arrow
        if (classid == siciTdb) {
            QPolygon leftarrow = QPolygon(4);
            leftarrow.putPoints(0, 4, w * 3 /4, h / 2,
                    w - 1, 1, w / 2 - 6, h / 2, w - 1, h - 1);
            p.drawPolygon(leftarrow);
        }
        // paint right arrow
        QPolygon rightarrow = QPolygon(4);
        rightarrow.putPoints(0, 4, w / 4, h / 2, 1, 1,
                w / 2 + 6, h / 2, 1, h - 1);
        p.drawPolygon(rightarrow);

        // paint track
        p.fillRect(0, h / 2 - 3, w, 7, QBrush(Qt::black));

        // paint track lights
        if (trackindicatoroff) {
            for (int i = 0; i < 7; ++i)
                p.fillRect(4 + 7 * i, h / 2 - 2, 5, 5,
                        QBrush(Qt::lightGray));
        }
        else {
            QColor c;
            if (occupied)
                c = QColor(Qt::red);
            else {
                if (routed)
                    c = QColor(255, 225, 0);
                else
                    c = QColor(Qt::darkGray);
            }
            p.setPen(QPen(c, 3, Qt::SolidLine, Qt::RoundCap, Qt::MiterJoin));
            p.drawLine(w / 3, h / 2, 2 * w / 3, h / 2);
            p.setPen(QPen(Qt::black));
        }

        // paint text label
        if (sSoldText != "-1" && !sSoldText.isEmpty()) {
            QFont f(QApplication::font());
            f.setPointSize(QApplication::font().pointSize() - 3);
            p.setFont(f);
            QFontMetrics fm(f);
            QRect br = fm.boundingRect(sSoldText);
            br.setWidth(br.width() + 4);
            br.setHeight(br.height() + 2);

            br.moveTopLeft(QPoint(w / 2 - br.width() / 2, 2));

            p.fillRect(br, QBrush(Qt::white));
            p.drawText(br, Qt::AlignCenter | Qt::TextSingleLine |
                    Qt::TextDontClip, sSoldText);
        }
    }

    // left direction arrow
    else if (classid == siciTdl) {
        QPainter p(&background);
        p.setRenderHint(QPainter::Antialiasing);

        int w = background.width();
        int h = background.height();

        // paint arrows
        p.setBrush(QBrush(Qt::black));
        QPolygon leftarrow = QPolygon(4);
        leftarrow.putPoints(0, 4, w * 3 /4, h / 2,
                w - 1, 1, w / 2 - 6, h / 2, w - 1, h - 1);
        p.drawPolygon(leftarrow);

        // paint track
        p.fillRect(0, h / 2 - 3, w, 7, QBrush(Qt::black));

        // paint track lights
        if (trackindicatoroff) {
            for (int i = 0; i < 7; ++i)
                p.fillRect(4 + 7 * i, h / 2 - 2, 5, 5,
                        QBrush(Qt::lightGray));
        }
        else {
            QColor c;
            if (occupied)
                c = QColor(Qt::red);
            else {
                if (routed)
                    c = QColor(255, 225, 0);
                else
                    c = QColor(Qt::darkGray);
            }
            p.setPen(QPen(c, 3, Qt::SolidLine, Qt::RoundCap, Qt::MiterJoin));
            p.drawLine(w / 3, h / 2, 2 * w / 3, h / 2);
            p.setPen(QPen(Qt::black));
        }

        // paint text label
        if (sSoldText != "-1" && !sSoldText.isEmpty()) {
            QFont f(QApplication::font());
            f.setPointSize(QApplication::font().pointSize() - 3);
            p.setFont(f);
            QFontMetrics fm(f);
            QRect br = fm.boundingRect(sSoldText);
            br.setWidth(br.width() + 4);
            br.setHeight(br.height() + 2);

            br.moveTopLeft(QPoint(w / 2 - br.width() / 2, 2));

            p.fillRect(br, QBrush(Qt::white));
            p.drawText(br, Qt::AlignCenter | Qt::TextSingleLine |
                    Qt::TextDontClip, sSoldText);
        }
    }

    // address
    else if (classid == siciAdr) {
        QPainter p(&background);

        int w = background.width();
        int h = background.height();

        // paint track
        p.fillRect(0, h / 2 - 3, 2, 7, QBrush(Qt::black));
        p.fillRect(w - 2, h / 2 - 3, 2, 7, QBrush(Qt::black));

        // paint address field
        p.setPen(QPen(Qt::black));
        p.setBrush(QColor(Qt::black));

        p.drawRect(2, 8, w - 4, h - 16);

        // paint address value
        p.setPen(QPen(Qt::red));

        QFont f(QApplication::font());
        f.setPointSize(QApplication::font().pointSize() + 1);
        f.setWeight(QFont::DemiBold);
        f.setStretch(90);
        p.setFont(f);
        sSoldText.asprintf("%06d", editsAddress);
        QFontMetrics fm(f);
        QRect br = fm.boundingRect(sSoldText);
        br.moveTopLeft(QPoint(w / 2 - br.width() / 2,
                    h / 2 - br.height() / 2));
	p.drawText(br, Qt::AlignCenter | Qt::TextSingleLine | Qt::TextDontClip,
                sSoldText);
    }

    // level crossing
    else if (classid == siciBue) {
        QPainter p(&background);

        int w = background.width();
        int h = background.height();

        // paint road
        p.fillRect(w / 2 - 7, 0, 15, h, QBrush(Qt::darkGray));

        // paint track
        p.fillRect(0, h / 2 - 3, w, 7, QBrush(Qt::black));

        // paint track button
        p.drawPixmap(w / 2  - 4, h / 2 - 3, QPixmap(button_yellow_xpm));

        // paint lights
        p.setBrush(Qt::darkGray);
        p.drawEllipse(6, 4, 5, 5);
        p.drawEllipse(w - 12, 4, 5, 5);
        //p.drawRect(background.width()/2 - 3, 4, 6, 6);
    }

    // relay
    else if (classid == siciRel) {
        QPainter p(&background);

        int w = background.width();
        int h = background.height();

        // paint symbol
        p.drawLine(1, h / 8, w / 6 + 3, h / 8);
        p.drawLine(w / 6 + 3, h / 8, w / 6 + 3, h / 4);

        p.drawLine(1, h * 7 / 8, w / 6 + 3, h * 7 / 8);
        p.drawLine(w / 6 + 3, h * 7 / 8, w / 6 + 3, h * 3 / 4);

        p.drawRect(w / 6, h / 4 + 1, w / 8, h / 2);
        p.drawLine(w / 6, h / 4 + 1, w * 7 / 24 - 1, h * 3 / 4 - 1);

        p.setPen(QPen(Qt::DotLine));
        p.drawLine(w / 4 + 3, h / 2, w * 3 / 4, h / 2);
        p.setPen(QPen(Qt::SolidLine));

        // paint text label
        if (sSoldText != "-1" && !sSoldText.isEmpty()) {
            QFont f(QApplication::font());
            f.setPointSize(QApplication::font().pointSize() - 3);
            p.setFont(f);
            QFontMetrics fm(f);
            QString s;

            if (pref.addresslabeling)
                s.setNum(address1);
            else
                s = sSoldText;

            QRect br = fm.boundingRect(s);
            br.setWidth(br.width() + 4);
            br.setHeight(br.height() - 2);

            br.moveBottomRight(QPoint(w - 3, h - 3));

            p.fillRect(br, QBrush(Qt::white));
            p.drawText(br, Qt::AlignCenter | Qt::TextSingleLine |
                    Qt::TextDontClip, s);
        }

        // paint signalization (black or yellow filled circle)
        if (state == 1)
            p.setBrush(QColor(Qt::yellow));
        else
            p.setBrush(QColor(Qt::black));

        p.drawEllipse(w * 3 / 4 - 3, h / 2 - 5, 10, 10);
    }

    // motor
    else if (classid == siciMdc) {
        QPainter p(&background);

        int w = background.width();
        int h = background.height();

        // paint signalization (black or yellow filled circle)
        p.setPen(QPen(Qt::black, 2));

        if (state == 1)
            p.setBrush(QColor(Qt::yellow));

        p.drawEllipse(w / 2 - 9, h - 18 - 1, 16, 16);

        // paint symbol
        // red connector left
        p.setPen(Qt::red);
        p.drawLine(3, h - 9 - 2, w / 2 - 10, h - 9 - 2);
        p.drawLine(w / 2 - 11, h - 9 - 3, w / 2 - 10, h - 9 - 3);
        p.drawLine(w / 2 - 11, h - 9 - 1, w / 2 - 10, h - 9 - 1);
        // plus sign
        p.drawLine(3, h - 9 - 6, 7, h - 9 - 6);
        p.drawLine(5, h - 9 - 8, 5, h - 9 - 4);

        // blue connector right
        p.setPen(Qt::blue);
        p.drawLine(w - 5, h - 9 - 2, w / 2 + 8, h - 9 - 2);
        p.drawLine(w / 2 + 9, h - 9 - 3, w / 2 + 8, h - 9 - 3);
        p.drawLine(w / 2 + 9, h - 9 - 1, w / 2 + 8, h - 9 - 1);
        // minus sign
        p.drawLine(w - 5, h - 9 - 6, w - 9, h - 9 - 6);

        // motor labels
        p.setPen(Qt::black);
        p.drawLine(w / 2 - 3, h - 9, w / 2 + 1, h - 9);
        p.drawPoint(w / 2 - 3, h - 7);
        p.drawPoint(w / 2 - 1, h - 7);
        p.drawPoint(w / 2 + 1, h - 7);

        QPolygon motor = QPolygon(5);
        motor.putPoints(0, 5, 0, 3, 0, 0, 2, 2, 4, 0, 4, 3);
        motor.translate(w / 2 - 3, h - 14);
        p.drawPolyline(motor);

        // paint text label
        if (sSoldText != "-1" && !sSoldText.isEmpty()) {
            QFont f(QApplication::font());
            f.setPointSize(QApplication::font().pointSize() - 3);
            p.setFont(f);
            QFontMetrics fm(f);
            QString s;

            if (pref.addresslabeling)
                s.setNum(address2);
            else
                s = sSoldText;

            QRect br = fm.boundingRect(s);
            br.setWidth(br.width() + 4);
            br.setHeight(br.height() - 2);

            br.moveTopLeft(QPoint(w / 2 - br.width() / 2, 2));

            p.fillRect(br, QBrush(Qt::white));
            p.drawText(br, Qt::AlignCenter | Qt::TextSingleLine |
                    Qt::TextDontClip, s);
        }
    }

    // decoupler
    else if (classid == siciEnk) {
        QPainter p(&background);

        int w = background.width();
        int h = background.height();

        // paint track
        p.fillRect(0, h / 2 - 3, w, 7, QBrush(Qt::black));

        // paint symbol
        p.fillRect(21, 10, 14, 3, QBrush(Qt::black));

        // paint track lights
        if (trackindicatoroff) {
            for (int i = 0; i < 7; ++i)
                p.fillRect(4 + 7 * i, h / 2 - 2, 5, 5,
                        QBrush(Qt::lightGray));
        }
        else {
            QColor c;
            if (occupied)
                c = QColor(Qt::red);
            else {
                if (routed)
                    c = QColor(255, 225, 0);
                else
                    c = QColor(Qt::darkGray);
            }
            p.setPen(QPen(c, 3, Qt::SolidLine, Qt::RoundCap, Qt::MiterJoin));
            p.drawLine(w / 3, h / 2, 2 * w / 3, h / 2);
            p.setPen(Qt::black);
        }

        // paint text label
        if (sSoldText != "-1" && !sSoldText.isEmpty()) {
            QFont f(QApplication::font());
            f.setPointSize(QApplication::font().pointSize() - 3);
            p.setFont(f);
            QFontMetrics fm(f);
            QString s;

            if (pref.addresslabeling)
                s.setNum(address1);
            else
                s = sSoldText;

            QRect br = fm.boundingRect(s);
            br.setWidth(br.width() + 4);
            br.setHeight(br.height() - 2);

            br.moveBottomRight(QPoint(w / 2 + br.width() / 2, h - 3));

            p.fillRect(br, QBrush(Qt::white));
            p.drawText(br, Qt::AlignCenter | Qt::TextSingleLine |
                    Qt::TextDontClip, s);
        }

        // paint signalization (white triangle)
        if (state == 1) {
            p.setPen(Qt::white);
            p.setBrush(QColor(Qt::white));
            QPolygon triangle = QPolygon(4);
            triangle.putPoints(0, 4, 21, 9, 27, 3, 28, 3, 34, 9);
            p.drawPolygon(triangle);
        }
    }

    // blind element
    else if (classid == siciBld) {
        QPainter p(&background);

        int w = background.width();
        int h = background.height();

        // paint track
        p.fillRect(0, h / 2 - 3, w, 7, QBrush(Qt::black));

        // paint track lights
        if (trackindicatoroff) {
            for (int i = 0; i < 7; ++i)
                p.fillRect(4 + 7 * i, h / 2 - 2, 5, 5,
                        QBrush(Qt::lightGray));
        }
        else {
            QColor c;
            if (occupied)
                c = QColor(Qt::red);
            else {
                if (routed)
                    c = QColor(255, 225, 0);
                else
                    c = QColor(Qt::darkGray);
            }
            p.setPen(QPen(c, 3, Qt::SolidLine, Qt::RoundCap, Qt::MiterJoin));
            p.drawLine(w / 3, h / 2, 2 * w / 3, h / 2);
            p.setPen(Qt::black);
        }

        // paint text label
        if (sSoldText != "-1" && !sSoldText.isEmpty()) {
            QFont f(QApplication::font());
            f.setPointSize(QApplication::font().pointSize() - 3);
            p.setFont(f);
            QFontMetrics fm(f);
            QString s;

            if (pref.addresslabeling)
                s.setNum(address1);
            else
                s = sSoldText;

            QRect br = fm.boundingRect(s);
            br.setWidth(br.width() + 4);
            br.setHeight(br.height() - 2);

            br.moveBottomRight(QPoint(w / 2 + br.width()/2, h - 3));

            p.fillRect(br, QBrush(Qt::white));
            p.drawText(br, Qt::AlignCenter | Qt::TextSingleLine |
                    Qt::TextDontClip, s);
        }

        // paint signalization
        if (state == 1)
            p.setPen(Qt::green);
        else
            p.setPen(Qt::red);

        p.drawLine(19, 12, 19 + 17, 12);
        p.drawLine(18, 13, 18 + 19, 13);
        p.drawLine(18, 21, 18 + 19, 21);
        p.drawLine(19, 22, 19 + 17, 22);
    }

    // right shunt wait signal
    else if (classid == siciWs1) {
        QPainter p(&background);

        int w = background.width();
        int h = background.height();

        // paint track
        p.fillRect(0, h / 2 - 3, w, 7, QBrush(Qt::black));

        // paint track lights
        if (trackindicatoroff) {
            for (int i = 0; i < 7; ++i)
                p.fillRect(4 + 7 * i, h / 2 - 2, 5, 5, QBrush(Qt::lightGray));
        }
        else {
            QColor c;
            if (occupied)
                c = QColor(Qt::red);
            else {
                if (routed)
                    c = QColor(255, 225, 0);
                else
                    c = QColor(Qt::darkGray);
            }
            p.fillRect(w / 3 + 1, h / 2 - 1, w / 3, 3, QBrush(c));
            p.setPen(QPen(Qt::black));
        }

        // paint track button
        p.drawPixmap(5 * w / 6 - 4 , h / 2 - 3,
                QPixmap(button_gray_xpm));

        // paint lock light
        if (lockCounter == 0)
            p.setBrush(Qt::darkGray);
        else
            p.setBrush(QColor(255, 225, 0));

        p.drawEllipse(w / 2 - 5, h - 10, 4, 4);

        // paint signal icon
        p.drawPixmap(w - 16 , h / 2 + 5, QPixmap(signal_w_xpm));
        p.fillRect(w - 25, 25, 2, 5, QBrush(Qt::black));
        p.drawLine(w - 15, 27, w - 23, 27);

        // paint signal light
        if (state == 1) {
            p.setPen(QPen(Qt::white));
            p.setBrush(Qt::white);
        }
        else
            p.setBrush(Qt::darkGray);

        p.drawRect(w - 18, h / 2 + 6, 3, 3);
        p.drawRect(w - 8, h / 2 + 13, 3, 3);
        p.setPen(QPen(Qt::black));

        // paint text label
        if (sSoldText != "-1" && !sSoldText.isEmpty()) {
            QFont f(QApplication::font());
            f.setPointSize(QApplication::font().pointSize() - 3);
            p.setFont(f);
            QFontMetrics fm(f);
            QString s;

            if (pref.addresslabeling)
                s.setNum(address1);
            else
                s = sSoldText;

            QRect br = fm.boundingRect(s);
            br.setWidth(br.width() + 4);
            br.setHeight(br.height() - 2);

            br.moveTopLeft(QPoint(w / 2 - br.width()/2, 2));

            p.fillRect(br, QBrush(Qt::white));
            p.drawText(br, Qt::AlignCenter | Qt::TextSingleLine |
                    Qt::TextDontClip, s);
        }
    }

    // left shunt wait signal
    else if (classid == siciWs3) {
        QPainter p(&background);

        int w = background.width();
        int h = background.height();

        // paint track
        p.fillRect(0, h / 2 - 3, w, 7, QBrush(Qt::black));

        // paint track lights
        if (trackindicatoroff) {
            for (int i = 0; i < 7; ++i)
                p.fillRect(4 + 7 * i, h / 2 - 2, 5, 5, QBrush(Qt::lightGray));
        }
        else {
            QColor c;
            if (occupied)
                c = QColor(Qt::red);
            else {
                if (routed)
                    c = QColor(255, 225, 0);
                else
                    c = QColor(Qt::darkGray);
            }
            p.fillRect(w / 3 + 1, h / 2 - 1, w / 3, 3, QBrush(c));
            p.setPen(QPen(Qt::black));
        }

        // paint track button
        p.drawPixmap(w / 6  - 4, h / 2 - 3,
                QPixmap(button_gray_xpm));

        // paint lock light
        if (lockCounter == 0)
            p.setBrush(Qt::darkGray);
        else
            p.setBrush(QColor(255, 225, 0));

        p.drawEllipse(w / 2 - 1, 5, 4, 4);

        // paint signal icon
        p.drawPixmap(6, 2, QPixmap(signal_wr_xpm));
        p.fillRect(22, 5, 2, 5, QBrush(Qt::black));
        p.drawLine(13, 7, 21, 7);

        // paint signal light
        if (state == 1) {
            p.setPen(QPen(Qt::white));
            p.setBrush(Qt::white);
        }
        else
            p.setBrush(Qt::darkGray);

        p.drawRect(4, 2, 3, 3);
        p.drawRect(14, 9, 3, 3);
        p.setPen(QPen(Qt::black));

        // paint text label
        if (sSoldText != "-1" && !sSoldText.isEmpty()) {
            QFont f(QApplication::font());
            f.setPointSize(QApplication::font().pointSize() - 3);
            p.setFont(f);
            QFontMetrics fm(f);
            QString s;

            if (pref.addresslabeling)
                s.setNum(address1);
            else
                s = sSoldText;

            QRect br = fm.boundingRect(s);
            br.setWidth(br.width() + 4);
            br.setHeight(br.height() - 2);

            br.moveBottomRight(QPoint(w / 2 + br.width()/2, h - 3));

            p.fillRect(br, QBrush(Qt::white));
            p.drawText(br, Qt::AlignCenter | Qt::TextSingleLine |
                    Qt::TextDontClip, s);
        }
    }

    // right signal HSS
    else if (classid == siciHss1) {
        QPainter p(&background);

        int w = background.width();
        int h = background.height();

        // paint track
        p.fillRect(0, h / 2 - 3, w, 7, QBrush(Qt::black));

        // paint track lights
        if (trackindicatoroff) {
            for (int i = 0; i < 7; ++i)
                p.fillRect(4 + 7 * i, h / 2 - 2, 5, 5, QBrush(Qt::lightGray));
        }
        else {
            QColor c;
            if (occupied)
                c = QColor(Qt::red);
            else {
                if (routed)
                    c = QColor(255, 225, 0);
                else
                    c = QColor(Qt::darkGray);
            }
            p.fillRect(w / 3 + 1, h / 2 - 1, w / 3, 3, QBrush(c));
            p.setPen(QPen(Qt::black));
        }

        // paint track button
        p.drawPixmap(5 * w / 6 - 4 , h / 2 - 3,
                QPixmap(button_red_xpm));
        p.drawPixmap(w / 6  - 4, h / 2 - 3,
                QPixmap(button_gray_xpm));

        // paint signal icon
        p.setBrush(Qt::black);
        p.drawEllipse(17, h - 11, LIGHTSIZE, LIGHTSIZE);
        p.fillRect(22, h - 11, 24, 7, QBrush(Qt::black));
        p.fillRect(12, 25, 2, 5, QBrush(Qt::black));
        p.drawLine(14, 27, 16, 27);

        /**
         *   state subtype bottom-left bottom-right top-left top-right
         * -------------------------------------------------------------
         *     0       0          y           y
         *     1       0                                 g        g
         *     0       6          y           y
         *     2       6          y                               g
         *     0       4          y           y
         *     1       4                                 g        g
         *     2       4          y                               g
         * -------------------------------------------------------------
         **/

        // fprintf(stderr, "dir: %d type: %d\n", state, iSoldSubType);

        // paint signal light
        // fix potential wrong direction value
        if (iSoldSubType == 6 && state == DIR_HP1)
            state = DIR_HP2;

        switch (state) {
            case DIR_HP0:              // HP0 => 0
                p.setBrush(Qt::red);
                // bottom
                p.drawEllipse(w - 22, h - 11, LIGHTSIZE, LIGHTSIZE);
                // top
                p.setBrush(Qt::black);
                p.drawEllipse(w - 12, h - 11, LIGHTSIZE, LIGHTSIZE);
                break;
            case DIR_HP2:              // HP2 => 2
                if (pref.hp2) {
                    p.setBrush(Qt::yellow);
                    // top
                    p.drawEllipse(w - 12, h - 11, LIGHTSIZE, LIGHTSIZE);
                    // bottom
                    p.setBrush(Qt::black);
                    p.drawEllipse(w - 22, h - 11, LIGHTSIZE, LIGHTSIZE);
                    break;
                }
                // fall through
            case DIR_HP1:              // HP1 => 1
                p.setBrush(Qt::green);
                // top
                p.drawEllipse(w - 12, h - 11, LIGHTSIZE, LIGHTSIZE);
                // bottom
                p.setBrush(Qt::black);
                p.drawEllipse(w - 22, h - 11, LIGHTSIZE, LIGHTSIZE);
                break;
            case DIR_SH1:              // SH1 => 3
                {
                p.setPen(Qt::yellow);
                // paint only shunt light
                int startx = 19;
                p.drawLine(startx, h - 10, startx + 4, h - 6);
                p.drawLine(startx + 1, h - 10, startx + 5, h - 6);
                p.setPen(Qt::black);
                // top
                p.drawEllipse(w - 12, h - 11, LIGHTSIZE, LIGHTSIZE);
                break;
                }
        }

        // paint lock light
        if (lockCounter == 0)
            p.setBrush(Qt::darkGray);
        else
            p.setBrush(QColor(255, 225, 0));

        p.drawEllipse(5, h - 10, 4, 4);

        // paint FfM
        if (ffm) {
            p.setBrush(ffmactive ? Qt::yellow : Qt::darkGray);
            p.drawRect(w - 11, 4, 5, 5);
        }

        // paint text label
        if (sSoldText != "-1" && !sSoldText.isEmpty()) {
            QFont f(QApplication::font());
            f.setPointSize(QApplication::font().pointSize() - 3);
            p.setFont(f);
            QFontMetrics fm(f);
            QString s;

            if (pref.addresslabeling)
                s.setNum(address1);
            else
                s = sSoldText;

            QRect br = fm.boundingRect(s);
            br.setWidth(br.width() + 4);
            br.setHeight(br.height() - 2);

            br.moveTopLeft(QPoint(w / 2 - br.width()/2, 2));

            p.fillRect(br, QBrush(Qt::white));
            p.drawText(br, Qt::AlignCenter | Qt::TextSingleLine |
                    Qt::TextDontClip, s);
        }
    }

    // left signal HSS
    else if (classid == siciHss3) {
        QPainter p(&background);

        int w = background.width();
        int h = background.height();

        // paint track
        p.fillRect(0, h / 2 - 3, w, 7, QBrush(Qt::black));

        // paint track lights
        if (trackindicatoroff) {
            for (int i = 0; i < 7; ++i)
                p.fillRect(4 + 7 * i, h / 2 - 2, 5, 5, QBrush(Qt::lightGray));
        }
        else {
            QColor c;
            if (occupied)
                c = QColor(Qt::red);
            else {
                if (routed)
                    c = QColor(255, 225, 0);
                else
                    c = QColor(Qt::darkGray);
            }
            p.fillRect(w / 3 + 1, h / 2 - 1, w / 3, 3, QBrush(c));
            p.setPen(QPen(Qt::black));
        }

        // paint track button
        p.drawPixmap(w / 6  - 4, h / 2 - 3,
                QPixmap(button_red_xpm));
        p.drawPixmap(5 * w / 6  - 4, h / 2 - 3,
                QPixmap(button_gray_xpm));

        // paint signal icon
        p.setBrush(Qt::black);
        p.drawEllipse(31, 4, LIGHTSIZE, LIGHTSIZE);
        p.fillRect(9, 4, 24, 7, QBrush(Qt::black));
        p.fillRect(41, 5, 2, 5, QBrush(Qt::black));
        p.drawLine(38, 7, 40, 7);

        /**
         *   state  subtype bottom-left bottom-right top-left top-right
         * -------------------------------------------------------------
         *     0       0          y           y
         *     1       0                                 g        g
         *     0       6          y           y
         *     2       6          y                               g
         *     0       4          y           y
         *     1       4                                 g        g
         *     2       4          y                               g
         * -------------------------------------------------------------
         **/

        // fprintf(stderr, "dir: %d type: %d\n", state, iSoldSubType);

        // paint signal light
        // fix potential wrong direction value
        if (iSoldSubType == 6 && state == DIR_HP1)
            state = DIR_HP2;

        switch (state) {
            case DIR_HP0:              // HP0 => 0
                p.setBrush(Qt::red);
                // bottom
                p.drawEllipse(14, 4, LIGHTSIZE, LIGHTSIZE);
                // top
                p.setBrush(Qt::black);
                p.drawEllipse(4, 4, LIGHTSIZE, LIGHTSIZE);
                break;
            case DIR_HP2:              // HP2 => 2
                if (pref.hp2) {
                    p.setBrush(Qt::yellow);
                    // top
                    p.drawEllipse(4, 4, LIGHTSIZE, LIGHTSIZE);
                    // bottom
                    p.setBrush(Qt::black);
                    p.drawEllipse(14, 4, LIGHTSIZE, LIGHTSIZE);
                    break;
                }
                // fall through
            case DIR_HP1:              // HP1 => 1
                p.setBrush(Qt::green);
                // top
                p.drawEllipse(4, 4, LIGHTSIZE, LIGHTSIZE);
                // bottom
                p.setBrush(Qt::black);
                p.drawEllipse(14, 4, LIGHTSIZE, LIGHTSIZE);
                break;
            case DIR_SH1:              // SH1 => 3
                {
                    p.setPen(Qt::yellow);
                    // paint only shunt light
                    int startx = w - 22;
                    p.drawLine(startx, 9, startx - 4, 5);
                    p.drawLine(startx + 1, 9, startx - 3, 5);
                    p.setPen(Qt::black);
                    // top
                    p.drawEllipse(4, 4, LIGHTSIZE, LIGHTSIZE);
                    break;
                }
        }

        // paint lock light
        if (lockCounter == 0)
            p.setBrush(Qt::darkGray);
        else
            p.setBrush(QColor(255, 225, 0));

        p.drawEllipse(w - 11, 5, 4, 4);

        // paint FfM
        if (ffm) {
            p.setBrush(ffmactive ? Qt::yellow : Qt::darkGray);
            p.drawRect(5, h - 10, 5, 5);
        }

        // paint text label
        if (sSoldText != "-1" && !sSoldText.isEmpty()) {
            QFont f(QApplication::font());
            f.setPointSize(QApplication::font().pointSize() - 3);
            p.setFont(f);
            QFontMetrics fm(f);
            QString s;

            if (pref.addresslabeling)
                s.setNum(address1);
            else
                s = sSoldText;

            QRect br = fm.boundingRect(s);
            br.setWidth(br.width() + 4);
            br.setHeight(br.height() - 2);

            br.moveBottomRight(QPoint(w / 2 + br.width()/2, h - 3));

            p.fillRect(br, QBrush(Qt::white));
            p.drawText(br, Qt::AlignCenter | Qt::TextSingleLine |
                    Qt::TextDontClip, s);
        }
    }

    // signal right HS
    else if (classid == siciHs1) {
        QPainter p(&background);

        int w = background.width();
        int h = background.height();

        // paint track
        p.fillRect(0, h / 2 - 3, w, 7, QBrush(Qt::black));

        // paint track lights
        if (trackindicatoroff) {
            for (int i = 0; i < 7; ++i)
                p.fillRect(4 + 7 * i, h / 2 - 2, 5, 5, QBrush(Qt::lightGray));
        }
        else {
            QColor c;
            if (occupied)
                c = QColor(Qt::red);
            else {
                if (routed)
                    c = QColor(255, 225, 0);
                else
                    c = QColor(Qt::darkGray);
            }
            p.fillRect(w / 3 + 1, h / 2 - 1, w / 3, 3, QBrush(c));
            p.setPen(QPen(Qt::black));
        }

        // paint track button
        p.drawPixmap(5 * w / 6 - 4 , h / 2 - 3,
                QPixmap(button_red_xpm));

        // paint signal icon
        p.fillRect(w - 20, h - 11, 10, 7, QBrush(Qt::black));
        p.fillRect(w - 27, 25, 2, 5, QBrush(Qt::black));
        p.drawLine(w - 23, 27, w - 25, 27);

        /**
         *   state subtype bottom-left bottom-right top-left top-right
         * -------------------------------------------------------------
         *     0       0          y           y
         *     1       0                                 g        g
         *     0       6          y           y
         *     2       6          y                               g
         *     0       4          y           y
         *     1       4                                 g        g
         *     2       4          y                               g
         * -------------------------------------------------------------
         **/

        // fprintf(stderr, "dir: %d type: %d\n", state, iSoldSubType);

        // paint signal light
        // fix potential wrong direction value
        if (iSoldSubType == 6 && state == DIR_HP1)
            state = DIR_HP2;

        switch (state) {
            case DIR_HP0:              // HP0 => 0
                p.setBrush(Qt::red);
                // bottom
                p.drawEllipse(w - 22, h - 11, LIGHTSIZE, LIGHTSIZE);
                // top
                p.setBrush(Qt::black);
                p.drawEllipse(w - 12, h - 11, LIGHTSIZE, LIGHTSIZE);
                break;
            case DIR_HP2:              // HP2 => 2
                if (pref.hp2) {
                    p.setBrush(Qt::yellow);
                    // top
                    p.drawEllipse(w - 12, h - 11, LIGHTSIZE, LIGHTSIZE);
                    // bottom
                    p.setBrush(Qt::black);
                    p.drawEllipse(w - 22, h - 11, LIGHTSIZE, LIGHTSIZE);
                    break;
                }
                // fall through
            case DIR_HP1:              // HP1 => 1
                p.setBrush(Qt::green);
                // top
                p.drawEllipse(w - 12, h - 11, LIGHTSIZE, LIGHTSIZE);
                // bottom
                p.setBrush(Qt::black);
                p.drawEllipse(w - 22, h - 11, LIGHTSIZE, LIGHTSIZE);
                break;
        }

        // paint lock light
        if (lockCounter == 0)
            p.setBrush(Qt::darkGray);
        else
            p.setBrush(QColor(255, 225, 0));

        p.drawEllipse(w / 2 - 6, h - 10, 4, 4);

        // paint FfM
        if (ffm) {
            p.setBrush(ffmactive ? Qt::yellow : Qt::darkGray);
            p.drawRect(w - 11, 4, 5, 5);
        }

        // paint text label
        if (sSoldText != "-1" && !sSoldText.isEmpty()) {
            QFont f(QApplication::font());
            f.setPointSize(QApplication::font().pointSize() - 3);
            p.setFont(f);
            QFontMetrics fm(f);
            QString s;

            if (pref.addresslabeling)
                s.setNum(address1);
            else
                s = sSoldText;

            QRect br = fm.boundingRect(s);
            br.setWidth(br.width() + 4);
            br.setHeight(br.height() - 2);

            br.moveTopLeft(QPoint(w / 2 - br.width()/2, 2));

            p.fillRect(br, QBrush(Qt::white));
            p.drawText(br, Qt::AlignCenter | Qt::TextSingleLine |
                    Qt::TextDontClip, s);
        }
    }

    // signal left HS
    else if (classid == siciHs3) {
        QPainter p(&background);

        int w = background.width();
        int h = background.height();

        // paint track
        p.fillRect(0, h / 2 - 3, w, 7, QBrush(Qt::black));

        // paint track lights
        if (trackindicatoroff) {
            for (int i = 0; i < 7; ++i)
                p.fillRect(4 + 7 * i, h / 2 - 2, 5, 5, QBrush(Qt::lightGray));
        }
        else {
            QColor c;
            if (occupied)
                c = QColor(Qt::red);
            else {
                if (routed)
                    c = QColor(255, 225, 0);
                else
                    c = QColor(Qt::darkGray);
            }
            p.fillRect(w / 3 + 1, h / 2 - 1, w / 3, 3, QBrush(c));
            p.setPen(QPen(Qt::black));
        }

        // paint track button
        p.drawPixmap(w / 6  - 4, h / 2 - 3,
                QPixmap(button_red_xpm));

        // paint signal icon
        p.fillRect(7, 4, 10, 7, QBrush(Qt::black));
        p.fillRect(24, 5, 2, 5, QBrush(Qt::black));
        p.drawLine(21, 7, 24, 7);

        /**
         *   state subtype bottom-left bottom-right top-left top-right
         * -------------------------------------------------------------
         *     0       0          y           y
         *     1       0                                 g        g
         *     0       6          y           y
         *     2       6          y                               g
         *     0       4          y           y
         *     1       4                                 g        g
         *     2       4          y                               g
         * -------------------------------------------------------------
         **/

        // paint signal light
        // fix potential wrong direction value
        if (iSoldSubType == 6 && state == DIR_HP1)
            state = DIR_HP2;

        switch (state) {
            case DIR_HP0:              // HP0 => 0
                p.setBrush(Qt::red);
                // bottom
                p.drawEllipse(14, 4, LIGHTSIZE, LIGHTSIZE);
                // top
                p.setBrush(Qt::black);
                p.drawEllipse(4, 4, LIGHTSIZE, LIGHTSIZE);
                break;
            case DIR_HP2:              // HP2 => 2
                if (pref.hp2) {
                    p.setBrush(Qt::yellow);
                    // top
                    p.drawEllipse(4, 4, LIGHTSIZE, LIGHTSIZE);
                    // bottom
                    p.setBrush(Qt::black);
                    p.drawEllipse(14, 4, LIGHTSIZE, LIGHTSIZE);
                    break;
                }
                // fall through
            case DIR_HP1:              // HP1 => 1
                p.setBrush(Qt::green);
                // top
                p.drawEllipse(4, 4, LIGHTSIZE, LIGHTSIZE);
                // bottom
                p.setBrush(Qt::black);
                p.drawEllipse(14, 4, LIGHTSIZE, LIGHTSIZE);
                break;
        }

        // paint lock light
        if (lockCounter == 0)
            p.setBrush(Qt::darkGray);
        else
            p.setBrush(QColor(255, 225, 0));

        p.drawEllipse(w / 2, 5, 4, 4);

        // paint FfM
        if (ffm) {
            p.setBrush(ffmactive ? Qt::yellow : Qt::darkGray);
            p.drawRect(5, h - 10, 5, 5);
        }

        // paint text label
        if (sSoldText != "-1" && !sSoldText.isEmpty()) {
            QFont f(QApplication::font());
            f.setPointSize(QApplication::font().pointSize() - 3);
            p.setFont(f);
            QFontMetrics fm(f);
            QString s;

            if (pref.addresslabeling)
                s.setNum(address1);
            else
                s = sSoldText;

            QRect br = fm.boundingRect(s);
            br.setWidth(br.width() + 4);
            br.setHeight(br.height() - 2);

            br.moveBottomRight(QPoint(w / 2 + br.width()/2, h - 3));

            p.fillRect(br, QBrush(Qt::white));
            p.drawText(br, Qt::AlignCenter | Qt::TextSingleLine |
                    Qt::TextDontClip, s);
        }
    }

    // right signal VS
    else if (classid == siciVs1) {
        QPainter p(&background);

        int w = background.width();
        int h = background.height();

        // paint track
        p.fillRect(0, h / 2 - 3, w, 7, QBrush(Qt::black));

        // paint signal icon
        QPolygon icon = QPolygon(6);
        icon.putPoints(0, 6, 0, 0, 0, 3, 3, 6, 12, 6, 12, 3, 9, 0);
        p.setBrush(Qt::black);

        icon.translate(w - 18, h - 11);
        p.drawPolygon(icon);
        p.fillRect(w - 23, 25, 2, 5, QBrush(Qt::black));
        p.drawLine(w - 18, 27, w - 21, 27);

        /**
         *   state subtype bottom-left bottom-right top-left top-right
         * -------------------------------------------------------------
         *     0       0          y           y
         *     1       0                                 g        g
         *     0       6          y           y
         *     2       6          y                               g
         *     0       4          y           y
         *     1       4                                 g        g
         *     2       4          y                               g
         * -------------------------------------------------------------
         **/

        // fprintf(stderr, "dir: %d type: %d\n", state, iSoldSubType);

        // paint signal light
        // fix potential wrong direction value
        if (iSoldSubType == 6 && state == DIR_HP1)
            state = DIR_HP2;

        switch (state) {
            case DIR_HP0:              // VS0 => 0
                // bottom left
                p.fillRect(w - 17, h - 10, 2, 2, QBrush(Qt::yellow));
                // bottom right
                p.fillRect(w - 15, h - 7, 2, 2, QBrush(Qt::yellow));
                break;
            case DIR_HP2:              // VS2 => 2
                if (pref.hp2) {
                    // bottom left
                    p.fillRect(w - 17, h - 10, 2, 2, QBrush(Qt::yellow));
                    //top right
                    p.fillRect(w - 8, h - 7, 2, 2, QBrush(Qt::green));
                    break;
                }
            case DIR_HP1:              // VS1 => 1
                //top left
                p.fillRect(w - 10, h - 10, 2, 2, QBrush(Qt::green));
                //top right
                p.fillRect(w - 8, h - 7, 2, 2, QBrush(Qt::green));
                break;
        }

        // paint track lights
        if (trackindicatoroff) {
            for (int i = 0; i < 7; ++i)
                p.fillRect(4 + 7 * i, h / 2 - 2, 5, 5, QBrush(Qt::lightGray));
        }
        else {
            QColor c;
            if (occupied)
                c = QColor(Qt::red);
            else {
                if (routed)
                    c = QColor(255, 225, 0);
                else
                    c = QColor(Qt::darkGray);
            }
            p.fillRect(w / 3 + 1, h / 2 - 1, w / 3, 3, QBrush(c));
            p.setPen(QPen(Qt::black));
        }

        // paint text label
        if (sSoldText != "-1" && !sSoldText.isEmpty()) {
            QFont f(QApplication::font());
            f.setPointSize(QApplication::font().pointSize() - 3);
            p.setFont(f);
            QFontMetrics fm(f);
            QString s;

            if (pref.addresslabeling)
                s.setNum(address1);
            else
                s = sSoldText;

            QRect br = fm.boundingRect(s);
            br.setWidth(br.width() + 4);
            br.setHeight(br.height() - 2);

            br.moveTopLeft(QPoint(w / 2 - br.width()/2, 2));

            p.fillRect(br, QBrush(Qt::white));
            p.drawText(br, Qt::AlignCenter | Qt::TextSingleLine |
                    Qt::TextDontClip, s);
        }
    }

    // left signal VS
    else if (classid == siciVs3) {
        QPainter p(&background);

        int w = background.width();
        int h = background.height();

        // paint track
        p.fillRect(0, h / 2 - 3, w, 7, QBrush(Qt::black));

        // paint signal icon
        QPolygon icon = QPolygon(6);
        icon.putPoints(0, 6, 0, 0, 0, 3, 3, 6, 12, 6, 12, 3, 9, 0);
        p.setBrush(Qt::black);

        icon.translate(4, 4);
        p.drawPolygon(icon);
        p.fillRect(20, 5, 2, 5, QBrush(Qt::black));
        p.drawLine(17, 7, 20, 7);

        /**
         *   state subtype bottom-left bottom-right top-left top-right
         * -------------------------------------------------------------
         *     0       0          y           y
         *     1       0                                 g        g
         *     0       6          y           y
         *     2       6          y                               g
         *     0       4          y           y
         *     1       4                                 g        g
         *     2       4          y                               g
         * -------------------------------------------------------------
         **/

        // fprintf(stderr, "dir: %d type: %d\n", state, iSoldSubType);

        // paint signal light
        // fix potential wrong direction value
        if (iSoldSubType == 6 && state == DIR_HP1)
            state = DIR_HP2;

        switch (state) {
            case DIR_HP0:              // VS0 => 0
                // bottom left
                p.fillRect(14, 8, 2, 2, QBrush(Qt::yellow));
                // bottom right
                p.fillRect(12, 5, 2, 2, QBrush(Qt::yellow));
                break;
            case DIR_HP2:              // VS2 => 2
                if (pref.hp2) {
                    // bottom left
                    p.fillRect(14, 8, 2, 2, QBrush(Qt::yellow));
                    //top right
                    p.fillRect(5, 5, 2, 2, QBrush(Qt::green));
                    break;
                }
            case DIR_HP1:              // VS1 => 1
                //top left
                p.fillRect(7, 8, 2, 2, QBrush(Qt::green));
                //top right
                p.fillRect(5, 5, 2, 2, QBrush(Qt::green));
                break;
        }

        // paint track lights
        if (trackindicatoroff) {
            for (int i = 0; i < 7; ++i)
                p.fillRect(4 + 7 * i, h / 2 - 2, 5, 5, QBrush(Qt::lightGray));
        }
        else {
            QColor c;
            if (occupied)
                c = QColor(Qt::red);
            else {
                if (routed)
                    c = QColor(255, 225, 0);
                else
                    c = QColor(Qt::darkGray);
            }
            p.fillRect(w / 3 + 1, h / 2 - 1, w / 3, 3, QBrush(c));
            p.setPen(QPen(Qt::black));
        }

        // paint text label
        if (sSoldText != "-1" && !sSoldText.isEmpty()) {
            QFont f(QApplication::font());
            f.setPointSize(QApplication::font().pointSize() - 3);
            p.setFont(f);
            QFontMetrics fm(f);
            QString s;

            if (pref.addresslabeling)
                s.setNum(address1);
            else
                s = sSoldText;

            QRect br = fm.boundingRect(s);
            br.setWidth(br.width() + 4);
            br.setHeight(br.height() - 2);

            br.moveBottomRight(QPoint(w / 2 + br.width()/2, h - 3));

            p.fillRect(br, QBrush(Qt::white));
            p.drawText(br, Qt::AlignCenter | Qt::TextSingleLine |
                    Qt::TextDontClip, s);
        }
    }

    // right signal ZP
    else if (classid == siciZp1) {
        QPainter p(&background);

        int w = background.width();
        int h = background.height();

        // paint track
        p.fillRect(0, h / 2 - 3, w, 7, QBrush(Qt::black));

        // paint lock light
        if (lockCounter == 0)
            p.setBrush(Qt::darkGray);
        else
            p.setBrush(QColor(255, 225, 0));

        p.drawEllipse(w / 2 - 3, h - 10, 4, 4);

        // paint signal icon
        p.fillRect(w - 23, 25, 2, 5, QBrush(Qt::black));
        p.fillRect(w - 16, 22, 11, 11, QBrush(Qt::black));
        p.drawLine(w - 17, 27, w - 21, 27);

        // paint signal light
        QPolygon lights = QPolygon(12);
        lights.putPoints(0, 12, 3, 0, 5, 0, 1, 1, 7, 1, 0, 3, 8, 3,
                0, 5, 8, 5, 1, 7, 7, 7, 3, 8, 5, 8);

        lights.translate(w - 15, 23);

        if (state == 1)
            p.setPen(QPen(Qt::green));
        else
            p.setPen(QPen(Qt::darkGray));

        p.drawPoints(lights);

        // paint track lights
        if (trackindicatoroff) {
            for (int i = 0; i < 7; ++i)
                p.fillRect(4 + 7 * i, h / 2 - 2, 5, 5, QBrush(Qt::lightGray));
        }
        else {
            QColor c;
            if (occupied)
                c = QColor(Qt::red);
            else {
                if (routed)
                    c = QColor(255, 225, 0);
                else
                    c = QColor(Qt::darkGray);
            }
            p.fillRect(w / 3 + 1, h / 2 - 1, w / 3, 3, QBrush(c));
            p.setPen(QPen(Qt::black));
        }

        // paint text label
        if (sSoldText != "-1" && !sSoldText.isEmpty()) {
            QFont f(QApplication::font());
            f.setPointSize(QApplication::font().pointSize() - 3);
            p.setFont(f);
            QFontMetrics fm(f);
            QString s;

            if (pref.addresslabeling)
                s.setNum(address1);
            else
                s = sSoldText;

            QRect br = fm.boundingRect(s);
            br.setWidth(br.width() + 4);
            br.setHeight(br.height() - 2);

            br.moveTopLeft(QPoint(w / 2 - br.width()/2, 2));

            p.fillRect(br, QBrush(Qt::white));
            p.drawText(br, Qt::AlignCenter | Qt::TextSingleLine |
                    Qt::TextDontClip, s);
        }
    }

    // left signal ZP
    else if (classid == siciZp3) {
        QPainter p(&background);

        int w = background.width();
        int h = background.height();

        // paint track
        p.fillRect(0, h / 2 - 3, w, 7, QBrush(Qt::black));

        // paint lock light
        if (lockCounter == 0)
            p.setBrush(Qt::darkGray);
        else
            p.setBrush(QColor(255, 225, 0));

        p.drawEllipse(w / 2 - 2, 5, 4, 4);

        // paint signal icon
        p.fillRect(21, 5, 2, 5, QBrush(Qt::black));
        p.fillRect(5, 2, 11, 11, QBrush(Qt::black));
        p.drawLine(16, 7, 20, 7);

        // paint signal light
        QPolygon lights = QPolygon(12);
        lights.putPoints(0, 12, 3, 0, 5, 0, 1, 1, 7, 1, 0, 3, 8, 3,
                0, 5, 8, 5, 1, 7, 7, 7, 3, 8, 5, 8);

        lights.translate(6, 3);

        if (state == 1)
            p.setPen(QPen(Qt::green));
        else
            p.setPen(QPen(Qt::darkGray));

        p.drawPoints(lights);

        // paint track lights
        if (trackindicatoroff) {
            for (int i = 0; i < 7; ++i)
                p.fillRect(4 + 7 * i, h / 2 - 2, 5, 5, QBrush(Qt::lightGray));
        }
        else {
            QColor c;
            if (occupied)
                c = QColor(Qt::red);
            else {
                if (routed)
                    c = QColor(255, 225, 0);
                else
                    c = QColor(Qt::darkGray);
            }
            p.fillRect(w / 3 + 1, h / 2 - 1, w / 3, 3, QBrush(c));
            p.setPen(QPen(Qt::black));
        }

        // paint text label
        if (sSoldText != "-1" && !sSoldText.isEmpty()) {
            QFont f(QApplication::font());
            f.setPointSize(QApplication::font().pointSize() - 3);
            p.setFont(f);
            QFontMetrics fm(f);
            QString s;

            if (pref.addresslabeling)
                s.setNum(address1);
            else
                s = sSoldText;

            QRect br = fm.boundingRect(s);
            br.setWidth(br.width() + 4);
            br.setHeight(br.height() - 2);

            br.moveBottomRight(QPoint(w / 2 + br.width()/2, h - 3));

            p.fillRect(br, QBrush(Qt::white));
            p.drawText(br, Qt::AlignCenter | Qt::TextSingleLine |
                    Qt::TextDontClip, s);
        }
    }

    // straight track
    else if (classid == siciSt1) {
        QPainter p(&background);

        int w = background.width();
        int h = background.height();

        // paint track
        p.fillRect(0, h / 2 - 3, w, 7, QBrush(Qt::black));

        // paint track lights
        if (trackindicatoroff) {
            for (int i = 0; i < 7; ++i)
                p.fillRect(4 + 7 * i, h / 2 - 2, 5, 5,
                        QBrush(Qt::lightGray));
        }
        else {
            QColor c;
            if (occupied)
                c = QColor(Qt::red);
            else {
                if (routed)
                    c = QColor(255, 225, 0);
                else
                    c = QColor(Qt::darkGray);
            }
            p.fillRect(w / 3, h / 2 - 1, w / 3 + 2, 3, QBrush(c));
            p.setPen(QPen(Qt::black));
        }

        // paint text label
        if (sSoldText != "-1" && !sSoldText.isEmpty()) {
            QFont f(QApplication::font());
            f.setPointSize(QApplication::font().pointSize() - 2);
            p.setFont(f);
            QFontMetrics fm(f);

            QRect br = fm.boundingRect(sSoldText);
            br.setWidth(br.width() + 4);
            br.setHeight(br.height() - 2);
            br.moveTop(1);

            p.fillRect(br, QBrush(Qt::white));
            p.drawText(br, Qt::AlignCenter | Qt::TextSingleLine |
                    Qt::TextDontClip, sSoldText);
        }
    }

    // vertical track
    else if (classid == siciSt2) {
        QPainter p(&background);

        int w = background.width();
        int h = background.height();

        // paint track
        p.fillRect(w / 2 - 3, 0, 7, h, QBrush(Qt::black));

        // paint track lights
        if (trackindicatoroff) {
            for (int i = 0; i < 4; ++i)
                p.fillRect(w / 2 - 2 , 4 + 7 * i, 5, 5,
                        QBrush(Qt::lightGray));
        }
        else {
            QColor c;
            if (occupied)
                c = QColor(Qt::red);
            else {
                if (routed)
                    c = QColor(255, 225, 0);
                else
                    c = QColor(Qt::darkGray);
            }
            p.setPen(QPen(c, 3, Qt::SolidLine, Qt::RoundCap, Qt::MiterJoin));
            p.drawLine(w / 2, h / 3, w / 2, 2 * h / 3);
            p.setPen(QPen(Qt::black));
        }
    }

    // vertical turn top left
    else if (classid == siciCl2) {
        QPainter p(&background);
        p.setRenderHint(QPainter::Antialiasing);

        int w = background.width();
        int h = background.height();

        // paint track
        p.save();
        p.setPen(QPen(Qt::black, 7, Qt::SolidLine, Qt::RoundCap));
        p.drawLine(0, 0, w / 2, h / 2);
        p.drawLine(w / 2, h / 2, w / 2, h);
        p.restore();

        // paint track lights
        if (trackindicatoroff) {
            for (int i = 0; i < 2; ++i)
                p.fillRect(w / 2 - 2 , h / 2 + 2 + 7 * i, 5, 5,
                        QBrush(Qt::lightGray));
            p.save();
            p.translate(w / 2, h / 2);
            p.rotate(-WANGLE);
            for (int i = 0; i < 3; ++i)
                p.fillRect(5 + 7 * i, -2, 5, 5,
                        QBrush(Qt::lightGray));
            p.restore();
        }
        else {
            QColor c;
            if (occupied)
                c = QColor(Qt::red);
            else {
                if (routed)
                    c = QColor(255, 225, 0);
                else
                    c = QColor(Qt::darkGray);
            }
            p.setPen(QPen(c, 3, Qt::SolidLine, Qt::RoundCap, Qt::MiterJoin));
            p.drawLine(w / 2, h / 2 + 2, w / 2, 2 * h / 3);
            p.save();
            p.translate(w / 2, h / 2);
            p.rotate(-WANGLE);
            p.drawLine(0, 0, w / 5, 0);
            p.restore();
            p.setPen(QPen(Qt::black));
        }
    }

    // vertical turn top right
    else if (classid == siciCr2) {
        QPainter p(&background);
        p.setRenderHint(QPainter::Antialiasing);

        int w = background.width();
        int h = background.height();

        // paint track
        p.save();
        p.setPen(QPen(Qt::black, 7, Qt::SolidLine, Qt::RoundCap));
        p.drawLine(w, 0, w / 2, h / 2);
        p.drawLine(w / 2, h / 2, w / 2, h);
        p.restore();

        // paint track lights
        if (trackindicatoroff) {
            for (int i = 0; i < 2; ++i)
                p.fillRect(w / 2 - 2 , h / 2 + 2 + 7 * i, 5, 5,
                        QBrush(Qt::lightGray));
            p.save();
            p.translate(w / 2, h / 2);
            p.rotate(-SANGLE);
            for (int i = 0; i < 3; ++i)
                p.fillRect(5 + 7 * i, -2, 5, 5,
                        QBrush(Qt::lightGray));
            p.restore();
        }
        else {
            QColor c;
            if (occupied)
                c = QColor(Qt::red);
            else {
                if (routed)
                    c = QColor(255, 225, 0);
                else
                    c = QColor(Qt::darkGray);
            }
            p.setPen(QPen(c, 3, Qt::SolidLine, Qt::RoundCap, Qt::MiterJoin));
            p.drawLine(w / 2, h / 2 + 2, w / 2, 2 * h / 3);
            p.save();
            p.translate(w / 2, h / 2);
            p.rotate(-SANGLE);
            p.drawLine(0, 0, w / 5, 0);
            p.restore();
            p.setPen(QPen(Qt::black));
        }
    }

    // vertical turn bottom left
    else if (classid == siciCl4) {
        QPainter p(&background);
        p.setRenderHint(QPainter::Antialiasing);

        int w = background.width();
        int h = background.height();

        // paint track
        p.save();
        p.setPen(QPen(Qt::black, 7, Qt::SolidLine, Qt::RoundCap));
        p.drawLine(w / 2, 0, w / 2, h / 2 - 1);
        p.drawLine(w / 2, h / 2 - 1, w, h - 1);
        p.restore();

        // paint track lights
        if (trackindicatoroff) {
            for (int i = 0; i < 2; ++i)
                p.fillRect(w / 2 - 2 , 4 + 7 * i, 5, 5,
                        QBrush(Qt::lightGray));
            p.save();
            p.translate(w / 2, h / 2 - 1);
            p.rotate(SANGLE);
            for (int i = 0; i < 3; ++i)
                p.fillRect(5 + 7 * i, -2, 5, 5,
                        QBrush(Qt::lightGray));
            p.restore();
        }
        else {
            QColor c;
            if (occupied)
                c = QColor(Qt::red);
            else {
                if (routed)
                    c = QColor(255, 225, 0);
                else
                    c = QColor(Qt::darkGray);
            }
            p.setPen(QPen(c, 3, Qt::SolidLine, Qt::RoundCap, Qt::MiterJoin));
            p.drawLine(w / 2, h / 3, w / 2, h / 2 - 2);
            p.save();
            p.translate(w / 2, h / 2 - 1);
            p.rotate(SANGLE);
            p.drawLine(0, 0, w / 5, 0);
            p.restore();
            p.setPen(QPen(Qt::black));
        }
    }

    // vertical turn bottom right
    else if (classid == siciCr4) {
        QPainter p(&background);
        p.setRenderHint(QPainter::Antialiasing);

        int w = background.width();
        int h = background.height();

        // paint track
        p.save();
        p.setPen(QPen(Qt::black, 7, Qt::SolidLine, Qt::RoundCap));
        p.drawLine(w / 2, 0, w / 2, h / 2 - 1);
        p.drawLine(w / 2, h / 2 - 1 , 0, h - 1);
        p.restore();

        // paint track lights
        if (trackindicatoroff) {
            for (int i = 0; i < 2; ++i)
                p.fillRect(w / 2 - 2 , 4 + 7 * i, 5, 5,
                        QBrush(Qt::lightGray));
            p.save();
            p.translate(w / 2, h / 2);
            p.rotate(WANGLE);
            for (int i = 0; i < 3; ++i)
                p.fillRect(5 + 7 * i, -2, 5, 5,
                        QBrush(Qt::lightGray));
            p.restore();
        }
        else {
            QColor c;
            if (occupied)
                c = QColor(Qt::red);
            else {
                if (routed)
                    c = QColor(255, 225, 0);
                else
                    c = QColor(Qt::darkGray);
            }
            p.setPen(QPen(c, 3, Qt::SolidLine, Qt::RoundCap, Qt::MiterJoin));
            p.drawLine(w / 2, h / 3, w / 2, h / 2 - 2);
            p.save();
            p.translate(w / 2, h / 2 - 1);
            p.rotate(WANGLE);
            p.drawLine(0, 0, w / 5, 0);
            p.restore();
            p.setPen(QPen(Qt::black));
        }
    }

    // track turn right and left
    else if (classid == siciCr1 || classid == siciCl1) {
        bool left = (classid == siciCl1);

        QPainter p(&background);
        p.setRenderHint(QPainter::Antialiasing);

        int w = background.width();
        int h = background.height();

        // paint track
        static const QPoint lpoints[3] = {
            QPoint(0, h / 2),
            QPoint(w / 2, h / 2),
            QPoint(w, 0),
        };

        static const QPoint rpoints[3] = {
            QPoint(0, h / 2),
            QPoint(w / 2, h / 2),
            QPoint(w, h - 1),
        };

        p.save();
        p.setPen(QPen(Qt::black, 7, Qt::SolidLine, Qt::RoundCap));
        if (left)
            p.drawPolyline(lpoints, 3);
        else
            p.drawPolyline(rpoints, 3);
        p.restore();

        // paint track lights
        if (trackindicatoroff) {
            for (int i = 0; i < 3; ++i)
                p.fillRect(4 + 7 * i, h / 2 - 2, 5, 5,
                        QBrush(Qt::lightGray));
            p.save();
            p.translate(w / 2, h / 2);

            if (left)
                p.rotate(-SANGLE);
            else
                p.rotate(SANGLE);

            for (int i = 0; i < 3; ++i)
                p.fillRect(5 + 7 * i, -2, 5, 5, QBrush(Qt::lightGray));
            p.restore();
        }
        else {
            QColor c;
            if (occupied)
                c = QColor(Qt::red);
            else {
                if (routed)
                    c = QColor(255, 225, 0);
                else
                    c = QColor(Qt::darkGray);
            }
            p.fillRect(w / 3, h / 2 - 1, w / 5 , 3, QBrush(c));
            p.save();
            p.translate(w / 2, h / 2);

            if (left)
                p.rotate(-SANGLE);
            else
                p.rotate(SANGLE);

            p.setPen(QPen(c, 3, Qt::SolidLine, Qt::RoundCap, Qt::MiterJoin));
            p.drawLine(0, 0, w / 5, 0);
            p.restore();
        }
    }

    // (rotated) track turn right and left
    else if (classid == siciCr3 || classid == siciCl3) {
        bool left = (classid == siciCl3);

        QPainter p(&background);
        p.setRenderHint(QPainter::Antialiasing);

        int w = background.width();
        int h = background.height();

        // paint track
        static const QPoint lpoints[3] = {
            QPoint(0, h - 1),
            QPoint(w / 2, h / 2),
            QPoint(w, h / 2),
        };

        static const QPoint rpoints[3] = {
            QPoint(0, 0),
            QPoint(w / 2, h / 2),
            QPoint(w, h / 2),
        };

        p.save();
        p.setPen(QPen(Qt::black, 7, Qt::SolidLine, Qt::RoundCap));
        if (left)
            p.drawPolyline(lpoints, 3);
        else
            p.drawPolyline(rpoints, 3);
        p.restore();

        // paint track lights
        if (trackindicatoroff) {
            for (int i = 4; i < 7; ++i)
                p.fillRect(4 + 7 * i, h / 2 - 2, 5, 5,
                        QBrush(Qt::lightGray));

            p.save();
            p.translate(w / 2, h / 2);

            if (left)
                p.rotate(WANGLE);
            else
                p.rotate(-WANGLE);

            for (int i = 0; i < 3; ++i)
                p.fillRect(5 + 7 * i, -2, 5, 5, QBrush(Qt::lightGray));
            p.restore();
        }
        else {
            QColor c;
            if (occupied)
                c = QColor(Qt::red);
            else {
                if (routed)
                    c = QColor(255, 225, 0);
                else
                    c = QColor(Qt::darkGray);
            }
            p.fillRect(w / 2, h / 2 - 1, w / 5 , 3, QBrush(c));
            p.save();
            p.translate(w / 2, h / 2);

            if (left)
                p.rotate(WANGLE);
            else
                p.rotate(-WANGLE);

            p.setPen(QPen(c, 3, Qt::SolidLine, Qt::RoundCap, Qt::MiterJoin));
            p.drawLine(0, 0, w / 5, 0);
            p.restore();
        }
    }

    // diagonal track right and left
    else if (classid == siciSt3 || classid == siciSt4) {
        bool left = (classid == siciSt4);

        QPainter p(&background);
        p.setRenderHint(QPainter::Antialiasing);

        int w = background.width();
        int h = background.height();

        // paint track
        p.setPen(QPen(Qt::black, 7));
        if (left)
            p.drawLine(0, h - 1, w, 0);
        else
            p.drawLine(0, 0, w, h - 1);

        // paint track lights
        if (trackindicatoroff) {
            p.save();
            p.translate(w / 2, h / 2);

            if (left)
                p.rotate(-SANGLE);
            else
                p.rotate(SANGLE);

            for (int i = -3; i < 4; ++i)
                p.fillRect(-3 + 7 * i, -2, 5, 5, QBrush(Qt::lightGray));

            p.restore();
        }
        else {
            QColor c;
            if (occupied)
                c = QColor(Qt::red);
            else {
                if (routed)
                    c = QColor(255, 225, 0);
                else
                    c = QColor(Qt::darkGray);
            }
            p.save();
            p.translate(w / 2, h / 2);

            if (left)
                p.rotate(-SANGLE);
            else
                p.rotate(SANGLE);

            p.setPen(QPen(c, 3, Qt::SolidLine, Qt::RoundCap, Qt::MiterJoin));
            p.drawLine(-w / 6, 0, w / 6, 0);

            p.restore();
        }
    }

    // diagonal crossing (hosentraeger)
    else if (classid == siciKrh) {
        QPainter p(&background);
        p.setRenderHint(QPainter::Antialiasing);

        int w = background.width();
        int h = background.height();

        // paint track
        p.setPen(QPen(Qt::black, 7));
        p.drawLine(0, 0, w, h - 1);
        p.drawLine(0, h - 1, w, 0);

        // paint track lights
        if (trackindicatoroff) {
            p.save();
            p.translate(w / 2, h / 2);

            p.rotate(-SANGLE);
            for (int i = 0; i < 7; ++i)
                p.fillRect(-23 + 7 * i, -2, 5, 5, QBrush(Qt::lightGray));
            p.rotate(2 * SANGLE);
            for (int i = 0; i < 7; ++i)
                p.fillRect(-23 + 7 * i, -2, 5, 5, QBrush(Qt::lightGray));
            p.restore();
        }

        /**
         * routedtrack routed occupied track1 track2
         * -----------------------------------------
         *      0        0       0       g      g
         *      1        1       0       y      g
         *      2        1       0       g      y
         *      0        0       1       r      r
         *      1        1       1       r      g
         *      2        1       1       g      r
         * -----------------------------------------
         **/
        else {
            QColor c1, c2;
            if (occupied)
                if (routedtrack == 1) {
                    c1 = QColor(Qt::red);
                    c2 = QColor(Qt::darkGray);
                }
                else if (routedtrack == 2) {
                    c1 = QColor(Qt::darkGray);
                    c2 = QColor(Qt::red);
                }
                else {
                    c1 = QColor(Qt::red);
                    c2 = QColor(Qt::red);
                }
            else {
                if (routed)
                    if (routedtrack == 1) {
                        c1 = QColor(255, 225, 0);
                        c2 = QColor(Qt::darkGray);
                    }
                    else {
                        c1 = QColor(Qt::darkGray);
                        c2 = QColor(255, 225, 0);
                    }
                else {
                    c1 = QColor(Qt::darkGray);
                    c2 = QColor(Qt::darkGray);
                }
            }
            int startx = w / 5 - 2;
            int stopx = 2 * w / 5 - 4;
            int starty = h / 5 - 2;
            int stopy = 2 * h / 5 - 3;

            // track 1
            p.setPen(QPen(c1, 3, Qt::SolidLine, Qt::RoundCap, Qt::MiterJoin));
            p.drawLine(startx, h - starty - 1, stopx, h - stopy - 1);
            p.drawLine(w - startx - 1, starty, w - stopx - 1, stopy);

            // track 2
            p.setPen(QPen(c2, 3, Qt::SolidLine, Qt::RoundCap, Qt::MiterJoin));
            p.drawLine(startx, starty, stopx, stopy);
            p.drawLine(w - startx - 1, h - starty - 1, w - stopx - 1,
                    h - stopy - 1);
        }

        // paint lock light
        p.setPen(QPen(Qt::black, 1));
        if (lockCounter == 0)
            p.setBrush(Qt::darkGray);
        else
            p.setBrush(QColor(255, 225, 0));

        p.drawEllipse(w / 4 - 3, h / 2 - 2, 4, 4);
    }

    // left crossing
    else if (classid == siciKl1) {
        QPainter p(&background);
        p.setRenderHint(QPainter::Antialiasing);

        int w = background.width();
        int h = background.height();

        // paint track
        p.setPen(QPen(Qt::black, 7));
        p.drawLine(0, h / 2, w, h / 2);
        p.drawLine(0, h - 1, w, 0);

        // paint track lights
        if (trackindicatoroff) {
            for (int i = 0; i < 7; ++i)
                p.fillRect(4 + 7 * i, h / 2 - 2, 5, 5,
                        QBrush(Qt::lightGray));
            p.save();
            p.translate(w / 2, h / 2);

            p.rotate(-SANGLE);
            for (int i = 0; i < 7; ++i)
                p.fillRect(-23 + 7 * i, -2, 5, 5, QBrush(Qt::lightGray));
            p.restore();
        }

        /**
         * routedtrack routed occupied track1 track2
         * -----------------------------------------
         *      0        0       0       g      g
         *      1        1       0       y      g
         *      2        1       0       g      y
         *      0        0       1       r      r
         *      1        1       1       r      g
         *      2        1       1       g      r
         * -----------------------------------------
         **/
        else {
            QColor c1, c2;
            if (occupied)
                if (routedtrack == 1) {
                    c1 = QColor(Qt::red);
                    c2 = QColor(Qt::darkGray);
                }
                else if (routedtrack == 2) {
                    c1 = QColor(Qt::darkGray);
                    c2 = QColor(Qt::red);
                }
                else {
                    c1 = QColor(Qt::red);
                    c2 = QColor(Qt::red);
                }
            else {
                if (routed)
                    if (routedtrack == 1) {
                        c1 = QColor(255, 225, 0);
                        c2 = QColor(Qt::darkGray);
                    }
                    else {
                        c1 = QColor(Qt::darkGray);
                        c2 = QColor(255, 225, 0);
                    }
                else {
                    c1 = QColor(Qt::darkGray);
                    c2 = QColor(Qt::darkGray);
                }
            }
            int startx = w / 5 - 2;
            int stopx = 2 * w / 5 - 4;
            int starty = h / 5 - 2;
            int stopy = 2 * h / 5 - 3;

            // diagonal track (1)
            p.setPen(QPen(c1, 3, Qt::SolidLine, Qt::RoundCap, Qt::MiterJoin));
            p.drawLine(startx, h - starty - 1, stopx, h - stopy - 1);
            p.drawLine(w - startx - 1, starty, w - stopx - 1, stopy);

            // straight track (2)
            p.setPen(QPen(c2, 3, Qt::SolidLine, Qt::RoundCap, Qt::MiterJoin));
            p.drawLine(startx, h /2, stopx, h / 2);
            p.drawLine(w - startx - 1, h / 2, w - stopx - 1, h / 2);
        }

        // paint lock light
        p.setPen(QPen(Qt::black, 1));
        if (lockCounter == 0)
            p.setBrush(Qt::darkGray);
        else
            p.setBrush(QColor(255, 225, 0));

        p.drawEllipse(w / 4, 6, 4, 4);
    }

    // right crossing
    else if (classid == siciKr1) {
        QPainter p(&background);
        p.setRenderHint(QPainter::Antialiasing);

        int w = background.width();
        int h = background.height();

        // paint track
        p.setPen(QPen(Qt::black, 7));
        p.drawLine(0, h / 2, w, h / 2);
        p.drawLine(0, 0, w, h - 1);

        // paint track lights
        if (trackindicatoroff) {
            for (int i = 0; i < 7; ++i)
                p.fillRect(4 + 7 * i, h / 2 - 2, 5, 5,
                        QBrush(Qt::lightGray));
            p.save();
            p.translate(w / 2, h / 2);

            p.rotate(SANGLE);
            for (int i = 0; i < 7; ++i)
                p.fillRect(-23 + 7 * i, -2, 5, 5, QBrush(Qt::lightGray));
            p.restore();
        }

        /**
         * routedtrack routed occupied track1 track2
         * -----------------------------------------
         *      0        0       0       g      g
         *      1        1       0       y      g
         *      2        1       0       g      y
         *      0        0       1       r      r
         *      1        1       1       r      g
         *      2        1       1       g      r
         * -----------------------------------------
         **/
        else {
            QColor c1, c2;
            if (occupied)
                if (routedtrack == 1) {
                    c1 = QColor(Qt::red);
                    c2 = QColor(Qt::darkGray);
                }
                else if (routedtrack == 2) {
                    c1 = QColor(Qt::darkGray);
                    c2 = QColor(Qt::red);
                }
                else {
                    c1 = QColor(Qt::red);
                    c2 = QColor(Qt::red);
                }
            else {
                if (routed)
                    if (routedtrack == 1) {
                        c1 = QColor(255, 225, 0);
                        c2 = QColor(Qt::darkGray);
                    }
                    else {
                        c1 = QColor(Qt::darkGray);
                        c2 = QColor(255, 225, 0);
                    }
                else {
                    c1 = QColor(Qt::darkGray);
                    c2 = QColor(Qt::darkGray);
                }
            }
            int startx = w / 5 - 2;
            int stopx = 2 * w / 5 - 4;
            int starty = h / 5 - 2;
            int stopy = 2 * h / 5 - 3;

            // diagonal track (1)
            p.setPen(QPen(c1, 3, Qt::SolidLine, Qt::RoundCap, Qt::MiterJoin));
            p.drawLine(startx, starty, stopx, stopy);
            p.drawLine(w - startx - 1, h - starty - 1, w - stopx - 1,
                    h - stopy - 1);

            // straight track (2)
            p.setPen(QPen(c2, 3, Qt::SolidLine, Qt::RoundCap, Qt::MiterJoin));
            p.drawLine(startx, h / 2, stopx, h / 2);
            p.drawLine(w - startx - 1, h / 2, w - stopx - 1, h / 2);
        }

        // paint lock light
        p.setPen(QPen(Qt::black, 1));
        if (lockCounter == 0)
            p.setBrush(Qt::darkGray);
        else
            p.setBrush(QColor(255, 225, 0));

        p.drawEllipse(3 * w/ 4 - 4, 6, 4, 4);
    }

    // single slip switch left position 1
    else if (classid == siciSl1) {
        QPainter p(&background);
        p.setRenderHint(QPainter::Antialiasing);

        int w = background.width();
        int h = background.height();

        // paint track
        p.setPen(QPen(Qt::black, 7));
        p.drawLine(0, h / 2, w, h / 2);
        p.drawLine(0, h - 1, w, 0);

        // paint drive symbol
        p.setPen(QPen(Qt::black, 1));
        p.drawLine(w / 2 - 7, h / 2 - 5, w / 2 - 2, h / 2 - 5);
        p.drawLine(w / 2 - 2, h / 2 - 5, w / 2 + 3, h / 2 - 8);

        // paint lock light
        if (lockCounter == 0)
            p.setBrush(Qt::darkGray);
        else
            p.setBrush(QColor(255, 225, 0));

        p.drawEllipse(w / 4, 6, 4, 4);

        // paint track lights
        if (trackindicatoroff) {
            for (int i = 0; i < 7; ++i)
                p.fillRect(4 + 7 * i, h / 2 - 2, 5, 5,
                        QBrush(Qt::lightGray));
            p.save();
            p.translate(w / 2, h / 2);

            p.rotate(-SANGLE);
            for (int i = 0; i < 7; ++i)
                p.fillRect(-23 + 7 * i, -2, 5, 5, QBrush(Qt::lightGray));
            p.restore();
        }

        /**
         * lastdir  currentdir  newdir  c1  c2  c3  c4
         * -------------------------------------------
         *     0       0/0        0     -   -   -   -
         *     1       1/1        1     -   -   -   -
         *     2       2/2        2     -   -   -   -
         *
         *     0       0/1        1     -   1   -   2
         *     1       1/0        0     -   2   -   1
         *
         *     1       1/2        2     1   -   2   -
         *     2       2/1        1     2   -   1   -
         *
         *     2       2/0        0     2   2   1   1
         *     0       0/2        2     1   1   2   2
         * -------------------------------------------
         **/
        else {
            QColor c1, c2, c3, c4;
            if (occupied) {
                if (state == 0) {
                    if (lightson) {
                        c1 = QColor(Qt::red);
                        c2 = QColor(Qt::red);
                    }
                    else {
                        c1 = QColor(Qt::darkGray);
                        c2 = QColor(Qt::darkGray);
                    }
                    c3 = QColor(Qt::darkGray);
                    c4 = QColor(Qt::darkGray);
                }
                else if (state == 1) {
                    if (lightson) {
                        c1 = QColor(Qt::red);
                        c4 = QColor(Qt::red);
                    }
                    else {
                        c1 = QColor(Qt::darkGray);
                        c4 = QColor(Qt::darkGray);
                    }
                    c2 = QColor(Qt::darkGray);
                    c3 = QColor(Qt::darkGray);
                }
                else if (state == 2) {
                    c1 = QColor(Qt::darkGray);
                    c2 = QColor(Qt::darkGray);
                    if (lightson) {
                        c3 = QColor(Qt::red);
                        c4 = QColor(Qt::red);
                    }
                    else {
                        c3 = QColor(Qt::darkGray);
                        c4 = QColor(Qt::darkGray);
                    }
                }
                // error indication
                else {
                    c1 = QColor(Qt::red);
                    c2 = QColor(Qt::red);
                    c3 = QColor(Qt::red);
                    c4 = QColor(Qt::red);
                }
            }
            else {
                if (state == 0) {
                    if (!lightson && ((lastdir == 0 && newdir == 2) ||
                            (lastdir == 2 && newdir == 0)))
                        c1 = QColor(Qt::darkGray);
                    else {
                        if ((blinkcounter > 0) || tablelight || routed || (lockCounter > 0))
                            c1 = QColor(255, 225, 0);
                        else
                            c1 = QColor(Qt::darkGray);
                    }

                    if (!lightson && ((lastdir != 0 && newdir == 0) ||
                            (lastdir == 0 && newdir != 0)))
                        c2 = QColor(Qt::darkGray);
                    else {
                        if ((blinkcounter > 0) || tablelight || routed || (lockCounter > 0))
                            c2 = QColor(255, 225, 0);
                        else
                            c2 = QColor(Qt::darkGray);
                    }

                    c3 = QColor(Qt::darkGray);
                    c4 = QColor(Qt::darkGray);
                }
                else if (state == 1) {
                    if (!lightson && ((lastdir == 1 && newdir == 2) ||
                        (lastdir == 2 && newdir == 1)))
                        c1 = QColor(Qt::darkGray);
                    else {
                        if ((blinkcounter > 0) || tablelight || routed || (lockCounter > 0))
                            c1 = QColor(255, 225, 0);
                        else
                            c1 = QColor(Qt::darkGray);
                    }

                    c2 = QColor(Qt::darkGray);
                    c3 = QColor(Qt::darkGray);

                    if (!lightson && ((lastdir == 1 && newdir == 0) ||
                                (lastdir == 0 && newdir == 1)))
                        c4 = QColor(Qt::darkGray);
                    else {
                        if ((blinkcounter > 0) || tablelight || routed || (lockCounter > 0))
                            c4 = QColor(255, 225, 0);
                        else
                            c4 = QColor(Qt::darkGray);
                    }
                }
                else if (state == 2) {
                    c1 = QColor(Qt::darkGray);
                    c2 = QColor(Qt::darkGray);

                    if (!lightson && ((lastdir == 2 && newdir != 2) ||
                                (lastdir != 2 && newdir == 2)))
                        c3 = QColor(Qt::darkGray);
                    else {
                        if ((blinkcounter > 0) || tablelight || routed || (lockCounter > 0))
                            c3 = QColor(255, 225, 0);
                        else
                            c3 = QColor(Qt::darkGray);
                    }

                    if (!lightson && ((lastdir == 2 && newdir == 0) ||
                                (lastdir == 0 && newdir == 2)))
                        c4 = QColor(Qt::darkGray);
                    else {
                        if ((blinkcounter > 0) || tablelight || routed || (lockCounter > 0))
                            c4 = QColor(255, 225, 0);
                        else
                            c4 = QColor(Qt::darkGray);
                    }
                }
                // error indication
                else {
                    c1 = QColor(255, 225, 0);
                    c2 = QColor(255, 225, 0);
                    c3 = QColor(255, 225, 0);
                    c4 = QColor(255, 225, 0);
                }
            }
            int startx = w / 5 - 2;
            int stopx = 2 * w / 5 - 4;
            int starty = h / 5 - 2;
            int stopy = 2 * h / 5 - 3;

            // swap track indicator assignment if element is rotated
            // straight track (1)
            p.setPen(QPen(c1, 3, Qt::SolidLine, Qt::RoundCap,
                        Qt::MiterJoin));
            p.drawLine(startx, h /2, stopx, h / 2);
            // straight track (2)
            p.setPen(QPen(c2, 3, Qt::SolidLine, Qt::RoundCap,
                        Qt::MiterJoin));
            p.drawLine(w - startx - 1, h / 2, w - stopx - 1, h / 2);
            // diagonal track (3)
            p.setPen(QPen(c3, 3, Qt::SolidLine, Qt::RoundCap,
                        Qt::MiterJoin));
            p.drawLine(startx, h - starty - 1, stopx, h - stopy - 1);
            // diagonal track (4)
            p.setPen(QPen(c4, 3, Qt::SolidLine, Qt::RoundCap,
                        Qt::MiterJoin));
            p.drawLine(w - startx - 1, starty, w - stopx - 1, stopy);

            p.setPen(QPen(Qt::black, 1));
        }

        // paint track button
        p.drawPixmap(w / 2 - 4, h / 2 - 3, QPixmap(button_black_xpm));

        // paint text label
        p.setPen(Qt::black);
        if (sSoldText != "-1" && !sSoldText.isEmpty()) {
            QFont f(QApplication::font());
            f.setPointSize(QApplication::font().pointSize() - 3);
            p.setFont(f);
            QFontMetrics fm(f);
            QString s;

            if (pref.addresslabeling)
                s.setNum(address1);
            else
                s = sSoldText;

            QRect br = fm.boundingRect(s);
            br.setWidth(br.width() + 4);
            br.setHeight(br.height() - 2);

            br.moveBottomRight(QPoint(w - 3, h - 3));

            p.fillRect(br, QBrush(Qt::white));
            p.setBrush(Qt::white);
            br.setX(br.x() + 1);
            br.setY(br.y() + 1);
            br.setHeight(br.height() - 2);

            p.drawText(br, Qt::AlignCenter | Qt::TextSingleLine |
                    Qt::TextDontClip, s);
        }
    }

    // single slip switch left position 3
    else if (classid == siciSl3) {
        QPainter p(&background);
        p.setRenderHint(QPainter::Antialiasing);

        int w = background.width();
        int h = background.height();

        // paint track
        p.setPen(QPen(Qt::black, 7));
        p.drawLine(0, h / 2, w, h / 2);
        p.drawLine(0, h - 1, w, 0);

        // paint drive symbol
        p.setPen(QPen(Qt::black, 1));
        p.drawLine(w / 2 + 1, h / 2 + 5, w / 2 + 7, h / 2 + 5);
        p.drawLine(w / 2 + 2, h / 2 + 5, w / 2 - 3, h / 2 + 7);

        // paint lock light
        if (lockCounter == 0)
            p.setBrush(Qt::darkGray);
        else
            p.setBrush(QColor(255, 225, 0));

        p.drawEllipse(3 * w / 4 - 2, h - 11, 4, 4);

        // paint track lights
        if (trackindicatoroff) {
            for (int i = 0; i < 7; ++i)
                p.fillRect(4 + 7 * i, h / 2 - 2, 5, 5,
                        QBrush(Qt::lightGray));
            p.save();
            p.translate(w / 2, h / 2);

            p.rotate(-SANGLE);
            for (int i = 0; i < 7; ++i)
                p.fillRect(-23 + 7 * i, -2, 5, 5, QBrush(Qt::lightGray));
            p.restore();
        }

        /**
         * lastdir  currentdir  newdir  c1  c2  c3  c4
         * -------------------------------------------
         *     0       0/0        0     -   -   -   -
         *     1       1/1        1     -   -   -   -
         *     2       2/2        2     -   -   -   -
         *
         *     0       0/1        1     -   1   -   2
         *     1       1/0        0     -   2   -   1
         *
         *     1       1/2        2     1   -   2   -
         *     2       2/1        1     2   -   1   -
         *
         *     2       2/0        0     2   2   1   1
         *     0       0/2        2     1   1   2   2
         * -------------------------------------------
         **/
        else {
            QColor c1, c2, c3, c4;
            if (occupied) {
                if (state == 0) {
                    if (lightson) {
                        c1 = QColor(Qt::red);
                        c2 = QColor(Qt::red);
                    }
                    else {
                        c1 = QColor(Qt::darkGray);
                        c2 = QColor(Qt::darkGray);
                    }
                    c3 = QColor(Qt::darkGray);
                    c4 = QColor(Qt::darkGray);
                }
                else if (state == 1) {
                    if (lightson) {
                        c1 = QColor(Qt::red);
                        c4 = QColor(Qt::red);
                    }
                    else {
                        c1 = QColor(Qt::darkGray);
                        c4 = QColor(Qt::darkGray);
                    }
                    c2 = QColor(Qt::darkGray);
                    c3 = QColor(Qt::darkGray);
                }
                else if (state == 2) {
                    c1 = QColor(Qt::darkGray);
                    c2 = QColor(Qt::darkGray);
                    if (lightson) {
                        c3 = QColor(Qt::red);
                        c4 = QColor(Qt::red);
                    }
                    else {
                        c3 = QColor(Qt::darkGray);
                        c4 = QColor(Qt::darkGray);
                    }
                }
                // error indication
                else {
                    c1 = QColor(Qt::red);
                    c2 = QColor(Qt::red);
                    c3 = QColor(Qt::red);
                    c4 = QColor(Qt::red);
                }
            }
            else {
                if (state == 0) {
                    if (!lightson && ((lastdir == 0 && newdir == 2) ||
                            (lastdir == 2 && newdir == 0)))
                        c1 = QColor(Qt::darkGray);
                    else {
                        if ((blinkcounter > 0) || tablelight || routed || (lockCounter > 0))
                            c1 = QColor(255, 225, 0);
                        else
                            c1 = QColor(Qt::darkGray);
                    }

                    if (!lightson && ((lastdir != 0 && newdir == 0) ||
                            (lastdir == 0 && newdir != 0)))
                        c2 = QColor(Qt::darkGray);
                    else {
                        if ((blinkcounter > 0) || tablelight || routed || (lockCounter > 0))
                            c2 = QColor(255, 225, 0);
                        else
                            c2 = QColor(Qt::darkGray);
                    }

                    c3 = QColor(Qt::darkGray);
                    c4 = QColor(Qt::darkGray);
                }
                else if (state == 1) {
                    if (!lightson && ((lastdir == 1 && newdir == 2) ||
                        (lastdir == 2 && newdir == 1)))
                        c1 = QColor(Qt::darkGray);
                    else {
                        if ((blinkcounter > 0) || tablelight || routed || (lockCounter > 0))
                            c1 = QColor(255, 225, 0);
                        else
                            c1 = QColor(Qt::darkGray);
                    }

                    c2 = QColor(Qt::darkGray);
                    c3 = QColor(Qt::darkGray);

                    if (!lightson && ((lastdir == 1 && newdir == 0) ||
                                (lastdir == 0 && newdir == 1)))
                        c4 = QColor(Qt::darkGray);
                    else {
                        if ((blinkcounter > 0) || tablelight || routed || (lockCounter > 0))
                            c4 = QColor(255, 225, 0);
                        else
                            c4 = QColor(Qt::darkGray);
                    }
                }
                else if (state == 2) {
                    c1 = QColor(Qt::darkGray);
                    c2 = QColor(Qt::darkGray);

                    if (!lightson && ((lastdir == 2 && newdir != 2) ||
                                (lastdir != 2 && newdir == 2)))
                        c3 = QColor(Qt::darkGray);
                    else {
                        if ((blinkcounter > 0) || tablelight || routed || (lockCounter > 0))
                            c3 = QColor(255, 225, 0);
                        else
                            c3 = QColor(Qt::darkGray);
                    }

                    if (!lightson && ((lastdir == 2 && newdir == 0) ||
                                (lastdir == 0 && newdir == 2)))
                        c4 = QColor(Qt::darkGray);
                    else {
                        if ((blinkcounter > 0) || tablelight || routed || (lockCounter > 0))
                            c4 = QColor(255, 225, 0);
                        else
                            c4 = QColor(Qt::darkGray);
                    }
                }
                // error indication
                else {
                    c1 = QColor(255, 225, 0);
                    c2 = QColor(255, 225, 0);
                    c3 = QColor(255, 225, 0);
                    c4 = QColor(255, 225, 0);
                }
            }
            int startx = w / 5 - 2;
            int stopx = 2 * w / 5 - 4;
            int starty = h / 5 - 2;
            int stopy = 2 * h / 5 - 3;

            // swap track indicator assignment if element is rotated
            // straight track (1)
            p.setPen(QPen(c1, 3, Qt::SolidLine, Qt::RoundCap,
                        Qt::MiterJoin));
            p.drawLine(w - startx - 1, h / 2, w - stopx - 1, h / 2);
            // straight track (2)
            p.setPen(QPen(c2, 3, Qt::SolidLine, Qt::RoundCap,
                        Qt::MiterJoin));
            p.drawLine(startx, h /2, stopx, h / 2);
            // diagonal track (3)
            p.setPen(QPen(c3, 3, Qt::SolidLine, Qt::RoundCap,
                        Qt::MiterJoin));
            p.drawLine(w - startx - 1, starty, w - stopx - 1, stopy);
            // diagonal track (4)
            p.setPen(QPen(c4, 3, Qt::SolidLine, Qt::RoundCap,
                        Qt::MiterJoin));
            p.drawLine(startx, h - starty - 1, stopx, h - stopy - 1);
            p.setPen(QPen(Qt::black, 1));
        }

        // paint track button
        p.drawPixmap(w / 2 - 4, h / 2 - 3, QPixmap(button_black_xpm));

        // paint text label
        p.setPen(Qt::black);
        if (sSoldText != "-1" && !sSoldText.isEmpty()) {
            QFont f(QApplication::font());
            f.setPointSize(QApplication::font().pointSize() - 3);
            p.setFont(f);
            QFontMetrics fm(f);
            QString s;

            if (pref.addresslabeling)
                s.setNum(address1);
            else
                s = sSoldText;

            QRect br = fm.boundingRect(s);
            br.setWidth(br.width() + 4);
            br.setHeight(br.height() - 2);

            br.moveTopLeft(QPoint(2, 2));

            p.fillRect(br, QBrush(Qt::white));
            p.setBrush(Qt::white);
            br.setX(br.x() + 1);
            br.setY(br.y() + 1);
            br.setHeight(br.height() - 2);

            p.drawText(br, Qt::AlignCenter | Qt::TextSingleLine |
                    Qt::TextDontClip, s);
        }
    }

    // single slip switch right position 1
    else if (classid == siciSr1) {
        QPainter p(&background);
        p.setRenderHint(QPainter::Antialiasing);

        int w = background.width();
        int h = background.height();

        // paint track
        p.setPen(QPen(Qt::black, 7));
        p.drawLine(0, h / 2, w, h / 2);
        p.drawLine(0, 0, w, h - 1);

        // paint drive symbol
        p.setPen(QPen(Qt::black, 1));
        p.drawLine(w / 2 + 7, h / 2 - 5, w / 2 + 2, h / 2 - 5);
        p.drawLine(w / 2 + 2, h / 2 - 5, w / 2 - 3, h / 2 - 8);

        // paint lock light
        if (lockCounter == 0)
            p.setBrush(Qt::darkGray);
        else
            p.setBrush(QColor(255, 225, 0));

        p.drawEllipse(3 * w/ 4 - 4, 6, 4, 4);

        // paint track lights
        if (trackindicatoroff) {
            for (int i = 0; i < 7; ++i)
                p.fillRect(4 + 7 * i, h / 2 - 2, 5, 5,
                        QBrush(Qt::lightGray));
            p.save();
            p.translate(w / 2, h / 2);

            p.rotate(SANGLE);
            for (int i = 0; i < 7; ++i)
                p.fillRect(-23 + 7 * i, -2, 5, 5, QBrush(Qt::lightGray));
            p.restore();
        }

        /**
         * lastdir  currentdir  newdir  c1  c2  c3  c4
         * -------------------------------------------
         *     0       0/0        0     -   -   -   -
         *     1       1/1        1     -   -   -   -
         *     2       2/2        2     -   -   -   -
         *
         *     0       0/1        1     -   1   -   2
         *     1       1/0        0     -   2   -   1
         *
         *     1       1/2        2     1   -   2   -
         *     2       2/1        1     2   -   1   -
         *
         *     2       2/0        0     2   2   1   1
         *     0       0/2        2     1   1   2   2
         * -------------------------------------------
         **/
        else {
            QColor c1, c2, c3, c4;
            if (occupied) {
                if (state == 0) {
                    if (lightson) {
                        c1 = QColor(Qt::red);
                        c2 = QColor(Qt::red);
                    }
                    else {
                        c1 = QColor(Qt::darkGray);
                        c2 = QColor(Qt::darkGray);
                    }
                    c3 = QColor(Qt::darkGray);
                    c4 = QColor(Qt::darkGray);
                }
                else if (state == 1) {
                    if (lightson) {
                        c1 = QColor(Qt::red);
                        c4 = QColor(Qt::red);
                    }
                    else {
                        c1 = QColor(Qt::darkGray);
                        c4 = QColor(Qt::darkGray);
                    }
                    c2 = QColor(Qt::darkGray);
                    c3 = QColor(Qt::darkGray);
                }
                else if (state == 2) {
                    c1 = QColor(Qt::darkGray);
                    c2 = QColor(Qt::darkGray);
                    if (lightson) {
                        c3 = QColor(Qt::red);
                        c4 = QColor(Qt::red);
                    }
                    else {
                        c3 = QColor(Qt::darkGray);
                        c4 = QColor(Qt::darkGray);
                    }
                }
                // error indication
                else {
                    c1 = QColor(Qt::red);
                    c2 = QColor(Qt::red);
                    c3 = QColor(Qt::red);
                    c4 = QColor(Qt::red);
                }
            }
            else {
                if (state == 0) {
                    if (!lightson && ((lastdir == 0 && newdir == 2) ||
                            (lastdir == 2 && newdir == 0)))
                        c1 = QColor(Qt::darkGray);
                    else {
                        if ((blinkcounter > 0) || tablelight || routed || (lockCounter > 0))
                            c1 = QColor(255, 225, 0);
                        else
                            c1 = QColor(Qt::darkGray);
                    }

                    if (!lightson && ((lastdir == 0 && newdir != 0) ||
                            (lastdir != 0 && newdir == 0)))
                        c2 = QColor(Qt::darkGray);
                    else {
                        if ((blinkcounter > 0) || tablelight || routed || (lockCounter > 0))
                            c2 = QColor(255, 225, 0);
                        else
                            c2 = QColor(Qt::darkGray);
                    }

                    c3 = QColor(Qt::darkGray);
                    c4 = QColor(Qt::darkGray);
                }
                else if (state == 1) {
                    if (!lightson && ((lastdir == 1 && newdir == 2) ||
                        (lastdir == 2 && newdir == 1)))
                        c1 = QColor(Qt::darkGray);
                    else {
                        if ((blinkcounter > 0) || tablelight || routed || (lockCounter > 0))
                            c1 = QColor(255, 225, 0);
                        else
                            c1 = QColor(Qt::darkGray);
                    }

                    c2 = QColor(Qt::darkGray);
                    c3 = QColor(Qt::darkGray);

                    if (!lightson && ((lastdir == 1 && newdir == 0) ||
                                (lastdir == 0 && newdir == 1)))
                        c4 = QColor(Qt::darkGray);
                    else {
                        if ((blinkcounter > 0) || tablelight || routed || (lockCounter > 0))
                            c4 = QColor(255, 225, 0);
                        else
                            c4 = QColor(Qt::darkGray);
                    }
                }
                else if (state == 2) {
                    c1 = QColor(Qt::darkGray);
                    c2 = QColor(Qt::darkGray);

                    if (!lightson && ((lastdir == 2 && newdir != 2) ||
                                (lastdir != 2 && newdir == 2)))
                        c3 = QColor(Qt::darkGray);
                    else {
                        if ((blinkcounter > 0) || tablelight || routed || (lockCounter > 0))
                            c3 = QColor(255, 225, 0);
                        else
                            c3 = QColor(Qt::darkGray);
                    }

                    if (!lightson && ((lastdir == 2 && newdir == 0) ||
                                (lastdir == 0 && newdir == 2)))
                        c4 = QColor(Qt::darkGray);
                    else {
                        if ((blinkcounter > 0) || tablelight || routed || (lockCounter > 0))
                            c4 = QColor(255, 225, 0);
                        else
                            c4 = QColor(Qt::darkGray);
                    }
                }
                // error indication
                else {
                    c1 = QColor(255, 225, 0);
                    c2 = QColor(255, 225, 0);
                    c3 = QColor(255, 225, 0);
                    c4 = QColor(255, 225, 0);
                }
            }
            int startx = w / 5 - 2;
            int stopx = 2 * w / 5 - 4;
            int starty = h / 5 - 2;
            int stopy = 2 * h / 5 - 3;

            // swap track indicator assignment if element is rotated
            // straight track (1)
            p.setPen(QPen(c1, 3, Qt::SolidLine, Qt::RoundCap,
                        Qt::MiterJoin));
            p.drawLine(w - startx - 1, h / 2, w - stopx - 1, h / 2);
            // straight track (2)
            p.setPen(QPen(c2, 3, Qt::SolidLine, Qt::RoundCap,
                        Qt::MiterJoin));
            p.drawLine(startx, h /2, stopx, h / 2);
            // diagonal track (3)
            p.setPen(QPen(c3, 3, Qt::SolidLine, Qt::RoundCap,
                        Qt::MiterJoin));
            p.drawLine(w - startx - 1, h - starty - 1, w - stopx - 1,
                    h - stopy - 1);
            // diagonal track (4)
            p.setPen(QPen(c4, 3, Qt::SolidLine, Qt::RoundCap,
                        Qt::MiterJoin));
            p.drawLine(startx, starty, stopx, stopy);

            p.setPen(QPen(Qt::black, 1));
        }

        // paint track button
        p.drawPixmap(w / 2 - 4, h / 2 - 3, QPixmap(button_black_xpm));

        // paint text label
        p.setPen(Qt::black);
        if (sSoldText != "-1" && !sSoldText.isEmpty()) {
            QFont f(QApplication::font());
            f.setPointSize(QApplication::font().pointSize() - 3);
            p.setFont(f);
            QFontMetrics fm(f);
            QString s;

            if (pref.addresslabeling)
                s.setNum(address1);
            else
                s = sSoldText;

            QRect br = fm.boundingRect(s);
            br.setWidth(br.width() + 4);
            br.setHeight(br.height() - 2);

            br.moveBottomLeft(QPoint(2, h - 3));

            p.fillRect(br, QBrush(Qt::white));
            p.setBrush(Qt::white);
            br.setX(br.x() + 1);
            br.setY(br.y() + 1);
            br.setHeight(br.height() - 2);

            p.drawText(br, Qt::AlignCenter | Qt::TextSingleLine |
                    Qt::TextDontClip, s);
        }
    }

    // single slip switch right position 3
    else if (classid == siciSr3) {
        QPainter p(&background);
        p.setRenderHint(QPainter::Antialiasing);

        int w = background.width();
        int h = background.height();

        // paint track
        p.setPen(QPen(Qt::black, 7));
        p.drawLine(0, h / 2, w, h / 2);
        p.drawLine(0, 0, w, h - 1);

        // paint drive symbol
        p.setPen(QPen(Qt::black, 1));
        p.drawLine(w / 2 - 1, h / 2 + 5, w / 2 - 7, h / 2 + 5);
        p.drawLine(w / 2 - 3, h / 2 + 5, w / 2 + 2, h / 2 + 7);

        // paint lock light
        if (lockCounter == 0)
            p.setBrush(Qt::darkGray);
        else
            p.setBrush(QColor(255, 225, 0));

        p.drawEllipse(w / 4, h - 11, 4, 4);

        // paint track lights
        if (trackindicatoroff) {
            for (int i = 0; i < 7; ++i)
                p.fillRect(4 + 7 * i, h / 2 - 2, 5, 5,
                        QBrush(Qt::lightGray));
            p.save();
            p.translate(w / 2, h / 2);

            p.rotate(SANGLE);
            for (int i = 0; i < 7; ++i)
                p.fillRect(-23 + 7 * i, -2, 5, 5, QBrush(Qt::lightGray));
            p.restore();
        }

        /**
         * lastdir  currentdir  newdir  c1  c2  c3  c4
         * -------------------------------------------
         *     0       0/0        0     -   -   -   -
         *     1       1/1        1     -   -   -   -
         *     2       2/2        2     -   -   -   -
         *
         *     0       0/1        1     -   1   -   2
         *     1       1/0        0     -   2   -   1
         *
         *     1       1/2        2     1   -   2   -
         *     2       2/1        1     2   -   1   -
         *
         *     2       2/0        0     2   2   1   1
         *     0       0/2        2     1   1   2   2
         * -------------------------------------------
         **/
        else {
            QColor c1, c2, c3, c4;
            if (occupied) {
                if (state == 0) {
                    if (lightson) {
                        c1 = QColor(Qt::red);
                        c2 = QColor(Qt::red);
                    }
                    else {
                        c1 = QColor(Qt::darkGray);
                        c2 = QColor(Qt::darkGray);
                    }
                    c3 = QColor(Qt::darkGray);
                    c4 = QColor(Qt::darkGray);
                }
                else if (state == 1) {
                    if (lightson) {
                        c1 = QColor(Qt::red);
                        c4 = QColor(Qt::red);
                    }
                    else {
                        c1 = QColor(Qt::darkGray);
                        c4 = QColor(Qt::darkGray);
                    }
                    c2 = QColor(Qt::darkGray);
                    c3 = QColor(Qt::darkGray);
                }
                else if (state == 2) {
                    c1 = QColor(Qt::darkGray);
                    c2 = QColor(Qt::darkGray);
                    if (lightson) {
                        c3 = QColor(Qt::red);
                        c4 = QColor(Qt::red);
                    }
                    else {
                        c3 = QColor(Qt::darkGray);
                        c4 = QColor(Qt::darkGray);
                    }
                }
                // error indication
                else {
                    c1 = QColor(Qt::red);
                    c2 = QColor(Qt::red);
                    c3 = QColor(Qt::red);
                    c4 = QColor(Qt::red);
                }
            }
            else {
                if (state == 0) {
                    if (!lightson && ((lastdir == 0 && newdir == 2) ||
                            (lastdir == 2 && newdir == 0)))
                        c1 = QColor(Qt::darkGray);
                    else {
                        if ((blinkcounter > 0) || tablelight || routed || (lockCounter > 0))
                            c1 = QColor(255, 225, 0);
                        else
                            c1 = QColor(Qt::darkGray);
                    }

                    if (!lightson && ((lastdir == 0 && newdir != 0) ||
                            (lastdir != 0 && newdir == 0)))
                        c2 = QColor(Qt::darkGray);
                    else {
                        if ((blinkcounter > 0) || tablelight || routed || (lockCounter > 0))
                            c2 = QColor(255, 225, 0);
                        else
                            c2 = QColor(Qt::darkGray);
                    }

                    c3 = QColor(Qt::darkGray);
                    c4 = QColor(Qt::darkGray);
                }
                else if (state == 1) {
                    if (!lightson && ((lastdir == 1 && newdir == 2) ||
                        (lastdir == 2 && newdir == 1)))
                        c1 = QColor(Qt::darkGray);
                    else {
                        if ((blinkcounter > 0) || tablelight || routed || (lockCounter > 0))
                            c1 = QColor(255, 225, 0);
                        else
                            c1 = QColor(Qt::darkGray);
                    }

                    c2 = QColor(Qt::darkGray);
                    c3 = QColor(Qt::darkGray);

                    if (!lightson && ((lastdir == 1 && newdir == 0) ||
                                (lastdir == 0 && newdir == 1)))
                        c4 = QColor(Qt::darkGray);
                    else {
                        if ((blinkcounter > 0) || tablelight || routed || (lockCounter > 0))
                            c4 = QColor(255, 225, 0);
                        else
                            c4 = QColor(Qt::darkGray);
                    }
                }
                else if (state == 2) {
                    c1 = QColor(Qt::darkGray);
                    c2 = QColor(Qt::darkGray);

                    if (!lightson && ((lastdir == 2 && newdir != 2) ||
                                (lastdir != 2 && newdir == 2)))
                        c3 = QColor(Qt::darkGray);
                    else {
                        if ((blinkcounter > 0) || tablelight || routed || (lockCounter > 0))
                            c3 = QColor(255, 225, 0);
                        else
                            c3 = QColor(Qt::darkGray);
                    }

                    if (!lightson && ((lastdir == 2 && newdir == 0) ||
                                (lastdir == 0 && newdir == 2)))
                        c4 = QColor(Qt::darkGray);
                    else {
                        if ((blinkcounter > 0) || tablelight || routed || (lockCounter > 0))
                            c4 = QColor(255, 225, 0);
                        else
                            c4 = QColor(Qt::darkGray);
                    }
                }
                // error indication
                else {
                    c1 = QColor(255, 225, 0);
                    c2 = QColor(255, 225, 0);
                    c3 = QColor(255, 225, 0);
                    c4 = QColor(255, 225, 0);
                }
            }
            int startx = w / 5 - 2;
            int stopx = 2 * w / 5 - 4;
            int starty = h / 5 - 2;
            int stopy = 2 * h / 5 - 3;

            // swap track indicator assignment if element is rotated
            // straight track (1)
            p.setPen(QPen(c1, 3, Qt::SolidLine, Qt::RoundCap,
                        Qt::MiterJoin));
            p.drawLine(startx, h /2, stopx, h / 2);
            // straight track (2)
            p.setPen(QPen(c2, 3, Qt::SolidLine, Qt::RoundCap,
                        Qt::MiterJoin));
            p.drawLine(w - startx - 1, h / 2, w - stopx - 1, h / 2);
            // diagonal track (3)
            p.setPen(QPen(c3, 3, Qt::SolidLine, Qt::RoundCap,
                        Qt::MiterJoin));
            p.drawLine(startx, starty, stopx, stopy);
            // diagonal track (4)
            p.setPen(QPen(c4, 3, Qt::SolidLine, Qt::RoundCap,
                        Qt::MiterJoin));
            p.drawLine(w - startx - 1, h - starty - 1, w - stopx - 1,
                    h - stopy - 1);

            p.setPen(QPen(Qt::black, 1));
        }

        // paint track button
        p.drawPixmap(w / 2 - 4, h / 2 - 3, QPixmap(button_black_xpm));

        // paint text label
        p.setPen(Qt::black);
        if (sSoldText != "-1" && !sSoldText.isEmpty()) {
            QFont f(QApplication::font());
            f.setPointSize(QApplication::font().pointSize() - 3);
            p.setFont(f);
            QFontMetrics fm(f);
            QString s;

            if (pref.addresslabeling)
                s.setNum(address1);
            else
                s = sSoldText;

            QRect br = fm.boundingRect(s);
            br.setWidth(br.width() + 4);
            br.setHeight(br.height() - 2);

            br.moveTopRight(QPoint(w - 3, 2));

            p.fillRect(br, QBrush(Qt::white));
            p.setBrush(Qt::white);
            br.setX(br.x() + 1);
            br.setY(br.y() + 1);
            br.setHeight(br.height() - 2);

            p.drawText(br, Qt::AlignCenter | Qt::TextSingleLine |
                    Qt::TextDontClip, s);
        }
    }

    // double slip switch left
    else if (classid == siciDl1) {
        QPainter p(&background);
        p.setRenderHint(QPainter::Antialiasing);

        int w = background.width();
        int h = background.height();

        // paint track
        p.setPen(QPen(Qt::black, 7));
        p.drawLine(0, h / 2, w, h / 2);
        p.drawLine(0, h - 1, w, 0);

        // paint drive symbol
        p.setPen(QPen(Qt::black, 1));
        p.drawLine(w / 2 + 1, h / 2 + 5, w / 2 + 7, h / 2 + 5);
        p.drawLine(w / 2 + 2, h / 2 + 5, w / 2 - 3, h / 2 + 7);
        p.drawLine(w / 2 - 7, h / 2 - 5, w / 2 - 2, h / 2 - 5);
        p.drawLine(w / 2 - 2, h / 2 - 5, w / 2 + 3, h / 2 - 8);

        // paint lock lights TODO: lock only one
        if (lockCounter == 0)
            p.setBrush(Qt::darkGray);
        else
            p.setBrush(QColor(255, 225, 0));

        p.drawEllipse(w / 2 + 4, h - 11, 4, 4);
        p.drawEllipse(w / 4, 6, 4, 4);

        // paint track lights
        if (trackindicatoroff) {
            for (int i = 0; i < 7; ++i)
                p.fillRect(4 + 7 * i, h / 2 - 2, 5, 5,
                        QBrush(Qt::lightGray));
            p.save();
            p.translate(w / 2, h / 2);

            p.rotate(-SANGLE);
            for (int i = 0; i < 7; ++i)
                p.fillRect(-23 + 7 * i, -2, 5, 5, QBrush(Qt::lightGray));
            p.restore();
        }

        //TODO: optimize, color selection is the same for DKL and DKR
        /**
         * lastdir  currentdir  newdir  c1  c2  c3  c4
         * -------------------------------------------
         *     0       0/0        0     -   -   -   -
         *     1       1/1        1     -   -   -   -
         *     2       2/2        2     -   -   -   -
         *     3       3/3        3     -   -   -   -
         *
         *     0       0/1        1     -   1   -   2
         *     1       1/0        0     -   2   -   1
         *
         *     0       0/2        2     1   1   2   2
         *     2       2/0        0     2   2   1   1
         *
         *     0       0/3        3     1   -   2   -
         *     3       3/0        0     2   -   1   -
         *
         *     1       1/2        2     1   -   2   -
         *     2       2/1        1     2   -   1   -
         *
         *     1       1/3        3     1   2   2   1
         *     3       3/1        1     2   1   1   2
         *
         *     2       2/3        3     -   2   -   1
         *     3       3/2        2     -   1   -   2
         * -------------------------------------------
         **/
        else {
            QColor c1, c2, c3, c4;
            if (state == 0) {
                if (!lightson && (
                            (lastdir == 0 && newdir >= 2)
                            ||
                            (lastdir >= 2 && newdir == 0)
                            )
                   )
                    c1 = QColor(Qt::darkGray);
                else {
                    if (occupied)
                        c1 = QColor(Qt::red);
                    else {
                        if ((blinkcounter > 0) || tablelight || routed
                                || (lockCounter > 0))
                            c1 = QColor(255, 225, 0);
                        else
                            c1 = QColor(Qt::darkGray);
                    }
                }

                if (!lightson && (
                            ((lastdir == 1 || lastdir == 2) && newdir == 0)
                            ||
                            (lastdir == 0 && (newdir == 1 || newdir == 2))
                            )
                   )
                    c2 = QColor(Qt::darkGray);
                else {
                    if (occupied)
                        c2 = QColor(Qt::red);
                    else {
                        if ((blinkcounter > 0) || tablelight || routed
                                || (lockCounter > 0))
                            c2 = QColor(255, 225, 0);
                        else
                            c2 = QColor(Qt::darkGray);
                    }
                }

                c3 = QColor(Qt::darkGray);
                c4 = QColor(Qt::darkGray);
            }
            else if (state == 1) {
                if (!lightson && (
                            (lastdir == 1 && (newdir == 2 || newdir == 3))
                            ||
                            ((lastdir == 2 || lastdir == 3) && newdir == 1)
                            )
                   )
                    c1 = QColor(Qt::darkGray);
                else {
                    if (occupied)
                        c1 = QColor(Qt::red);
                    else {
                        if ((blinkcounter > 0) || tablelight || routed
                                || (lockCounter > 0))
                            c1 = QColor(255, 225, 0);
                        else
                            c1 = QColor(Qt::darkGray);
                    }
                }

                c2 = QColor(Qt::darkGray);
                c3 = QColor(Qt::darkGray);

                if (!lightson && (
                            (lastdir == 1 && (newdir == 0 || newdir == 3))
                            ||
                            ((lastdir == 0 || lastdir == 3) && newdir == 1)
                            )
                   )
                    c4 = QColor(Qt::darkGray);
                else {
                    if (occupied)
                        c4 = QColor(Qt::red);
                    else {
                        if ((blinkcounter > 0) || tablelight || routed
                                || (lockCounter > 0))
                            c4 = QColor(255, 225, 0);
                        else
                            c4 = QColor(Qt::darkGray);
                    }
                }
            }
            else if (state == 2) {
                c1 = QColor(Qt::darkGray);
                c2 = QColor(Qt::darkGray);

                if (!lightson && (
                            (lastdir == 2 && (newdir == 0 || newdir == 1))
                            ||
                            ((lastdir == 0 || lastdir == 1) && newdir == 2)
                            )
                   )
                    c3 = QColor(Qt::darkGray);
                else {
                    if (occupied)
                        c3 = QColor(Qt::red);
                    else {
                        if ((blinkcounter > 0) || tablelight || routed
                                || (lockCounter > 0))
                            c3 = QColor(255, 225, 0);
                        else
                            c3 = QColor(Qt::darkGray);
                    }
                }

                if (!lightson && (
                            (lastdir == 2 && (newdir == 0 || newdir == 3))
                            ||
                            ((lastdir == 0 || lastdir == 3) && newdir == 2)
                            )
                   )
                    c4 = QColor(Qt::darkGray);
                else {
                    if (occupied)
                        c4 = QColor(Qt::red);
                    else {
                        if ((blinkcounter > 0) || tablelight || routed
                                || (lockCounter > 0))
                            c4 = QColor(255, 225, 0);
                        else
                            c4 = QColor(Qt::darkGray);
                    }
                }
            }
            else if (state == 3) {
                c1 = QColor(Qt::darkGray);

                if (!lightson && (
                            (lastdir == 3 && (newdir == 1 || newdir == 2))
                            ||
                            ((lastdir == 1 || lastdir == 2) && newdir == 3)
                            )
                   )
                    c2 = QColor(Qt::darkGray);
                else {
                    if (occupied)
                        c2 = QColor(Qt::red);
                    else {
                        if ((blinkcounter > 0) || tablelight || routed
                                || (lockCounter > 0))
                            c2 = QColor(255, 225, 0);
                        else
                            c2 = QColor(Qt::darkGray);
                    }
                }

                if (!lightson && (
                            (lastdir == 3 && (newdir == 0 || newdir == 1))
                            ||
                            ((lastdir == 0 || lastdir == 1) && newdir == 3)
                            )
                   )
                    c3 = QColor(Qt::darkGray);
                else {
                    if (occupied)
                        c3 = QColor(Qt::red);
                    else {
                        if ((blinkcounter > 0) || tablelight || routed
                                || (lockCounter > 0))
                            c3 = QColor(255, 225, 0);
                        else
                            c3 = QColor(Qt::darkGray);
                    }
                }

                c4 = QColor(Qt::darkGray);
            }
            // error indication
            else {
                c1 = QColor(255, 225, 0);
                c2 = QColor(255, 225, 0);
                c3 = QColor(255, 225, 0);
                c4 = QColor(255, 225, 0);
            }

            int startx = w / 5 - 2;
            int stopx = 2 * w / 5 - 4;
            int starty = h / 5 - 2;
            int stopy = 2 * h / 5 - 3;

            // straight track (1)
            p.setPen(QPen(c1, 3, Qt::SolidLine, Qt::RoundCap,
                        Qt::MiterJoin));
            p.drawLine(startx, h /2, stopx, h / 2);
            // straight track (2)
            p.setPen(QPen(c2, 3, Qt::SolidLine, Qt::RoundCap,
                        Qt::MiterJoin));
            p.drawLine(w - startx - 1, h / 2, w - stopx - 1, h / 2);
            // diagonal track (3)
            p.setPen(QPen(c3, 3, Qt::SolidLine, Qt::RoundCap,
                        Qt::MiterJoin));
            p.drawLine(startx, h - starty - 1, stopx, h - stopy - 1);
            // diagonal track (4)
            p.setPen(QPen(c4, 3, Qt::SolidLine, Qt::RoundCap,
                        Qt::MiterJoin));
            p.drawLine(w - startx - 1, starty, w - stopx - 1, stopy);

            p.setPen(QPen(Qt::black, 1));
        }

        // paint track button
        p.drawPixmap(w / 2 - 4, h / 2 - 3, QPixmap(button_black_xpm));

        // paint text label
        p.setPen(Qt::black);
        if (sSoldText != "-1" && !sSoldText.isEmpty()) {
            QFont f(QApplication::font());
            f.setPointSize(QApplication::font().pointSize() - 3);
            p.setFont(f);
            QFontMetrics fm(f);
            QString s;

            if (pref.addresslabeling)
                s.setNum(address1);
            else
                s = sSoldText;

            QRect br = fm.boundingRect(s);
            br.setWidth(br.width() + 4);
            br.setHeight(br.height() - 2);
            br.moveBottomRight(QPoint(w - 3, h - 3));
            p.fillRect(br, QBrush(Qt::white));
            p.setBrush(Qt::white);
            br.setX(br.x() + 1);
            br.setY(br.y() + 1);
            br.setHeight(br.height() - 2);

            p.drawText(br, Qt::AlignCenter | Qt::TextSingleLine |
                    Qt::TextDontClip, s);
        }
    }

    // double slip switch right
    else if (classid == siciDr1) {
        QPainter p(&background);
        p.setRenderHint(QPainter::Antialiasing);

        int w = background.width();
        int h = background.height();

        // paint track
        p.setPen(QPen(Qt::black, 7));
        p.drawLine(0, h / 2, w, h / 2);
        p.drawLine(0, 0, w, h - 1);

        // paint drive symbol
        p.setPen(QPen(Qt::black, 1));
        p.drawLine(w / 2 - 1, h / 2 + 5, w / 2 - 7, h / 2 + 5);
        p.drawLine(w / 2 - 3, h / 2 + 5, w / 2 + 2, h / 2 + 7);
        p.drawLine(w / 2 + 7, h / 2 - 5, w / 2 + 2, h / 2 - 5);
        p.drawLine(w / 2 + 2, h / 2 - 5, w / 2 - 3, h / 2 - 8);

        // paint lock light
        if (lockCounter == 0)
            p.setBrush(Qt::darkGray);
        else
            p.setBrush(QColor(255, 225, 0));

        p.drawEllipse(w / 2 - 8, h - 11, 4, 4);
        p.drawEllipse(3 * w/ 4 - 2 - 2  , 6, 4, 4);

        // paint track lights
        if (trackindicatoroff) {
            for (int i = 0; i < 7; ++i)
                p.fillRect(4 + 7 * i, h / 2 - 2, 5, 5,
                        QBrush(Qt::lightGray));
            p.save();
            p.translate(w / 2, h / 2);

            p.rotate(SANGLE);
            for (int i = 0; i < 7; ++i)
                p.fillRect(-23 + 7 * i, -2, 5, 5, QBrush(Qt::lightGray));
            p.restore();
        }

        /**
         * lastdir  currentdir  newdir  c1  c2  c3  c4
         * -------------------------------------------
         *     0       0/0        0     -   -   -   -
         *     1       1/1        1     -   -   -   -
         *     2       2/2        2     -   -   -   -
         *     3       3/3        3     -   -   -   -
         *
         *     0       0/1        1     -   1   -   2
         *     1       1/0        0     -   2   -   1
         *
         *     0       0/2        2     1   1   2   2
         *     2       2/0        0     2   2   1   1
         *
         *     0       0/3        3     1   -   2   -
         *     3       3/0        0     2   -   1   -
         *
         *     1       1/2        2     1   -   2   -
         *     2       2/1        1     2   -   1   -
         *
         *     1       1/3        3     1   2   2   1
         *     3       3/1        1     2   1   1   2
         *
         *     2       2/3        3     -   2   -   1
         *     3       3/2        2     -   1   -   2
         * -------------------------------------------
         **/
        else {
            QColor c1, c2, c3, c4;
            if (state == 0) {
                if (!lightson && (
                            (lastdir == 0 && newdir >= 2)
                            ||
                            (lastdir >= 2 && newdir == 0)
                            )
                   )
                    c1 = QColor(Qt::darkGray);
                else {
                    if (occupied)
                        c1 = QColor(Qt::red);
                    else {
                        if ((blinkcounter > 0) || tablelight || routed
                                || (lockCounter > 0))
                            c1 = QColor(255, 225, 0);
                        else
                            c1 = QColor(Qt::darkGray);
                    }
                }

                if (!lightson && (
                            ((lastdir == 1 || lastdir == 2) && newdir == 0)
                            ||
                            (lastdir == 0 && (newdir == 1 || newdir == 2))
                            )
                   )
                    c2 = QColor(Qt::darkGray);
                else {
                    if (occupied)
                        c2 = QColor(Qt::red);
                    else {
                        if ((blinkcounter > 0) || tablelight || routed
                                || (lockCounter > 0))
                            c2 = QColor(255, 225, 0);
                        else
                            c2 = QColor(Qt::darkGray);
                    }
                }

                c3 = QColor(Qt::darkGray);
                c4 = QColor(Qt::darkGray);
            }
            else if (state == 1) {
                if (!lightson && (
                            (lastdir == 1 && (newdir == 2 || newdir == 3))
                            ||
                            ((lastdir == 2 || lastdir == 3) && newdir == 1)
                            )
                   )
                    c1 = QColor(Qt::darkGray);
                else {
                    if (occupied)
                        c1 = QColor(Qt::red);
                    else {
                        if ((blinkcounter > 0) || tablelight || routed
                                || (lockCounter > 0))
                            c1 = QColor(255, 225, 0);
                        else
                            c1 = QColor(Qt::darkGray);
                    }
                }

                c2 = QColor(Qt::darkGray);
                c3 = QColor(Qt::darkGray);

                if (!lightson && (
                            (lastdir == 1 && (newdir == 0 || newdir == 3))
                            ||
                            ((lastdir == 0 || lastdir == 3) && newdir == 1)
                            )
                   )
                    c4 = QColor(Qt::darkGray);
                else {
                    if (occupied)
                        c4 = QColor(Qt::red);
                    else {
                        if ((blinkcounter > 0) || tablelight || routed
                                || (lockCounter > 0))
                            c4 = QColor(255, 225, 0);
                        else
                            c4 = QColor(Qt::darkGray);
                    }
                }
            }
            else if (state == 2) {
                c1 = QColor(Qt::darkGray);
                c2 = QColor(Qt::darkGray);

                if (!lightson && (
                            (lastdir == 2 && (newdir == 0 || newdir == 1))
                            ||
                            ((lastdir == 0 || lastdir == 1) && newdir == 2)
                            )
                   )
                    c3 = QColor(Qt::darkGray);
                else {
                    if (occupied)
                        c3 = QColor(Qt::red);
                    else {
                        if ((blinkcounter > 0) || tablelight || routed
                                || (lockCounter > 0))
                            c3 = QColor(255, 225, 0);
                        else
                            c3 = QColor(Qt::darkGray);
                    }
                }

                if (!lightson && (
                            (lastdir == 2 && (newdir == 0 || newdir == 3))
                            ||
                            ((lastdir == 0 || lastdir == 3) && newdir == 2)
                            )
                   )
                    c4 = QColor(Qt::darkGray);
                else {
                    if (occupied)
                        c4 = QColor(Qt::red);
                    else {
                        if ((blinkcounter > 0) || tablelight || routed
                                || (lockCounter > 0))
                            c4 = QColor(255, 225, 0);
                        else
                            c4 = QColor(Qt::darkGray);
                    }
                }
            }
            else if (state == 3) {
                c1 = QColor(Qt::darkGray);

                if (!lightson && (
                            (lastdir == 3 && (newdir == 1 || newdir == 2))
                            ||
                            ((lastdir == 1 || lastdir == 2) && newdir == 3)
                            )
                   )
                    c2 = QColor(Qt::darkGray);
                else {
                    if (occupied)
                        c2 = QColor(Qt::red);
                    else {
                        if ((blinkcounter > 0) || tablelight || routed
                                || (lockCounter > 0))
                            c2 = QColor(255, 225, 0);
                        else
                            c2 = QColor(Qt::darkGray);
                    }
                }

                if (!lightson && (
                            (lastdir == 3 && (newdir == 0 || newdir == 1))
                            ||
                            ((lastdir == 0 || lastdir == 1) && newdir == 3)
                            )
                   )
                    c3 = QColor(Qt::darkGray);
                else {
                    if (occupied)
                        c3 = QColor(Qt::red);
                    else {
                        if ((blinkcounter > 0) || tablelight || routed
                                || (lockCounter > 0))
                            c3 = QColor(255, 225, 0);
                        else
                            c3 = QColor(Qt::darkGray);
                    }
                }

                c4 = QColor(Qt::darkGray);
            }
            // error indication
            else {
                c1 = QColor(255, 225, 0);
                c2 = QColor(255, 225, 0);
                c3 = QColor(255, 225, 0);
                c4 = QColor(255, 225, 0);
            }

            int startx = w / 5 - 2;
            int stopx = 2 * w / 5 - 4;
            int starty = h / 5 - 2;
            int stopy = 2 * h / 5 - 3;

            // straight track (1)
            p.setPen(QPen(c1, 3, Qt::SolidLine, Qt::RoundCap,
                        Qt::MiterJoin));
            p.drawLine(w - startx - 1, h / 2, w - stopx - 1, h / 2);
            // straight track (2)
            p.setPen(QPen(c2, 3, Qt::SolidLine, Qt::RoundCap,
                        Qt::MiterJoin));
            p.drawLine(startx, h /2, stopx, h / 2);
            // diagonal track (3)
            p.setPen(QPen(c3, 3, Qt::SolidLine, Qt::RoundCap,
                        Qt::MiterJoin));
            p.drawLine(w - startx - 1, h - starty - 1, w - stopx - 1,
                    h - stopy - 1);
            // diagonal track (4)
            p.setPen(QPen(c4, 3, Qt::SolidLine, Qt::RoundCap,
                        Qt::MiterJoin));
            p.drawLine(startx, starty, stopx, stopy);

            p.setPen(QPen(Qt::black, 1));
        }

        // paint track button
        p.drawPixmap(w / 2 - 4, h / 2 - 3, QPixmap(button_black_xpm));

        // paint text label
        p.setPen(Qt::black);
        if (sSoldText != "-1" && !sSoldText.isEmpty()) {
            QFont f(QApplication::font());
            f.setPointSize(QApplication::font().pointSize() - 3);
            p.setFont(f);
            QFontMetrics fm(f);
            QString s;

            if (pref.addresslabeling)
                s.setNum(address1);
            else
                s = sSoldText;

            QRect br = fm.boundingRect(s);
            br.setWidth(br.width() + 4);
            br.setHeight(br.height() - 2);
            br.moveBottomLeft(QPoint(2, h - 3));
            p.fillRect(br, QBrush(Qt::white));
            p.setBrush(Qt::white);
            br.setX(br.x() + 1);
            br.setY(br.y() + 1);
            br.setHeight(br.height() - 2);

            p.drawText(br, Qt::AlignCenter | Qt::TextSingleLine |
                    Qt::TextDontClip, s);
        }
    }

    // right track with normal route button
    else if (classid == siciZt1 || classid == siciRt1) {
        QPainter p(&background);

        int w = background.width();
        int h = background.height();

        // paint track
        p.fillRect(0, h / 2 - 3, w, 7, QBrush(Qt::black));

        // paint track lights
        if (trackindicatoroff) {
            for (int i = 0; i < 7; ++i)
                p.fillRect(4 + 7 * i, h / 2 - 2, 5, 5,
                        QBrush(Qt::lightGray));
        }
        else {
            QColor c;
            if (occupied)
                c = QColor(Qt::red);
            else {
                if (routed)
                    c = QColor(255, 225, 0);
                else
                    c = QColor(Qt::darkGray);
            }
            p.fillRect(w / 3, h / 2 - 1, w / 3, 3, QBrush(c));
        }

        // paint track button
        if (classid == siciZt1)
            p.drawPixmap(5 * w / 6 - 4 , h / 2 - 3,
                    QPixmap(button_red_xpm));
        else
            p.drawPixmap(5 * w / 6 - 4 , h / 2 - 3,
                    QPixmap(button_gray_xpm));

        // paint text label
        if (sSoldText != "-1" && !sSoldText.isEmpty()) {
            QFont f(QApplication::font());
            f.setPointSize(QApplication::font().pointSize() - 3);
            p.setFont(f);
            QFontMetrics fm(f);
            QString s;

            if (pref.addresslabeling)
                s.setNum(address1);
            else
                s = sSoldText;

            QRect br = fm.boundingRect(s);
            br.setWidth(br.width() + 4);
            br.setHeight(br.height() - 2);

            br.moveBottomRight(QPoint(w / 2 + br.width()/2, h - 3));

            p.fillRect(br, QBrush(Qt::white));
            p.drawText(br, Qt::AlignCenter | Qt::TextSingleLine |
                    Qt::TextDontClip, s);
        }
    }

    // left track with normal route button
    else if (classid == siciZt3 || classid == siciRt3) {
        QPainter p(&background);

        int w = background.width();
        int h = background.height();

        // paint track
        p.fillRect(0, h / 2 - 3, w, 7, QBrush(Qt::black));

        // paint track lights
        if (trackindicatoroff) {
            for (int i = 0; i < 7; ++i)
                p.fillRect(4 + 7 * i, h / 2 - 2, 5, 5,
                        QBrush(Qt::lightGray));
        }
        else {
            QColor c;
            if (occupied)
                c = QColor(Qt::red);
            else {
                if (routed)
                    c = QColor(255, 225, 0);
                else
                    c = QColor(Qt::darkGray);
            }
            p.fillRect(w / 3, h / 2 - 1, w / 3, 3, QBrush(c));
        }

        // paint track button
            if (classid == siciZt3)
                p.drawPixmap(w / 6  - 4, h / 2 - 3,
                        QPixmap(button_red_xpm));
            else
                p.drawPixmap(w / 6  - 4, h / 2 - 3,
                        QPixmap(button_gray_xpm));

        // paint text label
        if (sSoldText != "-1" && !sSoldText.isEmpty()) {
            QFont f(QApplication::font());
            f.setPointSize(QApplication::font().pointSize() - 3);
            p.setFont(f);
            QFontMetrics fm(f);
            QString s;

            if (pref.addresslabeling)
                s.setNum(address1);
            else
                s = sSoldText;

            QRect br = fm.boundingRect(s);
            br.setWidth(br.width() + 4);
            br.setHeight(br.height() - 2);

            br.moveTopLeft(QPoint(w / 2 - br.width()/2, 2));

            p.fillRect(br, QBrush(Qt::white));
            p.drawText(br, Qt::AlignCenter | Qt::TextSingleLine |
                    Qt::TextDontClip, s);
        }
    }

    // right shunting signals SS, SSH, SSS
    else if (classid == siciSs1 || classid == siciSh1 ||
            classid == siciSd1) {
        QPainter p(&background);

        int w = background.width();
        int h = background.height();

        // paint track
        p.fillRect(0, h / 2 - 3, w, 7, QBrush(Qt::black));

        // paint track lights
        if (trackindicatoroff) {
            for (int i = 0; i < 7; ++i)
                p.fillRect(4 + 7 * i, h / 2 - 2, 5, 5, QBrush(Qt::lightGray));
        }
        else {
            QColor c;
            if (occupied)
                c = QColor(Qt::red);
            else {
                if (routed)
                    c = QColor(255, 225, 0);
                else
                    c = QColor(Qt::darkGray);
            }
            p.fillRect(w / 3 + 1, h / 2 - 1, w / 3, 3, QBrush(c));
        }

        // paint track button
        int xpos1 = w / 6  - 4;
        int xpos2 = 5 * w / 6 - 4;

        if (classid == siciSd1) {
            p.drawPixmap(xpos1, h / 2 - 3, QPixmap(button_gray_xpm));
            p.drawPixmap(xpos2, h / 2 - 3, QPixmap(button_gray_xpm));
        }

        xpos1 = xpos2;

        if (classid == siciSh1)
            p.drawPixmap(xpos1, h / 2 - 3, QPixmap(button_red_xpm));

        else if (classid == siciSs1) {
            p.drawPixmap(xpos1, h / 2 - 3, QPixmap(button_gray_xpm));
        }

        // paint signal icon
        p.setBrush(Qt::black);

        p.save();
        p.translate(w - 1, h - 1);
        p.rotate(180.0);
        p.drawRect(5, 5, 14, 4);
        p.drawLine(6, 4, 18, 4);
        p.drawLine(6, 10, 18, 10);
        p.drawLine(5 + 15, 7, 6 + 16, 7);
        p.drawLine(7 + 16, 5, 7 + 16, 9);
        p.drawLine(7 + 17, 5, 7 + 17, 9);

        // paint signal light
        // SH0
        if (state == 0) {
            p.setPen(QPen(Qt::red));
            p.drawLine(7, 5, 7, 9);
            p.drawLine(8, 5, 8, 9);
        }
        // SH1
        else {
            p.setPen(Qt::yellow);
            p.drawLine(14, 5, 18, 9);
            p.drawLine(13, 5, 17, 9);
        }
        p.setPen(QPen(Qt::black));

        p.restore();

        // paint lock light
        if (lockCounter == 0)
            p.setBrush(Qt::darkGray);
        else
            p.setBrush(QColor(255, 225, 0));

        p.drawEllipse(w / 2 - 4, h - 10, 4, 4);

        // paint FfM
        if (ffm) {
            p.setBrush(ffmactive ? Qt::yellow : Qt::darkGray);
            p.drawRect(w - 11, 4, 5, 5);
        }

        // paint text label
        if (sSoldText != "-1" && !sSoldText.isEmpty()) {
            QFont f(QApplication::font());
            f.setPointSize(QApplication::font().pointSize() - 3);
            p.setFont(f);
            QFontMetrics fm(f);
            QString s;

            if (pref.addresslabeling)
                s.setNum(address1);
            else
                s = sSoldText;

            QRect br = fm.boundingRect(s);
            br.setWidth(br.width() + 4);
            br.setHeight(br.height() - 2);

            br.moveTopLeft(QPoint(w / 2 - br.width()/2, 2));

            p.fillRect(br, QBrush(Qt::white));
            p.setBrush(Qt::white);
            br.setX(br.x() + 1);
            br.setY(br.y() + 1);
            br.setHeight(br.height() - 2);

            p.drawText(br, Qt::AlignCenter | Qt::TextSingleLine |
                    Qt::TextDontClip, s);
        }
    }

    // left shunting signals SS, SSH, SSS
    else if (classid == siciSs3 || classid == siciSh3 ||
            classid == siciSd3) {
        QPainter p(&background);

        int w = background.width();
        int h = background.height();

        // paint track
        p.fillRect(0, h / 2 - 3, w, 7, QBrush(Qt::black));

        // paint track lights
        if (trackindicatoroff) {
            for (int i = 0; i < 7; ++i)
                p.fillRect(4 + 7 * i, h / 2 - 2, 5, 5, QBrush(Qt::lightGray));
        }
        else {
            QColor c;
            if (occupied)
                c = QColor(Qt::red);
            else {
                if (routed)
                    c = QColor(255, 225, 0);
                else
                    c = QColor(Qt::darkGray);
            }
            p.fillRect(w / 3 + 1, h / 2 - 1, w / 3, 3, QBrush(c));
        }

        // paint track button
        int xpos1 = w / 6  - 4;
        int xpos2 = 5 * w / 6 - 4;

        if (classid == siciSd3) {
            p.drawPixmap(xpos1, h / 2 - 3, QPixmap(button_gray_xpm));
            p.drawPixmap(xpos2, h / 2 - 3, QPixmap(button_gray_xpm));
        }

        if (classid == siciSh3)
            p.drawPixmap(xpos1, h / 2 - 3, QPixmap(button_red_xpm));

        else if (classid == siciSs3) {
            p.drawPixmap(xpos1, h / 2 - 3, QPixmap(button_gray_xpm));
        }

        // paint signal icon
        p.setBrush(Qt::black);

        p.drawRect(5, 5, 14, 4);
        p.drawLine(6, 4, 18, 4);
        p.drawLine(6, 10, 18, 10);
        p.drawLine(5 + 15, 7, 6 + 16, 7);
        p.drawLine(7 + 16, 5, 7 + 16, 9);
        p.drawLine(7 + 17, 5, 7 + 17, 9);

        // paint signal light
        // SH0
        if (state == 0) {
            p.setPen(QPen(Qt::red));
            p.drawLine(7, 5, 7, 9);
            p.drawLine(8, 5, 8, 9);
        }
        // SH1
        else {
            p.setPen(Qt::yellow);
            p.drawLine(14, 5, 18, 9);
            p.drawLine(13, 5, 17, 9);
        }
        p.setPen(QPen(Qt::black));


        // paint lock light
        if (lockCounter == 0)
            p.setBrush(Qt::darkGray);
        else
            p.setBrush(QColor(255, 225, 0));

        p.drawEllipse(w / 2 - 1, 5, 4, 4);

        // paint FfM
        if (ffm) {
            p.setBrush(ffmactive ? Qt::yellow : Qt::darkGray);
            p.drawRect(5, h - 10, 5, 5);
        }

        // paint text label
        if (sSoldText != "-1" && !sSoldText.isEmpty()) {
            QFont f(QApplication::font());
            f.setPointSize(QApplication::font().pointSize() - 3);
            p.setFont(f);
            QFontMetrics fm(f);
            QString s;

            if (pref.addresslabeling)
                s.setNum(address1);
            else
                s = sSoldText;

            QRect br = fm.boundingRect(s);
            br.setWidth(br.width() + 4);
            br.setHeight(br.height() - 2);

            br.moveBottomRight(QPoint(w / 2 + br.width()/2, h - 3));

            p.fillRect(br, QBrush(Qt::white));
            p.setBrush(Qt::white);
            br.setX(br.x() + 1);
            br.setY(br.y() + 1);
            br.setHeight(br.height() - 2);

            p.drawText(br, Qt::AlignCenter | Qt::TextSingleLine |
                    Qt::TextDontClip, s);
        }
    }

    // switch left top (+ old left bottom)
    else if (classid == siciTl1) {
        QPainter p(&background);
        p.setRenderHint(QPainter::Antialiasing);

        int w = background.width();
        int h = background.height();

        // paint track
        p.fillRect(0, h / 2 - 3, w, 7, QBrush(Qt::black));

        p.save();
        p.translate(w / 2, h / 2);

        p.rotate(-SANGLE);

        p.setPen(QPen(Qt::black, 7));
        p.drawLine(1, 1, w / 2 + 5, 0);
        p.restore();

        // paint track lights
        if (trackindicatoroff) {
            for (int i = 0; i < 7; ++i)
                p.fillRect(4 + 7 * i, h / 2 - 2, 5, 5, QBrush(Qt::lightGray));
            // short track
            p.save();
            p.translate(w / 2, h / 2);

            p.rotate(-SANGLE);

            for (int i = 0; i < 3; ++i)
                p.fillRect(6 + 7 * i, -2, 5, 5, QBrush(Qt::lightGray));
            p.restore();

        }
        else {
            // first light
            QColor c;
            if (occupied)
                c = QColor(Qt::red);
            else {
                if (routed)
                    c = QColor(255, 225, 0);
                else
                    c = QColor(Qt::darkGray);
            }
            p.setPen(QPen(c, 3, Qt::SolidLine, Qt::RoundCap, Qt::MiterJoin));

            p.drawLine(5, h / 2 , 19,  h / 2);

            // second light
            if (state == 0 && lightson)
                if (occupied)
                    c = QColor(Qt::red);
                else {
                    if ((blinkcounter > 0) || tablelight || routed || (lockCounter > 0))
                        c = QColor(255, 225, 0);
                    else
                        c = QColor(Qt::darkGray);
                }
            else
                c = QColor(Qt::darkGray);

            p.setPen(QPen(c, 3, Qt::SolidLine, Qt::RoundCap, Qt::MiterJoin));

            p.drawLine(w - 5, h / 2 , w - 19,  h / 2);

            // third light
            if (state == 1 && lightson)
                if (occupied)
                    c = QColor(Qt::red);
                else {
                    if ((blinkcounter > 0) || tablelight || routed || (lockCounter > 0))
                        c = QColor(255, 225, 0);
                    else
                        c = QColor(Qt::darkGray);
                }
            else
                c = QColor(Qt::darkGray);

            p.setPen(QPen(c, 3, Qt::SolidLine, Qt::RoundCap, Qt::MiterJoin));

            p.save();
            p.translate(w / 2, h / 2);

            p.rotate(-SANGLE);

            p.drawLine(11, 0, 11 + 14, 0);
            p.restore();

            p.setPen(QPen(Qt::black));
        }

        // paint track button
        p.drawPixmap(w / 2 - 4, h / 2 - 3, QPixmap(button_black_xpm));

        // paint lock light
        if (lockCounter == 0)
            p.setBrush(Qt::darkGray);
        else
            p.setBrush(QColor(255, 225, 0));

        p.drawEllipse(w / 2 - 2, h - 11, 4, 4);

        // paint text label
        if (sSoldText != "-1" && !sSoldText.isEmpty()) {
            QFont f(QApplication::font());
            f.setPointSize(QApplication::font().pointSize() - 3);
            p.setFont(f);
            QFontMetrics fm(f);
            QString s;

            if (pref.addresslabeling)
                s.setNum(address1);
            else
                s = sSoldText;

            QRect br = fm.boundingRect(s);
            br.setWidth(br.width() + 4);
            br.setHeight(br.height() - 2);

            br.moveBottomLeft(QPoint(2, h - 3));

            p.fillRect(br, QBrush(Qt::white));
            p.setBrush(Qt::white);
            br.setX(br.x() + 1);
            br.setY(br.y() + 1);
            br.setHeight(br.height() - 2);

            p.drawText(br, Qt::AlignCenter | Qt::TextSingleLine |
                    Qt::TextDontClip, s);
        }
    }

    // switch left bottom
    else if (classid == siciTl3) {
        QPainter p(&background);
        p.setRenderHint(QPainter::Antialiasing);

        int w = background.width();
        int h = background.height();

        // paint track
        p.fillRect(0, h / 2 - 3, w, 7, QBrush(Qt::black));

        p.save();
        p.translate(w / 2, h / 2);

        p.rotate(WANGLE);

        p.setPen(QPen(Qt::black, 7));
        p.drawLine(2, 0, w / 2 + 5, 0);
        p.restore();

        // paint track lights
        if (trackindicatoroff) {
            for (int i = 0; i < 7; ++i)
                p.fillRect(4 + 7 * i, h / 2 - 2, 5, 5, QBrush(Qt::lightGray));
            // short track
            p.save();
            p.translate(w / 2, h / 2);

            p.rotate(WANGLE);

            for (int i = 0; i < 3; ++i)
                p.fillRect(6 + 7 * i, -2, 5, 5, QBrush(Qt::lightGray));
            p.restore();

        }
        else {
            // first light
            QColor c;
            if (occupied)
                c = QColor(Qt::red);
            else {
                if (routed)
                    c = QColor(255, 225, 0);
                else
                    c = QColor(Qt::darkGray);
            }
            p.setPen(QPen(c, 3, Qt::SolidLine, Qt::RoundCap, Qt::MiterJoin));

            p.drawLine(w - 5, h / 2 , w - 19,  h / 2);

            // second light
            if (state == 0 && lightson)
                if (occupied)
                    c = QColor(Qt::red);
                else {
                    if ((blinkcounter > 0) || tablelight || routed || (lockCounter > 0))
                        c = QColor(255, 225, 0);
                    else
                        c = QColor(Qt::darkGray);
                }
            else
                c = QColor(Qt::darkGray);

            p.setPen(QPen(c, 3, Qt::SolidLine, Qt::RoundCap, Qt::MiterJoin));

            p.drawLine(5, h / 2 , 19,  h / 2);

            // third light
            if (state == 1 && lightson)
                if (occupied)
                    c = QColor(Qt::red);
                else {
                    if ((blinkcounter > 0) || tablelight || routed || (lockCounter > 0))
                        c = QColor(255, 225, 0);
                    else
                        c = QColor(Qt::darkGray);
                }
            else
                c = QColor(Qt::darkGray);

            p.setPen(QPen(c, 3, Qt::SolidLine, Qt::RoundCap, Qt::MiterJoin));

            p.save();
            p.translate(w / 2, h / 2);

            p.rotate(WANGLE);

            p.drawLine(11, 0, 11 + 14, 0);
            p.restore();

            p.setPen(QPen(Qt::black));
        }

        // paint track button
        p.drawPixmap(w / 2 - 4, h / 2 - 3, QPixmap(button_black_xpm));

        // paint lock light
        if (lockCounter == 0)
            p.setBrush(Qt::darkGray);
        else
            p.setBrush(QColor(255, 225, 0));

        p.drawEllipse(w / 2 - 2, 6, 4, 4);

        // paint text label
        if (sSoldText != "-1" && !sSoldText.isEmpty()) {
            QFont f(QApplication::font());
            f.setPointSize(QApplication::font().pointSize() - 3);
            p.setFont(f);
            QFontMetrics fm(f);
            QString s;

            if (pref.addresslabeling)
                s.setNum(address1);
            else
                s = sSoldText;

            QRect br = fm.boundingRect(s);
            br.setWidth(br.width() + 4);
            br.setHeight(br.height() - 2);

            br.moveTopRight(QPoint(w - 3, 2));

            p.fillRect(br, QBrush(Qt::white));
            p.setBrush(Qt::white);
            br.setX(br.x() + 1);
            br.setY(br.y() + 1);
            br.setHeight(br.height() - 2);

            p.drawText(br, Qt::AlignCenter | Qt::TextSingleLine |
                    Qt::TextDontClip, s);
        }
    }

    // switch right top
    else if (classid == siciTr3) {
        QPainter p(&background);
        p.setRenderHint(QPainter::Antialiasing);

        int w = background.width();
        int h = background.height();

        // paint track
        p.fillRect(0, h / 2 - 3, w, 7, QBrush(Qt::black));

        p.save();
        p.translate(w / 2, h / 2);

        p.rotate(-WANGLE);

        p.setPen(QPen(Qt::black, 7));
        p.drawLine(0, 0, w / 2 + 5, 0);
        p.restore();

        // paint track lights
        if (trackindicatoroff) {
            for (int i = 0; i < 7; ++i)
                p.fillRect(4 + 7 * i, h / 2 - 2, 5, 5, QBrush(Qt::lightGray));
            // short track
            p.save();
            p.translate(w / 2, h / 2);

            p.rotate(-WANGLE);

            for (int i = 0; i < 3; ++i)
                p.fillRect(6 + 7 * i, -2, 5, 5, QBrush(Qt::lightGray));
            p.restore();

        }
        else {
            // first light
            QColor c;
            if (occupied)
                c = QColor(Qt::red);
            else {
                if (routed)
                    c = QColor(255, 225, 0);
                else
                    c = QColor(Qt::darkGray);
            }
            p.setPen(QPen(c, 3, Qt::SolidLine, Qt::RoundCap, Qt::MiterJoin));

            p.drawLine(w - 5, h / 2 , w - 19,  h / 2);

            // second light
            if (state == 0 && lightson)
                if (occupied)
                    c = QColor(Qt::red);
                else {
                    if ((blinkcounter > 0) || tablelight || routed || (lockCounter > 0))
                        c = QColor(255, 225, 0);
                    else
                        c = QColor(Qt::darkGray);
                }
            else
                c = QColor(Qt::darkGray);

            p.setPen(QPen(c, 3, Qt::SolidLine, Qt::RoundCap, Qt::MiterJoin));

            p.drawLine(5, h / 2 , 19,  h / 2);

            // third light
            if (state == 1 && lightson)
                if (occupied)
                    c = QColor(Qt::red);
                else {
                    if ((blinkcounter > 0) || tablelight || routed || (lockCounter > 0))
                        c = QColor(255, 225, 0);
                    else
                        c = QColor(Qt::darkGray);
                }
            else
                c = QColor(Qt::darkGray);

            p.setPen(QPen(c, 3, Qt::SolidLine, Qt::RoundCap, Qt::MiterJoin));

            p.save();
            p.translate(w / 2, h / 2);

            p.rotate(-WANGLE);

            p.drawLine(11, 0, 11 + 14, 0);
            p.restore();

            p.setPen(QPen(Qt::black));
        }

        // paint track button
        p.drawPixmap(w / 2 - 4, h / 2 - 3, QPixmap(button_black_xpm));

        // paint lock light
        if (lockCounter == 0)
            p.setBrush(Qt::darkGray);
        else
            p.setBrush(QColor(255, 225, 0));

        p.drawEllipse(w / 2 - 2, h - 11, 4, 4);

        // paint text label
        if (sSoldText != "-1" && !sSoldText.isEmpty()) {
            QFont f(QApplication::font());
            f.setPointSize(QApplication::font().pointSize() - 3);
            p.setFont(f);
            QFontMetrics fm(f);
            QString s;

            if (pref.addresslabeling)
                s.setNum(address1);
            else
                s = sSoldText;

            QRect br = fm.boundingRect(s);
            br.setWidth(br.width() + 4);
            br.setHeight(br.height() - 2);

            br.moveBottomRight(QPoint(w - 3, h - 3));

            p.fillRect(br, QBrush(Qt::white));
            p.setBrush(Qt::white);
            br.setX(br.x() + 1);
            br.setY(br.y() + 1);
            br.setHeight(br.height() - 2);

            p.drawText(br, Qt::AlignCenter | Qt::TextSingleLine |
                    Qt::TextDontClip, s);
        }
    }

    // switch right bottom (+ old right top)
    else if (classid == siciTr1) {
        QPainter p(&background);
        p.setRenderHint(QPainter::Antialiasing);

        int w = background.width();
        int h = background.height();

        // paint track
        p.fillRect(0, h / 2 - 3, w, 7, QBrush(Qt::black));

        p.save();
        p.translate(w / 2, h / 2);

        p.rotate(SANGLE);

        p.setPen(QPen(Qt::black, 7));
        p.drawLine(1, 1, w / 2 + 4, 0);
        p.restore();

        // paint track lights
        if (trackindicatoroff) {
            for (int i = 0; i < 7; ++i)
                p.fillRect(4 + 7 * i, h / 2 - 2, 5, 5, QBrush(Qt::lightGray));
            // short track
            p.save();
            p.translate(w / 2, h / 2);

            p.rotate(SANGLE);

            for (int i = 0; i < 3; ++i)
                p.fillRect(6 + 7 * i, -2, 5, 5, QBrush(Qt::lightGray));
            p.restore();

        }
        else {
            // first light
            QColor c;
            if (occupied)
                c = QColor(Qt::red);
            else {
                if (routed)
                    c = QColor(255, 225, 0);
                else
                    c = QColor(Qt::darkGray);
            }
            p.setPen(QPen(c, 3, Qt::SolidLine, Qt::RoundCap, Qt::MiterJoin));

            p.drawLine(5, h / 2 , 19,  h / 2);

            // second light
            if (state == 0 && lightson)
                if (occupied)
                    c = QColor(Qt::red);
                else {
                    if ((blinkcounter > 0) || tablelight || routed || (lockCounter > 0))
                        c = QColor(255, 225, 0);
                    else
                        c = QColor(Qt::darkGray);
                }
            else
                c = QColor(Qt::darkGray);

            p.setPen(QPen(c, 3, Qt::SolidLine, Qt::RoundCap, Qt::MiterJoin));

            p.drawLine(w - 5, h / 2 , w - 19,  h / 2);

            // third light
            if (state == 1 && lightson)
                if (occupied)
                    c = QColor(Qt::red);
                else {
                    if ((blinkcounter > 0) || tablelight || routed || (lockCounter > 0))
                        c = QColor(255, 225, 0);
                    else
                        c = QColor(Qt::darkGray);
                }
            else
                c = QColor(Qt::darkGray);

            p.setPen(QPen(c, 3, Qt::SolidLine, Qt::RoundCap, Qt::MiterJoin));

            p.save();
            p.translate(w / 2, h / 2);

            p.rotate(SANGLE);

            p.drawLine(11, 0, 11 + 14, 0);
            p.restore();

            p.setPen(QPen(Qt::black));
        }

        // paint track button
        p.drawPixmap(w / 2 - 4, h / 2 - 3, QPixmap(button_black_xpm));

        // paint lock light
        if (lockCounter == 0)
            p.setBrush(Qt::darkGray);
        else
            p.setBrush(QColor(255, 225, 0));

        p.drawEllipse(w / 2 - 2, 6, 4, 4);

        // paint text label
        if (sSoldText != "-1" && !sSoldText.isEmpty()) {
            QFont f(QApplication::font());
            f.setPointSize(QApplication::font().pointSize() - 3);
            p.setFont(f);
            QFontMetrics fm(f);
            QString s;

            if (pref.addresslabeling)
                s.setNum(address1);
            else
                s = sSoldText;

            QRect br = fm.boundingRect(s);
            br.setWidth(br.width() + 4);
            br.setHeight(br.height() - 2);

            br.moveTopLeft(QPoint(2, 2));

            p.fillRect(br, QBrush(Qt::white));
            p.setBrush(Qt::white);
            br.setX(br.x() + 1);
            br.setY(br.y() + 1);
            br.setHeight(br.height() - 2);

            p.drawText(br, Qt::AlignCenter | Qt::TextSingleLine |
                    Qt::TextDontClip, s);
        }
    }

    // diagonal turnout left top
    else if (classid == siciIl3) {
        QPainter p(&background);
        p.setRenderHint(QPainter::Antialiasing);

        int w = background.width();
        int h = background.height();

        // paint track
        p.fillRect(0, h / 2 - 3, w / 2, 7, QBrush(Qt::black));

        p.save();
        p.translate(w / 2, h / 2);

        p.rotate(-WANGLE);

        p.setPen(QPen(Qt::black, 7));
        p.drawLine(-w / 2 - 5, 0, w / 2 + 5, 0);
        p.restore();

        // paint track lights
        if (trackindicatoroff) {
            // short track
            for (int i = 0; i < 3; ++i)
                p.fillRect(4 + 7 * i, h / 2-2, 5, 5, QBrush(Qt::lightGray));

            // long track
            p.save();
            p.translate(w / 2, h / 2);

            p.rotate(-WANGLE);

            for (int i = 0; i < 3; ++i)
                p.fillRect(7 + 7 * i, - 2, 5, 5, QBrush(Qt::lightGray));

            for (int i = 0; i < 3; ++i)
                p.fillRect(-11 - 7 * i, - 2, 5, 5, QBrush(Qt::lightGray));

            p.restore();

        }
        else {
            QColor c;
            p.save();
            p.translate(w / 2, h / 2);

            p.rotate(SANGLE);

            // first light
            if (occupied)
                c = QColor(Qt::red);
            else {
                if (routed)
                    c = QColor(255, 225, 0);
                else
                    c = QColor(Qt::darkGray);
            }
            p.setPen(QPen(c, 3, Qt::SolidLine, Qt::RoundCap, Qt::MiterJoin));
            p.drawLine(10, 0, 24, 0);

            // second light
            if (state == 0 && lightson)
                if (occupied)
                    c = QColor(Qt::red);
                else {
                    if ((blinkcounter > 0) || tablelight || routed || (lockCounter > 0))
                        c = QColor(255, 225, 0);
                    else
                        c = QColor(Qt::darkGray);
                }
            else
                c = QColor(Qt::darkGray);

            p.setPen(QPen(c, 3, Qt::SolidLine, Qt::RoundCap, Qt::MiterJoin));

            p.drawLine(-10, 0, -24, 0);

            p.restore();

            // third light
            if (state == 1 && lightson)
                if (occupied)
                    c = QColor(Qt::red);
                else {
                    if ((blinkcounter > 0) || tablelight || routed || (lockCounter > 0))
                        c = QColor(255, 225, 0);
                    else
                        c = QColor(Qt::darkGray);
                }
            else
                c = QColor(Qt::darkGray);

            p.setPen(QPen(c, 3, Qt::SolidLine, Qt::RoundCap, Qt::MiterJoin));

            p.drawLine(5, h / 2, 5 + 14, h / 2);

            p.setPen(QPen(Qt::black));
        }

        // paint track button
        p.drawPixmap(w / 2 - 4, h / 2 - 3, QPixmap(button_black_xpm));

        // paint lock light
        if (lockCounter == 0)
            p.setBrush(Qt::darkGray);
        else
            p.setBrush(QColor(255, 225, 0));

        p.drawEllipse(w / 2 + 5, 8, 4, 4);

        // paint text label
        if (sSoldText != "-1" && !sSoldText.isEmpty()) {
            QFont f(QApplication::font());
            f.setPointSize(QApplication::font().pointSize() - 3);
            p.setFont(f);
            QFontMetrics fm(f);
            QString s;

            if (pref.addresslabeling)
                s.setNum(address1);
            else
                s = sSoldText;

            QRect br = fm.boundingRect(s);
            br.setWidth(br.width() + 4);
            br.setHeight(br.height() - 2);

            br.moveBottomLeft(QPoint(2, h - 3));

            p.fillRect(br, QBrush(Qt::white));
            p.setBrush(Qt::white);
            br.setX(br.x() + 1);
            br.setY(br.y() + 1);
            br.setHeight(br.height() - 2);

            p.drawText(br, Qt::AlignCenter | Qt::TextSingleLine |
                    Qt::TextDontClip, s);
        }
    }

    // diagonal turnout left bottom (+ old top)
    else if (classid == siciIl1) {
        QPainter p(&background);
        p.setRenderHint(QPainter::Antialiasing);

        int w = background.width();
        int h = background.height();

        // paint track
        p.fillRect(w / 2, h / 2 - 3, w, 7, QBrush(Qt::black));

        p.save();
        p.translate(w / 2, h / 2);

        p.rotate(SANGLE);

        p.setPen(QPen(Qt::black, 7));
        p.drawLine(-w / 2 - 5, 0, w / 2 + 5, 0);
        p.restore();

        // paint track lights
        if (trackindicatoroff) {
            // short track
            for (int i = 0; i < 3; ++i)
                p.fillRect(w / 2 + 5 + 7 * i, h / 2 - 2, 5, 5,
                        QBrush(Qt::lightGray));

            // long track
            p.save();
            p.translate(w / 2, h / 2);

            p.rotate(SANGLE);

            for (int i = 0; i < 3; ++i)
                p.fillRect(7 + 7 * i, - 2, 5, 5, QBrush(Qt::lightGray));

            for (int i = 0; i < 3; ++i)
                p.fillRect(-11 - 7 * i, - 2, 5, 5, QBrush(Qt::lightGray));

            p.restore();

        }
        else {
            QColor c;
            p.save();
            p.translate(w / 2, h / 2);

            p.rotate(SANGLE);

            // first light
            if (occupied)
                c = QColor(Qt::red);
            else {
                if (routed)
                    c = QColor(255, 225, 0);
                else
                    c = QColor(Qt::darkGray);
            }
            p.setPen(QPen(c, 3, Qt::SolidLine, Qt::RoundCap, Qt::MiterJoin));
            p.drawLine(-10, 0, -24, 0);

            // second light
            if (state == 0 && lightson)
                if (occupied)
                    c = QColor(Qt::red);
                else {
                    if ((blinkcounter > 0) || tablelight || routed || (lockCounter > 0))
                        c = QColor(255, 225, 0);
                    else
                        c = QColor(Qt::darkGray);
                }
            else
                c = QColor(Qt::darkGray);

            p.setPen(QPen(c, 3, Qt::SolidLine, Qt::RoundCap, Qt::MiterJoin));

            p.drawLine(10, 0, 24, 0);

            p.restore();

            // third light
            if (state == 1 && lightson)
                if (occupied)
                    c = QColor(Qt::red);
                else {
                    if ((blinkcounter > 0) || tablelight || routed || (lockCounter > 0))
                        c = QColor(255, 225, 0);
                    else
                        c = QColor(Qt::darkGray);
                }
            else
                c = QColor(Qt::darkGray);

            p.setPen(QPen(c, 3, Qt::SolidLine, Qt::RoundCap, Qt::MiterJoin));

            p.drawLine(w - 5, h / 2, w - 5 - 14, h / 2);

            p.setPen(QPen(Qt::black));
        }

        // paint track button
        p.drawPixmap(w / 2 - 4, h / 2 - 3, QPixmap(button_black_xpm));

        // paint lock light
        if (lockCounter == 0)
            p.setBrush(Qt::darkGray);
        else
            p.setBrush(QColor(255, 225, 0));

        p.drawEllipse(w / 2 - 8, h - 14, 4, 4);

        // paint text label
        if (sSoldText != "-1" && !sSoldText.isEmpty()) {
            QFont f(QApplication::font());
            f.setPointSize(QApplication::font().pointSize() - 3);
            p.setFont(f);
            QFontMetrics fm(f);
            QString s;

            if (pref.addresslabeling)
                s.setNum(address1);
            else
                s = sSoldText;

            QRect br = fm.boundingRect(s);
            br.setWidth(br.width() + 4);
            br.setHeight(br.height() - 2);

            br.moveTopRight(QPoint(w - 3, 2));

            p.fillRect(br, QBrush(Qt::white));
            p.setBrush(Qt::white);
            br.setX(br.x() + 1);
            br.setY(br.y() + 1);
            br.setHeight(br.height() - 2);

            p.drawText(br, Qt::AlignCenter | Qt::TextSingleLine |
                    Qt::TextDontClip, s);
        }
    }

    // diagonal turnout right top (+ old bottom)
    else if (classid == siciIr1) {
        QPainter p(&background);
        p.setRenderHint(QPainter::Antialiasing);

        int w = background.width();
        int h = background.height();

        // paint track
        p.fillRect(w / 2, h / 2 - 3, w, 7, QBrush(Qt::black));

        p.save();
        p.translate(w / 2, h / 2);

        p.rotate(-SANGLE);

        p.setPen(QPen(Qt::black, 7));
        p.drawLine(-w / 2 - 5, 0, w / 2 + 5, 0);
        p.restore();

        // paint track lights
        if (trackindicatoroff) {
            // short track
            for (int i = 0; i < 3; ++i)
                p.fillRect(w / 2 + 5 + 7 * i, h / 2 - 2, 5, 5,
                        QBrush(Qt::lightGray));

            // long track
            p.save();
            p.translate(w / 2, h / 2);

            p.rotate(-SANGLE);

            for (int i = 0; i < 3; ++i)
                p.fillRect(7 + 7 * i, - 2, 5, 5, QBrush(Qt::lightGray));

            for (int i = 0; i < 3; ++i)
                p.fillRect(-11 - 7 * i, - 2, 5, 5, QBrush(Qt::lightGray));

            p.restore();

        }
        else {
            QColor c;
            p.save();
            p.translate(w / 2, h / 2);

            p.rotate(-SANGLE);

            // first light
            if (occupied)
                c = QColor(Qt::red);
            else {
                if (routed)
                    c = QColor(255, 225, 0);
                else
                    c = QColor(Qt::darkGray);
            }
            p.setPen(QPen(c, 3, Qt::SolidLine, Qt::RoundCap, Qt::MiterJoin));
            p.drawLine(-10, 0, -24, 0);

            // second light
            if (state == 0 && lightson)
                if (occupied)
                    c = QColor(Qt::red);
                else {
                    if ((blinkcounter > 0) || tablelight || routed || (lockCounter > 0))
                        c = QColor(255, 225, 0);
                    else
                        c = QColor(Qt::darkGray);
                }
            else
                c = QColor(Qt::darkGray);

            p.setPen(QPen(c, 3, Qt::SolidLine, Qt::RoundCap, Qt::MiterJoin));

            p.drawLine(10, 0, 24, 0);

            p.restore();

            // third light
            if (state == 1 && lightson)
                if (occupied)
                    c = QColor(Qt::red);
                else {
                    if ((blinkcounter > 0) || tablelight || routed || (lockCounter > 0))
                        c = QColor(255, 225, 0);
                    else
                        c = QColor(Qt::darkGray);
                }
            else
                c = QColor(Qt::darkGray);

            p.setPen(QPen(c, 3, Qt::SolidLine, Qt::RoundCap, Qt::MiterJoin));

            p.drawLine(w - 5, h / 2, w - 5 - 14, h / 2);

            p.setPen(QPen(Qt::black));
        }

        // paint track button
        p.drawPixmap(w / 2 - 4, h / 2 - 3, QPixmap(button_black_xpm));

        // paint lock light
        if (lockCounter == 0)
            p.setBrush(Qt::darkGray);
        else
            p.setBrush(QColor(255, 225, 0));

        p.drawEllipse(w / 2 - 8, 8, 4, 4);

        // paint text label
        if (sSoldText != "-1" && !sSoldText.isEmpty()) {
            QFont f(QApplication::font());
            f.setPointSize(QApplication::font().pointSize() - 3);
            p.setFont(f);
            QFontMetrics fm(f);
            QString s;

            if (pref.addresslabeling)
                s.setNum(address1);
            else
                s = sSoldText;

            QRect br = fm.boundingRect(s);
            br.setWidth(br.width() + 4);
            br.setHeight(br.height() - 2);

            br.moveBottomRight(QPoint(w - 3, h - 3));

            p.fillRect(br, QBrush(Qt::white));
            p.setBrush(Qt::white);
            br.setX(br.x() + 1);
            br.setY(br.y() + 1);
            br.setHeight(br.height() - 2);

            p.drawText(br, Qt::AlignCenter | Qt::TextSingleLine |
                    Qt::TextDontClip, s);
        }
    }

    // diagonal turnout right bottom
    else if (classid == siciIr3) {
        QPainter p(&background);
        p.setRenderHint(QPainter::Antialiasing);

        int w = background.width();
        int h = background.height();

        // paint track
        p.fillRect(0, h / 2 - 3, w / 2, 7, QBrush(Qt::black));

        p.save();
        p.translate(w / 2, h / 2);

        p.rotate(WANGLE);

        p.setPen(QPen(Qt::black, 7));
        p.drawLine(-w / 2 - 5, 0, w / 2 + 5, 0);
        p.restore();

        // paint track lights
        if (trackindicatoroff) {
            // short track
            for (int i = 0; i < 3; ++i)
                p.fillRect(4 + 7 * i, h / 2-2, 5, 5, QBrush(Qt::lightGray));

            // long track
            p.save();
            p.translate(w / 2, h / 2);

            p.rotate(WANGLE);

            for (int i = 0; i < 3; ++i)
                p.fillRect(7 + 7 * i, - 2, 5, 5, QBrush(Qt::lightGray));

            for (int i = 0; i < 3; ++i)
                p.fillRect(-11 - 7 * i, - 2, 5, 5, QBrush(Qt::lightGray));

            p.restore();

        }
        else {
            QColor c;
            p.save();
            p.translate(w / 2, h / 2);

            p.rotate(-SANGLE);

            // first light
            if (occupied)
                c = QColor(Qt::red);
            else {
                if (routed)
                    c = QColor(255, 225, 0);
                else
                    c = QColor(Qt::darkGray);
            }
            p.setPen(QPen(c, 3, Qt::SolidLine, Qt::RoundCap, Qt::MiterJoin));
            p.drawLine(10, 0, 24, 0);

            // second light
            if (state == 0 && lightson)
                if (occupied)
                    c = QColor(Qt::red);
                else {
                    if ((blinkcounter > 0) || tablelight || routed || (lockCounter > 0))
                        c = QColor(255, 225, 0);
                    else
                        c = QColor(Qt::darkGray);
                }
            else
                c = QColor(Qt::darkGray);

            p.setPen(QPen(c, 3, Qt::SolidLine, Qt::RoundCap, Qt::MiterJoin));

            p.drawLine(-10, 0, -24, 0);

            p.restore();

            // third light
            if (state == 1 && lightson)
                if (occupied)
                    c = QColor(Qt::red);
                else {
                    if ((blinkcounter > 0) || tablelight || routed || (lockCounter > 0))
                        c = QColor(255, 225, 0);
                    else
                        c = QColor(Qt::darkGray);
                }
            else
                c = QColor(Qt::darkGray);

            p.setPen(QPen(c, 3, Qt::SolidLine, Qt::RoundCap, Qt::MiterJoin));

            p.drawLine(5, h / 2, 5 + 14, h / 2);

            p.setPen(QPen(Qt::black));
        }

        // paint track button
        p.drawPixmap(w / 2 - 4, h / 2 - 3, QPixmap(button_black_xpm));

        // paint lock light
        if (lockCounter == 0)
            p.setBrush(Qt::darkGray);
        else
            p.setBrush(QColor(255, 225, 0));

        p.drawEllipse(w / 2 + 5, h - 14, 4, 4);

        // paint text label
        if (sSoldText != "-1" && !sSoldText.isEmpty()) {
            QFont f(QApplication::font());
            f.setPointSize(QApplication::font().pointSize() - 3);
            p.setFont(f);
            QFontMetrics fm(f);
            QString s;

            if (pref.addresslabeling)
                s.setNum(address1);
            else
                s = sSoldText;

            QRect br = fm.boundingRect(s);
            br.setWidth(br.width() + 4);
            br.setHeight(br.height() - 2);

            br.moveTopLeft(QPoint(2, 2));

            p.fillRect(br, QBrush(Qt::white));
            p.setBrush(Qt::white);
            br.setX(br.x() + 1);
            br.setY(br.y() + 1);
            br.setHeight(br.height() - 2);

            p.drawText(br, Qt::AlignCenter | Qt::TextSingleLine |
                    Qt::TextDontClip, s);
        }
    }

    // y-turnout right (0: -\  1: -/)
    else if (classid == siciSy1) {
        QPainter p(&background);
        p.setRenderHint(QPainter::Antialiasing);

        int w = background.width();
        int h = background.height();

        // paint track
        p.save();
        p.translate(w / 2, h / 2);
        p.setPen(QPen(Qt::black, 7));

        p.drawLine(-w / 2, 0, 0, 0);
        p.rotate(-SANGLE);
        p.drawLine(0, 0, w / 2 + 5, 0);
        p.rotate(SANGLE * 2);
        p.drawLine(0, 0, w / 2 + 5, 0);

        p.restore();

        // paint track lights
        if (trackindicatoroff) {
            // short track
            p.save();
            p.translate(w / 2, h / 2);

            for (int i = 0; i < 3; ++i)
                p.fillRect(-11 - 7 * i, -2, 5, 5, QBrush(Qt::lightGray));
            p.rotate(-SANGLE);
            for (int i = 0; i < 3; ++i)
                p.fillRect(7 + 7 * i, -2, 5, 5, QBrush(Qt::lightGray));
            p.rotate(SANGLE * 2);
            for (int i = 0; i < 3; ++i)
                p.fillRect(7 + 7 * i, -2, 5, 5, QBrush(Qt::lightGray));
            p.restore();

        }
        else {
            // first light
            QColor c;
            if (occupied)
                c = QColor(Qt::red);
            else {
                if (routed)
                    c = QColor(255, 225, 0);
                else
                    c = QColor(Qt::darkGray);
            }
            p.setPen(QPen(c, 3, Qt::SolidLine, Qt::RoundCap, Qt::MiterJoin));

            p.drawLine(5, h / 2 , 19,  h / 2);

            // second light
            if (state == 0 && lightson)
                if (occupied)
                    c = QColor(Qt::red);
                else {
                    if ((blinkcounter > 0) || tablelight || routed || (lockCounter > 0))
                        c = QColor(255, 225, 0);
                    else
                        c = QColor(Qt::darkGray);
                }
            else
                c = QColor(Qt::darkGray);

            p.setPen(QPen(c, 3, Qt::SolidLine, Qt::RoundCap, Qt::MiterJoin));

            p.save();
            p.translate(w / 2, h / 2);

            p.rotate(SANGLE);

            p.drawLine(11, 0, 11 + 14, 0);
            p.restore();

            // third light
            if (state == 1 && lightson)
                if (occupied)
                    c = QColor(Qt::red);
                else {
                    if ((blinkcounter > 0) || tablelight || routed || (lockCounter > 0))
                        c = QColor(255, 225, 0);
                    else
                        c = QColor(Qt::darkGray);
                }
            else
                c = QColor(Qt::darkGray);

            p.setPen(QPen(c, 3, Qt::SolidLine, Qt::RoundCap, Qt::MiterJoin));

            p.save();
            p.translate(w / 2, h / 2);

            p.rotate(-SANGLE);

            p.drawLine(11, 0, 11 + 14, 0);
            p.restore();

            p.setPen(QPen(Qt::black));
        }

        // paint track button
        p.drawPixmap(w / 2 - 4, h / 2 - 3, QPixmap(button_black_xpm));

        // paint lock light
        if (lockCounter == 0)
            p.setBrush(Qt::darkGray);
        else
            p.setBrush(QColor(255, 225, 0));

        p.drawEllipse(w / 2 - 4, h - 11, 4, 4);

        // paint text label
        if (sSoldText != "-1" && !sSoldText.isEmpty()) {
            QFont f(QApplication::font());
            f.setPointSize(QApplication::font().pointSize() - 3);
            p.setFont(f);
            QFontMetrics fm(f);
            QString s;

            if (pref.addresslabeling)
                s.setNum(address1);
            else
                s = sSoldText;

            QRect br = fm.boundingRect(s);
            br.setWidth(br.width() + 4);
            br.setHeight(br.height() - 2);

            br.moveTopLeft(QPoint(2, 2));

            p.fillRect(br, QBrush(Qt::white));
            p.setBrush(Qt::white);
            br.setX(br.x() + 1);
            br.setY(br.y() + 1);
            br.setHeight(br.height() - 2);

            p.drawText(br, Qt::AlignCenter | Qt::TextSingleLine |
                    Qt::TextDontClip, s);
        }
    }

    // y-turnout left
    else if (classid == siciSy3) {
        QPainter p(&background);
        p.setRenderHint(QPainter::Antialiasing);

        int w = background.width();
        int h = background.height();

        // paint track
        p.save();
        p.translate(w / 2, h / 2);
        p.setPen(QPen(Qt::black, 7));

        p.drawLine(0, 0, w / 2, 0);
        p.rotate(WANGLE);
        p.drawLine(0, 0, w / 2 + 5, 0);
        p.rotate(-WANGLE * 2);
        p.drawLine(0, 0, w / 2 + 5, 0);

        p.restore();

        // paint track lights
        if (trackindicatoroff) {
            // short track
            p.save();
            p.translate(w / 2, h / 2);

            for (int i = 0; i < 3; ++i)
                p.fillRect(6 + 7 * i, -2, 5, 5, QBrush(Qt::lightGray));
            p.rotate(WANGLE);
            for (int i = 0; i < 3; ++i)
                p.fillRect(7 + 7 * i, -2, 5, 5, QBrush(Qt::lightGray));
            p.rotate(-WANGLE * 2);
            for (int i = 0; i < 3; ++i)
                p.fillRect(7 + 7 * i, -2, 5, 5, QBrush(Qt::lightGray));
            p.restore();

        }
        else {
            // first light
            QColor c;
            if (occupied)
                c = QColor(Qt::red);
            else {
                if (routed)
                    c = QColor(255, 225, 0);
                else
                    c = QColor(Qt::darkGray);
            }
            p.setPen(QPen(c, 3, Qt::SolidLine, Qt::RoundCap, Qt::MiterJoin));

            p.drawLine(w - 5, h / 2 , w - 19,  h / 2);

            // second light
            if (state == 0 && lightson)
                if (occupied)
                    c = QColor(Qt::red);
                else {
                    if ((blinkcounter > 0) || tablelight || routed || (lockCounter > 0))
                        c = QColor(255, 225, 0);
                    else
                        c = QColor(Qt::darkGray);
                }
            else
                c = QColor(Qt::darkGray);

            p.setPen(QPen(c, 3, Qt::SolidLine, Qt::RoundCap, Qt::MiterJoin));

            p.save();
            p.translate(w / 2, h / 2);

            p.rotate(-WANGLE);

            p.drawLine(11, 0, 11 + 14, 0);
            p.restore();

            // third light
            if (state == 1 && lightson)
                if (occupied)
                    c = QColor(Qt::red);
                else {
                    if ((blinkcounter > 0) || tablelight || routed || (lockCounter > 0))
                        c = QColor(255, 225, 0);
                    else
                        c = QColor(Qt::darkGray);
                }
            else
                c = QColor(Qt::darkGray);

            p.setPen(QPen(c, 3, Qt::SolidLine, Qt::RoundCap, Qt::MiterJoin));

            p.save();
            p.translate(w / 2, h / 2);
            p.rotate(WANGLE);
            p.drawLine(11, 0, 11 + 14, 0);
            p.restore();

            p.setPen(QPen(Qt::black));
        }

        // paint track button
        p.drawPixmap(w / 2 - 4, h / 2 - 3, QPixmap(button_black_xpm));

        // paint lock light
        if (lockCounter == 0)
            p.setBrush(Qt::darkGray);
        else
            p.setBrush(QColor(255, 225, 0));

        p.drawEllipse(w / 2, 6, 4, 4);

        // paint text label
        if (sSoldText != "-1" && !sSoldText.isEmpty()) {
            QFont f(QApplication::font());
            f.setPointSize(QApplication::font().pointSize() - 3);
            p.setFont(f);
            QFontMetrics fm(f);
            QString s;

            if (pref.addresslabeling)
                s.setNum(address1);
            else
                s = sSoldText;

            QRect br = fm.boundingRect(s);
            br.setWidth(br.width() + 4);
            br.setHeight(br.height() - 2);

            br.moveBottomRight(QPoint(w - 3, h - 3));

            p.fillRect(br, QBrush(Qt::white));
            p.setBrush(Qt::white);
            br.setX(br.x() + 1);
            br.setY(br.y() + 1);
            br.setHeight(br.height() - 2);

            p.drawText(br, Qt::AlignCenter | Qt::TextSingleLine |
                    Qt::TextDontClip, s);
        }
    }

    // 3-way turnout right (+ old left)
    else if (classid == siciTw1) {
        QPainter p(&background);
        p.setRenderHint(QPainter::Antialiasing);

        int w = background.width();
        int h = background.height();

        // paint track
        p.fillRect(0, h / 2 - 3, w, 7, QBrush(Qt::black));

        p.save();
        p.translate(w / 2, h / 2);
        p.setPen(QPen(Qt::black, 7));

        p.rotate(-SANGLE);
        p.drawLine(0, 0, w / 2 + 5, 0);
        p.rotate(SANGLE * 2);
        p.drawLine(0, 0, w / 2 + 5, 0);

        p.restore();

        // paint track lights (track indicator)
        if (trackindicatoroff) {
            p.save();
            p.translate(w / 2, h / 2);

            for (int i = 0; i < 3; ++i)
                p.fillRect(-11 - 7 * i, -2, 5, 5, QBrush(Qt::lightGray));
            for (int i = 0; i < 3; ++i)
                p.fillRect(6 + 7 * i, -2, 5, 5, QBrush(Qt::lightGray));
            p.rotate(-SANGLE);
            for (int i = 0; i < 3; ++i)
                p.fillRect(7 + 7 * i, -2, 5, 5, QBrush(Qt::lightGray));
            p.rotate(SANGLE * 2);
            for (int i = 0; i < 3; ++i)
                p.fillRect(7 + 7 * i, -2, 5, 5, QBrush(Qt::lightGray));

            p.restore();

        }
        else {
            // first light
            QColor c;
            if (occupied)
                c = QColor(Qt::red);
            else {
                if (routed)
                    c = QColor(255, 225, 0);
                else
                    c = QColor(Qt::darkGray);
            }
            p.setPen(QPen(c, 3, Qt::SolidLine, Qt::RoundCap, Qt::MiterJoin));

            p.drawLine(5, h / 2 , 19,  h / 2);

            // second light
            if (state == 0 && lightson)
                if (occupied)
                    c = QColor(Qt::red);
                else {
                    if ((blinkcounter > 0) || tablelight || routed || (lockCounter > 0))
                        c = QColor(255, 225, 0);
                    else
                        c = QColor(Qt::darkGray);
                }
            else
                c = QColor(Qt::darkGray);

            p.setPen(QPen(c, 3, Qt::SolidLine, Qt::RoundCap, Qt::MiterJoin));

            p.drawLine(w - 5, h / 2 , w - 19,  h / 2);

            // third light
            if (state == 1 && lightson)
                if (occupied)
                    c = QColor(Qt::red);
                else {
                    if ((blinkcounter > 0) || tablelight || routed || (lockCounter > 0))
                        c = QColor(255, 225, 0);
                    else
                        c = QColor(Qt::darkGray);
                }
            else
                c = QColor(Qt::darkGray);

            p.setPen(QPen(c, 3, Qt::SolidLine, Qt::RoundCap, Qt::MiterJoin));

            p.save();
            p.translate(w / 2, h / 2);

            p.rotate(-SANGLE);

            p.drawLine(11, 0, 11 + 14, 0);
            p.restore();

            // fourth light
            if (state == 2 && lightson)
                if (occupied)
                    c = QColor(Qt::red);
                else {
                    if ((blinkcounter > 0) || tablelight || routed || (lockCounter > 0))
                        c = QColor(255, 225, 0);
                    else
                        c = QColor(Qt::darkGray);
                }
            else
                c = QColor(Qt::darkGray);

            p.setPen(QPen(c, 3, Qt::SolidLine, Qt::RoundCap, Qt::MiterJoin));

            p.save();
            p.translate(w / 2, h / 2);

            p.rotate(SANGLE);

            p.drawLine(11, 0, 11 + 14, 0);
            p.restore();

            p.setPen(QPen(Qt::black));
        }

        // paint track button
        p.drawPixmap(w / 2 - 4, h / 2 - 3, QPixmap(button_black_xpm));

        // paint lock light
        if (lockCounter == 0)
            p.setBrush(Qt::darkGray);
        else
            p.setBrush(QColor(255, 225, 0));

        p.drawEllipse(w / 2 - 4, h - 11, 4, 4);

        // paint text label
        if (sSoldText != "-1" && !sSoldText.isEmpty()) {
            QFont f(QApplication::font());
            f.setPointSize(QApplication::font().pointSize() - 3);
            p.setFont(f);
            QFontMetrics fm(f);
            QString s;

            if (pref.addresslabeling)
                s.setNum(address1);
            else
                s = sSoldText;

            QRect br = fm.boundingRect(s);
            br.setWidth(br.width() + 4);
            br.setHeight(br.height() - 2);

            br.moveTopLeft(QPoint(2, 2));

            p.fillRect(br, QBrush(Qt::white));
            p.setBrush(Qt::white);
            br.setX(br.x() + 1);
            br.setY(br.y() + 1);
            br.setHeight(br.height() - 2);

            p.drawText(br, Qt::AlignCenter | Qt::TextSingleLine |
                    Qt::TextDontClip, s);
        }
    }

    // 3-way turnout left
    else if (classid == siciTw3) {
        QPainter p(&background);
        p.setRenderHint(QPainter::Antialiasing);

        int w = background.width();
        int h = background.height();

        // paint track
        p.fillRect(0, h / 2 - 3, w, 7, QBrush(Qt::black));

        p.save();
        p.translate(w / 2, h / 2);
        p.setPen(QPen(Qt::black, 7));

        p.rotate(WANGLE);
        p.drawLine(0, 0, w / 2 + 5, 0);
        p.rotate(-WANGLE * 2);
        p.drawLine(0, 0, w / 2 + 5, 0);

        p.restore();

        // paint track lights (track indicator)
        if (trackindicatoroff) {
            p.save();
            p.translate(w / 2, h / 2);

            for (int i = 0; i < 3; ++i)
                p.fillRect(-11 - 7 * i, -2, 5, 5, QBrush(Qt::lightGray));
            for (int i = 0; i < 3; ++i)
                p.fillRect(6 + 7 * i, -2, 5, 5, QBrush(Qt::lightGray));
            p.rotate(WANGLE);
            for (int i = 0; i < 3; ++i)
                p.fillRect(7 + 7 * i, -2, 5, 5, QBrush(Qt::lightGray));
            p.rotate(-WANGLE * 2);
            for (int i = 0; i < 3; ++i)
                p.fillRect(7 + 7 * i, -2, 5, 5, QBrush(Qt::lightGray));

            p.restore();

        }
        else {
            // first light
            QColor c;
            if (occupied)
                c = QColor(Qt::red);
            else {
                if (routed)
                    c = QColor(255, 225, 0);
                else
                    c = QColor(Qt::darkGray);
            }
            p.setPen(QPen(c, 3, Qt::SolidLine, Qt::RoundCap, Qt::MiterJoin));

            p.drawLine(w - 5, h / 2 , w - 19,  h / 2);

            // second light
            if (state == 0 && lightson)
                if (occupied)
                    c = QColor(Qt::red);
                else {
                    if ((blinkcounter > 0) || tablelight || routed || (lockCounter > 0))
                        c = QColor(255, 225, 0);
                    else
                        c = QColor(Qt::darkGray);
                }
            else
                c = QColor(Qt::darkGray);

            p.setPen(QPen(c, 3, Qt::SolidLine, Qt::RoundCap, Qt::MiterJoin));

            p.drawLine(5, h / 2 , 19,  h / 2);

            // third light
            if (state == 1 && lightson)
                if (occupied)
                    c = QColor(Qt::red);
                else {
                    if ((blinkcounter > 0) || tablelight || routed || (lockCounter > 0))
                        c = QColor(255, 225, 0);
                    else
                        c = QColor(Qt::darkGray);
                }
            else
                c = QColor(Qt::darkGray);

            p.setPen(QPen(c, 3, Qt::SolidLine, Qt::RoundCap, Qt::MiterJoin));

            p.save();
            p.translate(w / 2, h / 2);

            p.rotate(WANGLE);

            p.drawLine(11, 0, 11 + 14, 0);
            p.restore();

            // fourth light
            if (state == 2 && lightson)
                if (occupied)
                    c = QColor(Qt::red);
                else {
                    if ((blinkcounter > 0) || tablelight || routed || (lockCounter > 0))
                        c = QColor(255, 225, 0);
                    else
                        c = QColor(Qt::darkGray);
                }
            else
                c = QColor(Qt::darkGray);

            p.setPen(QPen(c, 3, Qt::SolidLine, Qt::RoundCap, Qt::MiterJoin));

            p.save();
            p.translate(w / 2, h / 2);

            p.rotate(-WANGLE);

            p.drawLine(11, 0, 11 + 14, 0);
            p.restore();

            p.setPen(QPen(Qt::black));
        }

        // paint track button
        p.drawPixmap(w / 2 - 4, h / 2 - 3, QPixmap(button_black_xpm));

        // paint lock light
        if (lockCounter == 0)
            p.setBrush(Qt::darkGray);
        else
            p.setBrush(QColor(255, 225, 0));

        p.drawEllipse(w / 2, 6, 4, 4);

        // paint text label
        if (sSoldText != "-1" && !sSoldText.isEmpty()) {
            QFont f(QApplication::font());
            f.setPointSize(QApplication::font().pointSize() - 3);
            p.setFont(f);
            QFontMetrics fm(f);
            QString s;

            if (pref.addresslabeling)
                s.setNum(address1);
            else
                s = sSoldText;

            QRect br = fm.boundingRect(s);
            br.setWidth(br.width() + 4);
            br.setHeight(br.height() - 2);

            br.moveBottomRight(QPoint(w - 3, h - 3));

            p.fillRect(br, QBrush(Qt::white));
            p.setBrush(Qt::white);
            br.setX(br.x() + 1);
            br.setY(br.y() + 1);
            br.setHeight(br.height() - 2);

            p.drawText(br, Qt::AlignCenter | Qt::TextSingleLine |
                    Qt::TextDontClip, s);
        }
    }

    // straight track with horizontal tunnel (old left + right)
    else if (classid == siciTuh) {
        QPainter p(&background);

        int w = background.width();
        int h = background.height();
        int tracklen = w / 2 - 8;

        // paint track
        // vertical section
        p.fillRect(w / 2 - 3, 0, 7, h, QBrush(Qt::black));

        // paint track lights
        if (trackindicatoroff) {
            for (int i = 0; i < 4; ++i)
                p.fillRect(w / 2 - 2 , 4 + 7 * i, 5, 5,
                        QBrush(Qt::lightGray));
        }
        else {
            QColor c;
            if (occupied)
                c = QColor(Qt::red);
            else {
                if (routed)
                    c = QColor(255, 225, 0);
                else
                    c = QColor(Qt::darkGray);
            }
            p.setPen(QPen(c, 3, Qt::SolidLine, Qt::RoundCap, Qt::MiterJoin));
            p.drawLine(w / 2, h / 3, w / 2, 2 * h / 3);
            p.setPen(QPen(Qt::black));
        }

        // horizontal sections
        p.fillRect(0, h / 2 - 3, tracklen, 7, QBrush(Qt::black));
        p.fillRect(w - tracklen + 1, h / 2 - 3, tracklen -1, 7,
                QBrush(Qt::black));

        // translate origin to center of pixmap
        p.translate(w / 2 - 6, h / 2);

        // paint tunnel entry
        QPolygon tunnel = QPolygon(4);
        tunnel.putPoints(0, 4, -8, -14, 0, -6, 0, 6, -8, 14);
        p.setPen(QPen(Qt::darkGray, 2));
        p.drawPolyline(tunnel);
        p.translate(13, 0);
        p.rotate(180.0);
        p.drawPolyline(tunnel);
    }

    // straight track with vertical tunnel
    else if (classid == siciTuv) {
        QPainter p(&background);

        int w = background.width();
        int h = background.height();
        int tracklen = h / 2 - 8;

        // paint track
        // horizontal section
        p.fillRect(0, h / 2 - 3, w, 7, QBrush(Qt::black));

        // paint track lights
        if (trackindicatoroff) {
            for (int i = 0; i < 7; ++i)
                p.fillRect(4 + 7 * i, h / 2 - 2, 5, 5,
                        QBrush(Qt::lightGray));
        }
        else {
            QColor c;
            if (occupied)
                c = QColor(Qt::red);
            else {
                if (routed)
                    c = QColor(255, 225, 0);
                else
                    c = QColor(Qt::darkGray);
            }
            p.setPen(QPen(c, 3, Qt::SolidLine, Qt::RoundCap, Qt::MiterJoin));
            p.drawLine(w / 3, h / 2, 2 * w / 3, h / 2);
            p.setPen(QPen(Qt::black));
        }

        // vertical sections
        p.fillRect(w / 2 - 3, 0, 7, tracklen, QBrush(Qt::black));
        p.fillRect(w / 2 - 3, h - tracklen, 7, tracklen, QBrush(Qt::black));

        // translate origin to center of pixmap
        p.translate(w / 2, h / 2);

        // paint tunnel entry
        QPolygon tunnel = QPolygon(4);
        tunnel.putPoints(0, 4, -8, -14, 0, -6, 0, 6, -8, 14);
        p.setPen(QPen(Qt::darkGray, 2));

        p.rotate(90.0);
        p.translate(-6, 0);
        p.drawPolyline(tunnel);
        p.translate(13, 0);
        p.rotate(180.0);
        p.drawPolyline(tunnel);
    }

    // left diagonal track with tunnel
    else if (classid == siciTul) {
        QPainter p(&background);
        p.setRenderHint(QPainter::Antialiasing);

        int w = background.width();
        int h = background.height();
        int tracklen = w / 2 + 5;

        // paint long diagonal track
        p.setPen(QPen(Qt::black, 7));
        p.drawLine(0, 0, w, h - 1);

        // paint track lights
        if (trackindicatoroff) {
            p.save();
            p.translate(w / 2, h / 2);
            p.rotate(SANGLE);

            for (int i = -3; i < 4; ++i)
                p.fillRect(-3 + 7 * i, -2, 5, 5, QBrush(Qt::lightGray));

            p.restore();
        }
        else {
            QColor c;
            if (occupied)
                c = QColor(Qt::red);
            else {
                if (routed && (routedtrack & 1) == 1)
                    c = QColor(255, 225, 0);
                else
                    c = QColor(Qt::darkGray);
            }
            p.save();
            p.translate(w / 2, h / 2);
            p.rotate(SANGLE);

            p.setPen(QPen(c, 3, Qt::SolidLine, Qt::RoundCap, Qt::MiterJoin));
            p.drawLine(-w / 6, 0, w / 6, 0);

            p.restore();
        }

        // translate origin to center of pixmap
        p.translate(w / 2, h / 2);

        p.rotate(-SANGLE);

        // paint two short tracks
        p.drawLine(-tracklen, 0, -9, 0);
        p.drawLine(9, 0, tracklen, 0);

        // paint tunnel entry
        QPolygon tunnel = QPolygon(4);
        tunnel.putPoints(0, 4, -8, -14, 0, -6, 0, 6, -8, 14);
        p.setPen(QPen(Qt::darkGray, 2));
        p.rotate(SANGLE - WANGLE + 90.0);
        p.translate(-6, -4);
        p.drawPolyline(tunnel);

        p.translate(6, 4);
        p.rotate(180.0);
        p.translate(-6, -4);
        p.drawPolyline(tunnel);
    }

    // right diagonal track with tunnel
    else if (classid == siciTur) {
        QPainter p(&background);
        p.setRenderHint(QPainter::Antialiasing);

        int w = background.width();
        int h = background.height();
        int tracklen = w / 2 + 5;

        // paint long diagonal track
        p.setPen(QPen(Qt::black, 7));
        p.drawLine(0, h - 1, w, 0);

        // paint track lights
        if (trackindicatoroff) {
            p.save();
            p.translate(w / 2, h / 2);
            p.rotate(-SANGLE);

            for (int i = -3; i < 4; ++i)
                p.fillRect(-3 + 7 * i, -2, 5, 5, QBrush(Qt::lightGray));

            p.restore();
        }
        else {
            QColor c;
            if (occupied)
                c = QColor(Qt::red);
            else {
                if (routed && (routedtrack & 1) == 1)
                    c = QColor(255, 225, 0);
                else
                    c = QColor(Qt::darkGray);
            }
            p.save();
            p.translate(w / 2, h / 2);
            p.rotate(-SANGLE);

            p.setPen(QPen(c, 3, Qt::SolidLine, Qt::RoundCap, Qt::MiterJoin));
            p.drawLine(-w / 6, 0, w / 6, 0);

            p.restore();
        }

        // translate origin to center of pixmap
        p.translate(w / 2, h / 2);
        p.rotate(-WANGLE);

        // paint two short tracks
        p.drawLine(-tracklen, 0, -9, 0);
        p.drawLine(9, 0, tracklen, 0);

        // paint tunnel entry
        QPolygon tunnel = QPolygon(4);
        tunnel.putPoints(0, 4, -8, -14, 0, -6, 0, 6, -8, 14);
        p.setPen(QPen(Qt::darkGray, 2));
        p.rotate(WANGLE - SANGLE + 90.0);
        p.translate(-6, 4);
        p.drawPolyline(tunnel);

        p.translate(6, -4);
        p.rotate(180.0);
        p.translate(-6, 4);
        p.drawPolyline(tunnel);
    }

    addTooltip();
    update();
    //repaint();
}


/**
 * remove every tool tip and if configured add new one
 * with current element data
 */
void element::addTooltip()
{
    if (!pref.datatooltips)
        return;

    QString a1 = QString::number(address1);
    QString a2 = QString::number(address2);

    QString tip1, tip2;

    tip1.asprintf(
            "Item No: %d\n"
            "ClassId: %d\n"
            "FB indicator off: %s (=%1d)\n"
            "Inverted: %s (=%1d)\n"
            "Protocol: %s\n"
            "Address 1: %s\n",
            iSoldIndex,
            classid,
            (trackindicatoroff ? "Yes" : "No"),
            trackindicatoroff,
            iSoldInvert == -1 ? "N/A" : (iSoldInvert ==
                0 ? "No" : "Yes"),
            iSoldInvert,
            protocol1 == GaMessage::proNone ? "N/A (=-1)"
                : (protocol1 == GaMessage::proMM ? "Motorola"
                        : (protocol1 == GaMessage::proDCC ? "NMRA/DCC"
                            : (protocol1 == GaMessage::proSelectrix ?
                                "Selectrix"
                                : "Server"))),
            address1 == -1 ?  "N/A (=-1)" : a1.toLatin1().data());

    tip2.asprintf(
            "Address 2: %s\n"
            "xc Conn 1: %s (=%1d)\n"
            "xc Conn 2: %s (=%1d)\n"
            "State: %d\n"
            "Subtype: %d\n"
            "Text: %s\n"
            "Locked: %s (=%1d)\n"
            "Time1 (ms): %d\n"
            "FB Contact: %d",
            address2 == -1 ? "N/A (=-1)" : a2.toLatin1().data(),
            xchangeport1 ==
            -1 ? "N/A" : (xchangeport1 == 0 ? "No" : "Yes"),
            xchangeport1,
            xchangeport2 ==
            -1 ? "N/A" : (xchangeport2 == 0 ? "No" : "Yes"),
            xchangeport2, state, iSoldSubType,
            sSoldText == "-1" ? "N/A (=-1)" : sSoldText.toLatin1().data(),
            lockCounter == -1 ? "N/A" : (isLocked() ? "Yes" : "No"),
            lockCounter, activetime1, iFBContact);

    tip1.append(tip2);

    setToolTip(tip1);
}


/**
 * paint yellow track and return vertical correction value
 * to_r = routing direction to right layout side
 * C = correction for index
 */

/* two dimensional routing:
 *       entry -> exit (rdC, rdN, rdS, rdW, rdE, rdNW, rdNE, ...)
 *
 *
 *  rdNW | rdN  | rdNE
 * ------+------+------
 *  rdW  | rdC  | rdE
 * ------+------+------
 *  rdSW | rdS  | rdSE
 *
 */
unsigned int element::routeElement(unsigned int entrydir, bool setroute)
{
    // route history stack
    penultimateroutedir = lastrouteentrydir;
    lastrouteentrydir = entrydir;
    unsigned int returnvalue = rdCenter;

    /* - */
    if (classid == siciSt1 || classid == siciEnk
            || classid == siciHs1 || classid == siciHs3
            || classid == siciZt1 || classid == siciZt3
            || classid == siciRt1 || classid == siciRt3
            || classid == siciHss1 || classid == siciHss3
            || classid == siciSs1 || classid == siciSs3
            || classid == siciSh1 || classid == siciSh3
            || classid == siciSd1 || classid == siciSd3
            || classid == siciBue || classid == siciTdr
            || classid == siciTdl || classid == siciTdb
            || classid == siciVs1 || classid == siciVs3
            || classid == siciWs1 || classid == siciWs3
            || classid == siciZp1 || classid == siciZp3
            || classid == siciBld
            || classid == siciAdr) {

        if (entrydir == rdW)
            returnvalue = rdE;
        else if (entrydir == rdE)
            returnvalue = rdW;
    }

    /* | */
    else if (classid == siciSt2) {
        if (entrydir == rdN)
            returnvalue = rdS;
        else if (entrydir == rdS)
            returnvalue = rdN;
    }

    /* \
       | */
    else if (classid == siciCl2) {
        if (entrydir == rdNW)
            returnvalue = rdS;
        else if (entrydir == rdS)
            returnvalue = rdNW;
    }

    /* /
       | */
    else if (classid == siciCr2) {
        if (entrydir == rdNE)
            returnvalue = rdS;
        else if (entrydir == rdS)
            returnvalue = rdNE;
    }

    /* |
       \ */
    else if (classid == siciCl4) {
        if (entrydir == rdN)
            returnvalue = rdSE;
        else if (entrydir == rdSE)
            returnvalue = rdN;
    }

    /* |
       / */
    else if (classid == siciCr4) {
        if (entrydir == rdN)
            returnvalue = rdSW;
        else if (entrydir == rdSW)
            returnvalue = rdN;
    }

    else if (classid == siciTl1) {
        // --
        if (state == 0) {
            if (entrydir == rdW)
                returnvalue = rdE;
            else if (entrydir == rdE)
                returnvalue = rdW;
        }
        else {
            // -/
            if (entrydir == rdW)
                returnvalue = rdNE;
            else if (entrydir == rdNE)
                returnvalue = rdW;
        }
    }

    else if (classid == siciTl3) {
        // --
        if (state == 0) {
            if (entrydir == rdW)
                returnvalue = rdE;
            else if (entrydir == rdE)
                returnvalue = rdW;
        }
        else {
            // /-
            if (entrydir == rdE)
                returnvalue = rdSW;
            else if (entrydir == rdSW)
                returnvalue = rdE;
        }
    }

    else if (classid == siciTr3) {
        // --
        if (state == 0) {
            if (entrydir == rdW)
                returnvalue = rdE;
            else if (entrydir == rdE)
                returnvalue = rdW;
        }
        else {
            // \-
            if (entrydir == rdE)
                returnvalue = rdNW;
            else if (entrydir == rdNW)
                returnvalue = rdE;
        }
    }

    else if (classid == siciTr1) {
        // --
        if (state == 0) {
            if (entrydir == rdW)
                returnvalue = rdE;
            else if (entrydir == rdE)
                returnvalue = rdW;
        }
        else {
            /* -\ */
            if (entrydir == rdW)
                returnvalue = rdSE;
            else if (entrydir == rdSE)
                returnvalue = rdW;
        }
    }

    /* / */
    else if (classid == siciSt4) {
        if (entrydir == rdSW)
            returnvalue = rdNE;
        else if (entrydir == rdNE)
            returnvalue = rdSW;
    }

    /* \ */
    else if (classid == siciSt3) {
        if (entrydir == rdNW)
            returnvalue = rdSE;
        else if (entrydir == rdSE)
            returnvalue = rdNW;
    }

    else if (classid == siciCl1) {
        /* -/ */
        if (entrydir == rdW)
            returnvalue = rdNE;
        else if (entrydir == rdNE)
            returnvalue = rdW;
    }

    else if (classid == siciCl3) {
        /* /- */
        if (entrydir == rdE)
            returnvalue = rdSW;
        else if (entrydir == rdSW)
            returnvalue = rdE;
    }

    else if (classid == siciCr1) {
        /* -\ */
        if (entrydir == rdW)
            returnvalue = rdSE;
        else if (entrydir == rdSE)
            returnvalue = rdW;
    }

    else if (classid == siciCr3) {
        /* \- */
        if (entrydir == rdE)
            returnvalue = rdNW;
        else if (entrydir == rdNW)
            returnvalue = rdE;
    }

    else if (classid == siciTw1) {
        // --
        if (state == 0) {
            if (entrydir == rdW)
                returnvalue = rdE;
            else if (entrydir == rdE)
                returnvalue = rdW;
        }
        else {
            /* -/ */
            if (state == 1) {
                if (entrydir == rdW)
                    returnvalue = rdNE;
                else if (entrydir == rdNE)
                    returnvalue = rdW;
            }
            /* -\ */
            else {
                if (entrydir == rdW)
                    returnvalue = rdSE;
                else if (entrydir == rdSE)
                    returnvalue = rdW;
            }
        }
    }

    else if (classid == siciTw3) {
        // --
        if (state == 0) {
            if (entrydir == rdW)
                returnvalue = rdE;
            else if (entrydir == rdE)
                returnvalue = rdW;
        }
        else {
            /* /- */
            if (state == 1) {
                if (entrydir == rdE)
                    returnvalue = rdSW;
                else if (entrydir == rdSW)
                    returnvalue = rdE;
            }
            /* \- */
            else {
                if (entrydir == rdE)
                    returnvalue = rdNW;
                else if (entrydir == rdNW)
                    returnvalue = rdE;
            }
        }
    }

    else if (classid == siciSy1) {
        /* -\ */
        if (state == 0) {
            if (entrydir == rdW)
                returnvalue = rdSE;
            else if (entrydir == rdSE)
                returnvalue = rdW;
        }
        /* -/ */
        else {
            if (entrydir == rdW)
                returnvalue = rdNE;
            else if (entrydir == rdNE)
                returnvalue = rdW;
        }
    }

    else if (classid == siciSy3) {
        /* \- */
        if (state == 0) {
            if (entrydir == rdE)
                returnvalue = rdNW;
            else if (entrydir == rdNW)
                returnvalue = rdE;
        }
        /* /- */
        else {
            if (entrydir == rdE)
                returnvalue = rdSW;
            else if (entrydir == rdSW)
                returnvalue = rdE;
        }
    }

    else if (classid == siciIl1) {
        /* \
           \ */
        if (state == 0) {
            if (entrydir == rdNW)
                returnvalue = rdSE;
            else if (entrydir == rdSE)
                returnvalue = rdNW;
        }
        else {
            // \_
            if (entrydir == rdNW)
                returnvalue = rdE;
            else if (entrydir == rdE)
                returnvalue = rdNW;
        }
    }

    else if (classid == siciIl3) {
        /* \
           \ */
        if (state == 0) {
            if (entrydir == rdNW)
                returnvalue = rdSE;
            else if (entrydir == rdSE)
                returnvalue = rdNW;
        }
        else {
            /* _
               \ */
            if (entrydir == rdW)
                returnvalue = rdSE;
            else if (entrydir == rdSE)
                returnvalue = rdW;
        }
    }

    else if (classid == siciIr1) {
        /*  /
            / */
        if (state == 0) {
            if (entrydir == rdNE)
                returnvalue = rdSW;
            else if (entrydir == rdSW)
                returnvalue = rdNE;
        }
        else {
            /*  _
                /  */
            if (entrydir == rdSW)
                returnvalue = rdE;
            else if (entrydir == rdE)
                returnvalue = rdSW;
        }
    }

    else if (classid == siciIr3) {
        /*  /
            / */
        if (state == 0) {
            if (entrydir == rdNE)
                returnvalue = rdSW;
            else if (entrydir == rdSW)
                returnvalue = rdNE;
        }
        else {
            // _/
            if (entrydir == rdNE)
                returnvalue = rdW;
            else if (entrydir == rdW)
                returnvalue = rdNE;
        }
    }

    else if (classid == siciSl3) {
        // --
        if (state == 0) {
            if (entrydir == rdE)
                returnvalue = rdW;
            else if (entrydir == rdW)
                returnvalue = rdE;
        }
        //  /
        // /
        else if (state == 2) {
            if (entrydir == rdNE)
                returnvalue = rdSW;
            else if (entrydir == rdSW)
                returnvalue = rdNE;
        }
        else {
            /*  _
               /  */
            if (entrydir == rdSW)
                returnvalue = rdE;
            else if (entrydir == rdE)
                returnvalue = rdSW;
        }

        // shortcut to avoid double painting
        routed = setroute;
        setupElementIcon();
        return returnvalue;
    }

    else if (classid == siciSl1) {
        // --
        if (state == 0) {
            if (entrydir == rdE)
                returnvalue = rdW;
            else if (entrydir == rdW)
                returnvalue = rdE;
        }
        //  /
        // /
        else if (state == 2) {
            if (entrydir == rdNE)
                returnvalue = rdSW;
            else if (entrydir == rdSW)
                returnvalue = rdNE;
        }
        else {
            // _/
            if (entrydir == rdNE)
                returnvalue = rdW;
            else if (entrydir == rdW)
                returnvalue = rdNE;
        }

        // shortcut to avoid double painting
        routed = setroute;
        setupElementIcon();
        return returnvalue;
    }

    else if (classid == siciSr3) {
        // --
        if (state == 0) {
            if (entrydir == rdE)
                returnvalue = rdW;
            else if (entrydir == rdW)
                returnvalue = rdE;
        }
        /* \
            \ */
        else if (state == 2) {
            if (entrydir == rdNW)
                returnvalue = rdSE;
            else if (entrydir == rdSE)
                returnvalue = rdNW;
        }
        else {
           /* _
               \ */
            if (entrydir == rdW)
                returnvalue = rdSE;
            else if (entrydir == rdSE)
                returnvalue = rdW;
        }

        // shortcut to avoid double painting
        routed = setroute;
        setupElementIcon();
        return returnvalue;
    }

    else if (classid == siciSr1) {
        // --
        if (state == 0) {
            if (entrydir == rdE)
                returnvalue = rdW;
            else if (entrydir == rdW)
                returnvalue = rdE;
        }
        /* \
            \ */
        else if (state == 2) {
            if (entrydir == rdNW)
                returnvalue = rdSE;
            else if (entrydir == rdSE)
                returnvalue = rdNW;
        }
        else {
            // \_
            if (entrydir == rdNW)
                returnvalue = rdE;
            else if (entrydir == rdE)
                returnvalue = rdNW;
        }

        // shortcut to avoid double painting
        routed = setroute;
        setupElementIcon();
        return returnvalue;
    }

    // 4-state-DKWs
    else if (classid == siciDl1 && iSoldSubType == 1) {
        // --
        if (state == 0) {
            if (entrydir == rdE)
                returnvalue = rdW;
            else if (entrydir == rdW)
                returnvalue = rdE;
        }
        // _/
        else if (state == 1) {
            if (entrydir == rdNE)
                returnvalue = rdW;
            else if (entrydir == rdW)
                returnvalue = rdNE;
        }
        //  /
        // /
        else if (state == 2) {
            if (entrydir == rdNE)
                returnvalue = rdSW;
            else if (entrydir == rdSW)
                returnvalue = rdNE;
        }
        /*  _
           /  */
        else if (state == 3) {
            if (entrydir == rdSW)
                returnvalue = rdE;
            else if (entrydir == rdE)
                returnvalue = rdSW;
        }

        // shortcut to avoid double painting
        routed = setroute;
        setupElementIcon();
        return returnvalue;
    }

    else if (classid == siciDr1 && iSoldSubType == 1) {
        // --
        if (state == 0) {
            if (entrydir == rdE)
                returnvalue = rdW;
            else if (entrydir == rdW)
                returnvalue = rdE;
        }
        // \_
        else if (state == 1) {
            if (entrydir == rdE)
                returnvalue = rdNW;
            else if (entrydir == rdNW)
                returnvalue = rdE;
        }
        /* \
            \ */
        else if (state == 2) {
            if (entrydir == rdNW)
                returnvalue = rdSE;
            else if (entrydir == rdSE)
                returnvalue = rdNW;
        }
        /* _
            \ */
        else if (state == 3) {
            if (entrydir == rdW)
                returnvalue = rdSE;
            else if (entrydir == rdSE)
                returnvalue = rdW;
        }

        // shortcut to avoid double painting
        routed = setroute;
        setupElementIcon();
        return returnvalue;
    }


    // 2-state-DKWs
    else if (classid == siciDl1 && iSoldSubType == 0) {
        // --
        if (state == 0) {
            if (entrydir == rdE)
                returnvalue = rdW;
            else if (entrydir == rdW)
                returnvalue = rdE;
        }
        /* _/
           _
          /  */
        else if (state == 1) {
            if (entrydir == rdW)
                returnvalue = rdNE;
            else if (entrydir == rdNE)
                returnvalue = rdW;
            else if (entrydir == rdE)
                returnvalue = rdSW;
            else if (entrydir == rdSW)
                returnvalue = rdE;
        }

        // shortcut to avoid double painting
        routed = setroute;
        setupElementIcon();
        return returnvalue;
    }

    else if (classid == siciDr1 && iSoldSubType == 0) {
        // --
        if (state == 0) {
            if (entrydir == rdE)
                returnvalue = rdW;
            else if (entrydir == rdW)
                returnvalue = rdE;
        }
        /* \_
           _
            \ */
        else if (state == 1) {
            if (entrydir == rdNW)
                returnvalue = rdE;
            else if (entrydir == rdE)
                returnvalue = rdNW;
            else if (entrydir == rdSE)
                returnvalue = rdW;
            else if (entrydir == rdW)
                returnvalue = rdSE;
        }

        // shortcut to avoid double painting
        routed = setroute;
        setupElementIcon();
        return returnvalue;
    }

    // rail crossings
    else if (classid == siciKrh) {

        //  /
        // /
        if (entrydir == rdSW) {
            returnvalue = rdNE;
            routedtrack = 1;
        }
        else if (entrydir == rdNE) {
            returnvalue = rdSW;
            routedtrack = 1;
        }

        /* \
            \ */
        else if (entrydir == rdNW) {
            returnvalue = rdSE;
            routedtrack = 2;
        }
        else if (entrydir == rdSE) {
            returnvalue = rdNW;
            routedtrack = 2;
        }

        if (!setroute)
            routedtrack = 0;
    }

    else if (classid == siciKl1) {
        // --
        if (entrydir == rdW) {
            returnvalue = rdE;
            routedtrack = 2;
        }
        else if (entrydir == rdE) {
            returnvalue = rdW;
            routedtrack = 2;
        }

        //  /
        // /
        else if (entrydir == rdSW) {
            returnvalue = rdNE;
            routedtrack = 1;
        }
        else if (entrydir == rdNE) {
            returnvalue = rdSW;
            routedtrack = 1;
        }

        if (!setroute) {
            routedtrack = 0;
        }
    }

    else if (classid == siciKr1) {
        // --
        if (entrydir == rdW) {
            returnvalue = rdE;
            routedtrack = 2;
        }
        else if (entrydir == rdE) {
            returnvalue = rdW;
            routedtrack = 2;
        }
        /* \
            \ */
        else if (entrydir == rdNW) {
            returnvalue = rdSE;
            routedtrack = 1;
        }
        else if (entrydir == rdSE) {
            returnvalue = rdNW;
            routedtrack = 1;
        }

        if (!setroute)
            routedtrack = 0;
    }

    //tunnel crossings
    else if (classid == siciTuv) {

        // | (tunnel)
        // |
        if (entrydir == rdS) {
            returnvalue = rdN;
            if (setroute)
                routedtrack |= 2;
            else
                routedtrack &= ~2;
            // no repaint neccessary
            return returnvalue;
        }
        else if (entrydir == rdN) {
            returnvalue = rdS;
            if (setroute)
                routedtrack |= 2;
            else
                routedtrack &= ~2;
            // no repaint neccessary
            return returnvalue;
        }

        /* -- (top) */
        else if (entrydir == rdW) {
            returnvalue = rdE;
            if (setroute)
                routedtrack |= 1;
            else
                routedtrack &= ~1;
        }
        else if (entrydir == rdE) {
            returnvalue = rdW;
            if (setroute)
                routedtrack |= 1;
            else
                routedtrack &= ~1;
        }
    }

    else if (classid == siciTuh) {

        // | (top)
        // |
        if (entrydir == rdS) {
            returnvalue = rdN;
            if (setroute)
                routedtrack |= 1;
            else
                routedtrack &= ~1;
        }
        else if (entrydir == rdN) {
            returnvalue = rdS;
            if (setroute)
                routedtrack |= 1;
            else
                routedtrack &= ~1;
        }

        /* --  (tunnel) */
        else if (entrydir == rdW) {
            returnvalue = rdE;
            if (setroute)
                routedtrack |= 2;
            else
                routedtrack &= ~2;
            // no repaint neccessary
            return returnvalue;
        }
        else if (entrydir == rdE) {
            returnvalue = rdW;
            if (setroute)
                routedtrack |= 2;
            else
                routedtrack &= ~2;
            // no repaint neccessary
            return returnvalue;
        }
    }

    else if (classid == siciTur) {

        //  / (top)
        // /
        if (entrydir == rdSW) {
            returnvalue = rdNE;
            if (setroute)
                routedtrack |= 1;
            else
                routedtrack &= ~1;
            // repaint neccessary
        }
        else if (entrydir == rdNE) {
            returnvalue = rdSW;
            if (setroute)
                routedtrack |= 1;
            else
                routedtrack &= ~1;
            // repaint neccessary
        }

        /* \  (tunnel)
            \ */
        else if (entrydir == rdNW) {
            returnvalue = rdSE;
            if (setroute)
                routedtrack |= 2;
            else
                routedtrack &= ~2;
            // no repaint neccessary
            return returnvalue;
        }
        else if (entrydir == rdSE) {
            returnvalue = rdNW;
            if (setroute)
                routedtrack |= 2;
            else
                routedtrack &= ~2;
            // no repaint neccessary
            return returnvalue;
        }
    }

    else if (classid == siciTul) {

        //  / (tunnel)
        // /
        if (entrydir == rdSW) {
            returnvalue = rdNE;
            if (setroute)
                routedtrack |= 2;
            else
                routedtrack &= ~2;
            // no repaint neccessary
            return returnvalue;
        }
        else if (entrydir == rdNE) {
            returnvalue = rdSW;
            if (setroute)
                routedtrack |= 2;
            else
                routedtrack &= ~2;
            // no repaint neccessary
            return returnvalue;
        }

        /* \  (top)
            \ */
        else if (entrydir == rdNW) {
            returnvalue = rdSE;
            if (setroute)
                routedtrack |= 1;
            else
                routedtrack &= ~1;
            // repaint neccessary
        }
        else if (entrydir == rdSE) {
            returnvalue = rdNW;
            if (setroute)
                routedtrack |= 1;
            else
                routedtrack &= ~1;
            // repaint neccessary
        }
    }


    // repaint element according to new routing state
    if (returnvalue != rdCenter)
        setRouted(setroute);

    return returnvalue;
}

/**
 * This slot is used for BiDi GL address messages
 */
void element::slotOccupyElement(unsigned int bus, unsigned int contact,
        bool ostate, unsigned int gladdress)
{
    /*track indicator*/
    if (bus == iFBBusNo) {

        // address panels don't get occupied
        if (classid == siciAdr) {

            //check if BiDi address indicator
            if (iSoldInvert == 2) {

                if (contact == (unsigned int)iFBContact)
                    updateTrainNumber(gladdress);
            }
        }
    }
}

/**
 * This slot is always called if a feedback port toggles.
 */
void element::slotOccupyElement(unsigned int bus, unsigned int contact,
        bool ostate)
{
    /*track indicator*/
    if (bus == iFBBusNo) {

        // address panels don't get occupied
        if (classid == siciAdr) {
            //check if EDiTS-Pro indicator
            if (iSoldInvert == 0) {
                unsigned int targetmod = (contact - 1) / 8 + 1;
                unsigned int selfmod = (iFBContact - 1) / 8 + 1;

                if (targetmod == selfmod)
                    updateEDiTSAddress(contact, ostate);
            }
            //reset BiDi address value if occupy section is left
            else if ((iSoldInvert == 2) && (contact == (unsigned int)iFBContact)
                    && !ostate) {
                updateTrainNumber(0);
            }
        }
        else if (contact == (unsigned int)iFBContact)
            setOccupied(ostate);
    }

    /* trigger 1 on and off*/
    if (enable1fbtrigger) {
        if ((bus == button1fbbus) && (contact == button1fbcontact)) {

            switch (classid) {
                case siciRel:
                    switchToDir(ostate ? 1 : 0);
                    break;
                default:
                    break;
            }
        }
    }

    /*handle aspect triggers*/
    if (ostate) {
        /*halt aspect trigger*/
        if (octriggerhalt) {
            if ((bus == fbtriggerhaltbus) &&
                    (contact == fbtriggerhaltcontact) && (state != 0)) {
                switchToDir(0);
            }
        }
        /*clear aspect trigger*/
        if (octriggerclear) {
            if ((bus == fbtriggerclearbus) &&
                    (contact == fbtriggerclearcontact) && (state == 0)) {
                switchToDir(1);
            }
        }
    }
    else {
        /*clear aspect trigger*/
        if (retriggerclear) {
            if ((bus == fbtriggerhaltbus) &&
                    (contact == fbtriggerhaltcontact) && (state == 0)) {
                switchToDir(1);
            }
        }
        /*halt aspect trigger*/
        if (retriggerhalt) {
            if ((bus == fbtriggerclearbus) &&
                    (contact == fbtriggerclearcontact) && (state != 0)) {
                switchToDir(0);
            }
        }
    }

    /*handle button triggers*/
    /*shortcut if (state = 0) => button release message*/
    if (!ostate)
        return;

    /*button 1 trigger only on*/
    if (enable1fbtrigger) {
        if ((bus == button1fbbus) && (contact == button1fbcontact)) {

            switch (classid) {
                case siciHss3:
                case siciHs1:
                case siciHs3:
                case siciZt1:
                case siciZt3:
                    emit elementClicked(this, kZfsClicked);
                    break;
                case siciHss1:
                case siciSs1:
                case siciSs3:
                case siciSd1:
                case siciSd3:
                case siciRt1:
                case siciRt3:
                    emit elementClicked(this, kRfsClicked);
                    break;
                case siciSh1:
                case siciSh3:
                    emit elementClicked(this, kZhsClicked);
                    break;
                default:
                    /*Check*/
                    emit elementClicked(this, kTurnoutClicked);
                    break;
            }
        }
    }

    /*button 2 trigger*/
    if (enable2fbtrigger) {
        if ((bus == button2fbbus) && (contact == button2fbcontact)) {
            switch (classid) {
                case siciHss3:
                case siciSd1:
                case siciSd3:
                    emit elementClicked(this, kRfsClicked);
                    break;
                case siciHss1:
                    emit elementClicked(this, kZfsClicked);
                    break;
                default:
                    break;
            }
        }
    }
}


/**
 * calculate the EDiTS address by bit field manipulation
 */
void element::updateEDiTSAddress(unsigned int contact, bool bstate)
{
    /*
     * address range for contact is 1..8, 9..16, 17..24, etc.
     * first mask three lower address bits (7), then evaluate bit value
     */
    unsigned int address = (contact - 1) & 7u;
    unsigned int bit = 1u << address;

    if (bstate)
        editsAddress = editsAddress | bit;
    else
        editsAddress = editsAddress & ~bit;

    /*
     * Most probably this is done eight times, due to eight feedback
     * address bits. If this eight feedback states are send in a fixed
     * sequence, it would be possible to check for the last arriving
     * bit and update the icon only once:
     * if (address == 7)
     */
    setupElementIcon();
}


/**
 * Set train number to EDiTS address view.
 * Only possible, if is in inverted state.
 */
void element::updateTrainNumber(unsigned int value)
{
    if ((iSoldInvert >= 1) && (value != editsAddress)) {
       editsAddress = value;
       setupElementIcon();
    }
}


/**
 * return true if this a train number display
 */
bool element::isTrainNumberDisplay()
{
    return (classid == siciAdr);
}


/**
 * Switch occupational state. If element was routed and occupation is
 * removed, route state is also removed.
 */
void element::setOccupied(bool ostate)
{
    if (occupied != ostate) {
        occupied = ostate;
        if (!ostate && routed)
            routed = false;
        /*TODO: repaint only if (!trackindicatoroff)*/
        setupElementIcon();
    }
}


void element::setRouted(bool rstate)
{
    if (routed != rstate || routedtrack != 0) {
        routed = rstate;
        /*TODO: repaint only if (!trackindicatoroff)*/
        setupElementIcon();
    }
}

/*siciMdc*/
void element::slotUpdateCommanderData(int keyno, int keycolor)
{
    address1 = address2 + keyno - 1;
    state = keycolor;

    setupElementIcon();
    sendSrcpState();
}


void element::writeFileTextToStream(QTextStream& ts)
{
    ts << GF_CLASSID << DS << classid << Qt::endl
        << GF_INDEX << DS << iSoldIndex<< Qt::endl;

    switch (classid) {

        case siciSt1:
        case siciTdr:
        case siciTdb:
            ts << GF_TEXT << DS << sSoldText << Qt::endl;
            // fall through
        case siciTuh:
        case siciTuv:
        case siciTul:
        case siciTur:
        case siciSt2:
        case siciSt3:
        case siciSt4:
        case siciCr1:
        case siciCr2:
        case siciCr3:
        case siciCr4:
        case siciCl1:
        case siciCl2:
        case siciCl3:
        case siciCl4:
            ts << GF_FBPORT << DS << iFBBusNo << DS << iFBContact << Qt::endl
                << GF_HIDELEDS << DS << trackindicatoroff << Qt::endl;
            break;

        case siciTxt:
            ts << GF_INVERSTO  << DS << iSoldInvert << Qt::endl
                << GF_TEXT << DS << sSoldText << Qt::endl;
            break;

        default:
            ts << GF_INVERSTO  << DS << iSoldInvert << Qt::endl
                << GF_PROTOCOL1  << DS <<
                ((protocol1 == GaMessage::proMM) ? "M" :
                 (protocol1 == GaMessage::proDCC) ? "N" :
                 (protocol1 == GaMessage::proServer) ? "P" :
                 (protocol1 == GaMessage::proSelectrix) ? "S" : "-1") << Qt::endl
                << GF_ADDRESS1  << DS << bus1 << DS << address1 <<
                DS << port1 << Qt::endl;
            if (address2 != -1) {
                ts << GF_PROTOCOL2  << DS <<
                ((protocol2 == GaMessage::proMM) ? "M" :
                 (protocol2 == GaMessage::proDCC) ? "N" :
                 (protocol2 == GaMessage::proServer) ? "P" :
                 (protocol2 == GaMessage::proSelectrix) ? "S" : "-1") << Qt::endl
                << GF_ADDRESS2  << DS << bus2 << DS << address2 <<
                    DS << port2 << Qt::endl;
            }
            ts << GF_XCHCONN1  << DS << xchangeport1 << Qt::endl;
            if (address2 != -1) {
                ts << GF_XCHCONN2  << DS << xchangeport2 << Qt::endl;
            }
            ts << GF_DIRECTION << DS << state << Qt::endl
                << GF_SUBTYPE   << DS << iSoldSubType << Qt::endl
                << GF_TEXT      << DS << sSoldText << Qt::endl
                << GF_ACTTIME1  << DS << activetime1 << Qt::endl
                << GF_ACTTIME2  << DS << activetime2 << Qt::endl
                << GF_FBPORT    << DS << iFBBusNo << DS << iFBContact << Qt::endl
                /*for now: save two triggered buttons, if used or not*/
                << GF_BUTTON1FB << DS << enable1fbtrigger << DS
                << button1fbbus << DS << button1fbcontact << Qt::endl
                << GF_BUTTON2FB << DS << enable2fbtrigger << DS
                << button2fbbus << DS << button2fbcontact << Qt::endl
                << GF_TRIGGERHALT << DS << octriggerhalt << DS
                << retriggerclear << DS << fbtriggerhaltbus << DS
                << fbtriggerhaltcontact << Qt::endl
                << GF_TRIGGERCLEAR << DS << octriggerclear << DS
                << retriggerhalt << DS << fbtriggerclearbus << DS
                << fbtriggerclearcontact << Qt::endl
                << GF_HIDELEDS  << DS << trackindicatoroff << Qt::endl;

            break;
    }
    ts << '%' << Qt::endl;
    modified = false;
}


QString element::getLabelText() const
{
    return sSoldText;
}


bool element::hasSameAddress(int bus, int address)
{
    return (bus == bus1 && address == address1
            && address1 > 0);
}


bool element::isLocked()
{
    return (lockCounter > 0);
}


/*already locked crossings are not lockable for a second time*/
bool element::isLockable()
{
    bool result = true;

    if ((classid == siciKrh || classid == siciKr1 || classid == siciKl1)
        && (lockCounter > 0))
            result = false;

    return result;
}

/*return occupancy state condering current track indicator state*/
bool element::isOccupied()
{
    if (trackindicatoroff)
        return false;

    return occupied;
}

/*increase or decrease lock state and repaint element if necessary*/
void element::setLocked(bool lock)
{
    if (lock) {
        ++lockCounter;
        if (lockCounter == 1)
            setupElementIcon();
    }
    else if (lockCounter > 0) {
        --lockCounter;
        if (lockCounter == 0)
            setupElementIcon();
    }
}


/**
 * switch "Fahrstrassenfestlegemelder" on or off;
 */
void element::activateFfM(bool active)
{
    if (ffm)
        if (ffmactive != active) {
            ffmactive = active;
            setupElementIcon();
        }
}

/**
 * test if "Fahrstrassenfestlegemelder" is switched on
 */
bool element::hasFfMLock()
{
    return ffmactive;
}


void element::getStateData(stateElement& se)
{
    se.name = sSoldText;
    se.bus = bus1;
    se.address = address1;
    se.state = state;
    se.elemPtr = this;
}


bool element::hasDifferentState(int dir)
{
    return state != dir;
}


bool element::hasShuntingRouteButtonOnly()
{
    return (classid == siciSs1 || classid == siciSs3 ||
            classid == siciSd1 || classid == siciSd3 ||
            classid == siciRt1 || classid == siciRt3 ||
            classid == siciWs1 || classid == siciWs3);
}


bool element::hasLEDsOn()
{
    return (!trackindicatoroff);
}


int element::getAddressCount()
{
    int returnvalue = 0;

    switch (classid) {
        // virtual addresses
        case siciZt1:
        case siciZt3:
        case siciRt1:
        case siciRt3:
        case siciKr1:
        case siciKl1:
        case siciKrh:
        case siciAdr:
        case siciBld:
            // real addresses
            //signals
        case siciSs1:
        case siciSs3:
        case siciSh1:
        case siciSh3:
        case siciSd1:
        case siciSd3:
        case siciWs1:
        case siciWs3:
        case siciZp1:
        case siciZp3:
            //turnouts
        case siciTr1:
        case siciTr3:
        case siciTl1:
        case siciTl3:
        case siciIr1:
        case siciIr3:
        case siciIl1:
        case siciIl3:
        case siciSy1:
        case siciSy3:
            // tools
        case siciDre:
        case siciEnk:
        case siciRel:
        case siciMdc:
            returnvalue = 1;
            break;

        case siciHs1:
        case siciHs3:
        case siciVs1:
        case siciVs3:
            if (iSoldSubType == 4)
                returnvalue = 2;
            else
                returnvalue = 1;
            break;

        case siciDr1:
        case siciDl1:
            if (iSoldSubType == 1)
                returnvalue = 2;
            else
                returnvalue = 1;
            break;

        case siciHss1:
        case siciHss3:
        case siciTw1:
        case siciTw3:
        case siciSr1:
        case siciSr3:
        case siciSl1:
        case siciSl3:
            returnvalue = 2;
            break;

        default:
            break;
    }
    return returnvalue;
}


void element::updateFeedbackState()
{
    // get current feedback status from server to update LEDstate
    if ((!trackindicatoroff) && (iFBContact > 0)) {

        FbMessage sm = FbMessage(SrcpMessage::mtFbGet, iFBBusNo,
                iFBContact);
        //sm.setFbData(iFBBusNo,
        //        (SrcpMessage::Feedback) pref.fbmoduletype, iFBContact);
        emit sendSrcpMessage(&sm);
    }
}


/*
 * update element if system font is changed e.g. by qtconfig
 * do nothing for empty element with no text
 */
void element::fontChange(const QFont&)
{
    if (classid == siciTxt && (sSoldText.isEmpty() || sSoldText == "-1"))
        return;
    else
        setupElementIcon();
}


bool element::sendSRCP08InitGA(unsigned int gano)
{
    bool returnvalue = false;

    if (isSwitchable()) {

        if (gano == 1) {
            GaMessage sm = GaMessage(SrcpMessage::mtGaInit, bus1, address1,
                    protocol1, 0, 0, 0);
            // sm.setGaData(protocol1, bus1, address1, 0, 0, 0);
            emit sendSrcpMessage(&sm);
        }
        else {
            GaMessage sm = GaMessage(SrcpMessage::mtGaInit, bus2, address2,
                    protocol2, 0, 0, 0);
            //sm.setGaData(protocol2, bus2, address2, 0, 0, 0);
            emit sendSrcpMessage(&sm);
        }

        returnvalue = true;
    }
    return returnvalue;
}


int element::getAddress1()
{
    return address1;
}


int element::getAddress2()
{
    return address2;
}


int element::getGA1BusNo()
{
    return bus1;
}


int element::getGA2BusNo()
{
    return bus2;
}


int element::getFBBusNo()
{
    return iFBBusNo;
}


bool element::hasThreeStates()
{
    return (classid == siciHs1 || classid == siciHs3 ||
            classid == siciVs1 || classid == siciVs3) &&
        iSoldSubType == 6;
}


bool element::isSwitched()
{
    return switched;
}


void element::setSwitched(bool sw)
{
    switched = sw;
}


/*
 * Blinking in original is one time in old direction and five times in new
 * direction resulting in 14 switch cycles.
 *
 * dir old ->| |<- new
 * on  --+ +-+ +-+ +-+ +-+ +-+ +-+ +--
 *      0| |1| |2| |3| |4| |5| |6| |7
 * off   +-+ +-+ +-+ +-+ +-+ +-+ +-+
 *       ^     ^
 *       |     |
 *       |     change visible direction
 *       send SRCP command
*/
void element::switchToDirBlinking(int ndir)
{
    // save last state for single-slip and double-slip switches
    lastdir = state;

    if (blinkcounter != 0) {
        blinkcounter = 1;
        newdir = ndir;
    }
    else {
        blinkcounter = 1;
        newdir = ndir;
        runTurnoutBlinkTimer();
    }
}


void element::runTurnoutBlinkTimer()
{
    if (blinkcounter == 1) {
         int olddir = state;
         state = newdir;
         sendSrcpState();
         // shortcut to prevent blinking
         if (!pref.blinkingturnouts) {
             blinkcounter = 14;
             setLightsOn(false);
         }
         else
             state = olddir;
    }
    if (blinkcounter % 2 == 1)
        setLightsOn(false);
    else {
        if (blinkcounter == 4)
            state = newdir;
        setLightsOn(true);
    }
    ++blinkcounter;
    if (blinkcounter < 13)
        QTimer::singleShot(500, this, SLOT(runTurnoutBlinkTimer()));
    else {
        blinkcounter = 0;
        if (!tablelight) {
            setupElementIcon();
            //repaint();
        }
        emit turnoutIsSwitched();
    }
}


void element::setLightsOn(bool ison)
{
    if (ison != lightson) {
        lightson = ison;
        setupElementIcon();
        repaint();
    }
}


bool element::routeDirectionsDiffer()
{
    return (penultimateroutedir != lastrouteentrydir);
}
/*
 * return direction an activated route will enter this signal
 */
unsigned int element::entryDir()
{
    unsigned returnvalue = rdCenter;

    // all signals starting routes to right
    if (
            classid == siciHs1 || classid == siciHss1 ||
            classid == siciSs1 || classid == siciSh1 ||
            classid == siciSd1 || classid == siciWs1 ||
            classid == siciZt1 || classid == siciRt1
       )
        returnvalue = rdW;

    // all signals starting routes to left
    else if (
            classid == siciHs3 || classid == siciHss3 ||
            classid == siciSs3 || classid == siciSh3 ||
            classid == siciSd3 || classid == siciWs3 ||
            classid == siciZt3 || classid == siciRt3
            )
        returnvalue = rdE;

    return returnvalue;
}


/*change table light state for turnouts*/
void element::setTableLight(bool ison)
{
    if (tablelight != ison) {
        tablelight = ison;
        if (turnout)
            setupElementIcon();
    }
}


bool element::showsStop()
{
    return (signal && (state == 0));
}


/*all elements that can receive feedback contact information via
 * drag-and-drop*/
bool element::canReceiveFbcDrop()
{
    /*exclude level crossing and virtual address indicator*/
    return routable && !(classid == siciBue ||
            (classid == siciAdr && iSoldInvert == 1));
}


void element::setDroppedFbContact(QByteArray& data)
{
    //wrap iFBBusNo, iFBContact, occupied
    FbContact fbc;

    if (data.size() != sizeof(fbc))
        return;

    memcpy(&fbc, data.data(), data.size());
    selectionMode = ksmNormal;
    iFBBusNo = fbc.bus;
    iFBContact = fbc.contact;
    occupied = fbc.state;
    trackindicatoroff = false;
    setupElementIcon();
}


bool element::hasLabel()
{
    return switchable || routemark || classid == siciSt1 || classid == siciTxt
        || classid == siciTdr || classid == siciTdl || classid == siciTdb;
}


bool element::hasTrackIndicator()
{
    return canReceiveFbcDrop();
}


int element::driveCount()
{
    if (hasVirtualAddress() || classid == siciAdr)
        return 0;

    return getAddressCount();
}


bool element::hasVirtualAddress()
{
    bool returnvalue = false;

    switch (classid) {
        case siciZt1:
        case siciZt3:
        case siciRt1:
        case siciRt3:
        case siciKr1:
        case siciKl1:
        case siciKrh:
        case siciBld:
            returnvalue = true;
            break;
        case siciAdr:
            if (iSoldInvert == 1)
                returnvalue = true;
            break;
        default:
            break;
    }
    return returnvalue;
}


bool element::hasVariants()
{
    return classid == siciHs1 || classid == siciHs3 ||
        classid == siciHss1 || classid == siciHss3 ||
        classid == siciDl1 || classid == siciDr1 ||
        classid == siciEnk ||
        classid == siciVs1 || classid == siciVs3 ||
        classid == siciAdr;
}


int element::buttonCount()
{
    if (classid == siciHss1 || classid == siciHss3
            || classid == siciSd1 || classid == siciSd3)
        return 2;

    if (routemark || turnout)
        return 1;

    return 0;
}

bool element::showLabelDialog()
{
    bool returnvalue = false;

    ElementLabelDialog* dlg = new ElementLabelDialog(this);
    if (dlg == NULL)
        return false;

    /*move dialog to mouse click point*/
    dlg->move(QCursor::pos());
    dlg->setSymbolText(sSoldText);
    if (dlg->exec() == QDialog::Accepted) {
        sSoldText = dlg->getSymbolText();
        returnvalue = true;
        setupElementIcon();
    }
    delete dlg;
    return returnvalue;
}

bool element::showTrackIndicatorDialog()
{
    bool returnvalue = false;

    //TODO: rename FeedbackTriggerDialog()
    FeedbackTriggerDialog* dlg = new FeedbackTriggerDialog(this);
    if (dlg == NULL)
        return false;

    /*move dialog to mouse click point*/
    dlg->move(QCursor::pos());
    dlg->setWindowTitle(tr("Edit item #%1").arg(iSoldIndex));
    dlg->setCheckBoxText(tr("&Enable track indicator"));
    dlg->enableTrigger(!trackindicatoroff);
    dlg->setFBBus(iFBBusNo);
    dlg->setFBContact(iFBContact);
    connect(dlg, SIGNAL(sigShowFBmodules()),
            this, SIGNAL(sigShowFBmodules()));

    if (dlg->exec() == QDialog::Accepted) {
        trackindicatoroff = !dlg->isTriggerEnabled();
        iFBBusNo = dlg->getFBBus();
        iFBContact = dlg->getFBContact();
        returnvalue = true;
        setupElementIcon();
    }

    disconnect(dlg, SIGNAL(sigShowFBmodules()),
            this, SIGNAL(sigShowFBmodules()));

    delete dlg;
    return returnvalue;
}

bool element::showDriveDialog()
{
    bool returnvalue = false;

    DriveDialog* dlg = new DriveDialog(this);
    if (dlg == NULL)
        return false;

    /*move dialog to mouse click point*/
    dlg->move(QCursor::pos());

    switch(classid) {
        case siciMdc:
            dlg->setProtocol(protocol2);
            dlg->setActiveTime(activetime2);
            dlg->setSRCPBus1(bus2);
            dlg->setAddress1(address2);
            dlg->setXChangeConn1(xchangeport2);
            dlg->setPort1(port2);
            break;
        default:
            dlg->setProtocol(protocol1);
            dlg->setActiveTime(activetime1);
            dlg->setSRCPBus1(bus1);
            dlg->setAddress1(address1);
            dlg->setXChangeConn1(xchangeport1);
            dlg->setPort1(port1);
            break;
    }

    if (dlg->exec() == QDialog::Accepted) {

        switch(classid) {
            case siciMdc:
                protocol2 =
                    (GaMessage::Protocol) dlg->getProtocol();
                activetime2 = dlg->getActiveTime();
                bus2 = dlg->getSRCPBus1();
                address2 = dlg->getAddress1();
                xchangeport2 = dlg->getXChangeConn1();
                port2 = dlg->getPort1();
                if (sSoldText.isEmpty())
                    sSoldText.setNum(address2);
                break;
            default:
                protocol1 =
                    (GaMessage::Protocol) dlg->getProtocol();
                activetime1 = dlg->getActiveTime();
                bus1 = dlg->getSRCPBus1();
                address1 = dlg->getAddress1();
                xchangeport1 = dlg->getXChangeConn1();
                port1 = dlg->getPort1();
                if (sSoldText.isEmpty())
                    sSoldText.setNum(address1);
                break;
                break;
        }
        returnvalue = true;
        setupElementIcon();
    }

    delete dlg;
    return returnvalue;
}

bool element::showDualDriveDialog()
{
    bool returnvalue = false;

    DualDriveDialog* dlg = new DualDriveDialog(this);
    if (dlg == NULL)
        return false;

    /*move dialog to mouse click point*/
    dlg->move(QCursor::pos());
    dlg->setProtocol1(protocol1);
    dlg->setActiveTime1(activetime1);
    dlg->setSRCPBus1(bus1);
    dlg->setAddress1(address1);
    dlg->setXChangeConn1(xchangeport1);
    dlg->setPort1(port1);

    dlg->setProtocol2(protocol2);
    dlg->setActiveTime2(activetime2);
    dlg->setSRCPBus2(bus2);
    dlg->setAddress2(address2);
    dlg->setXChangeConn2(xchangeport2);
    dlg->setPort2(port2);

    if (dlg->exec() == QDialog::Accepted) {
        protocol1 = dlg->getProtocol1();
        activetime1 = dlg->getActiveTime1();
        bus1 = dlg->getSRCPBus1();
        address1 = dlg->getAddress1();
        xchangeport1 = dlg->getXChangeConn1();
        port1 = dlg->getPort1();
        if (sSoldText.isEmpty())
            sSoldText.setNum(address1);

        protocol2 = dlg->getProtocol2();
        activetime2 = dlg->getActiveTime2();
        bus2 = dlg->getSRCPBus2();
        address2 = dlg->getAddress2();
        xchangeport2 = dlg->getXChangeConn2();
        port2 = dlg->getPort2();
        returnvalue = true;
        setupElementIcon();
    }

    delete dlg;
    return returnvalue;
}


bool element::showVirtualAddressDialog()
{
    bool returnvalue = false;

    VirtualAddressDialog* dlg = new VirtualAddressDialog(this);
    if (dlg == NULL)
        return false;

    /*move dialog to mouse click point*/
    dlg->move(QCursor::pos());
    dlg->setClassId(classid);
    dlg->setSRCPBus1(bus1);
    dlg->setAddress1(address1);

    if (dlg->exec() == QDialog::Accepted) {
        bus1 = dlg->getSRCPBus1();
        address1 = dlg->getAddress1();
        returnvalue = true;
        setupElementIcon();
    }

    delete dlg;
    return returnvalue;
}

bool element::showVariantDialog()
{
    bool returnvalue = false;
    QPixmap pm;

    VariantDialog* dlg = new VariantDialog(this);
    if (dlg == NULL)
        return false;

    /*move dialog to mouse click point*/
    dlg->move(QCursor::pos());

    switch (classid) {
        case siciHss1:
        case siciHss3:
            pm = QPixmap(signal_hss_st1_xpm);
            dlg->addVariant(tr("Hp&0, Hp1 and Sh1"), pm);
            pm = QPixmap(signal_hss_st2_xpm);
            dlg->addVariant(tr("Hp0, Hp&2 and Sh1"), pm);
            pm = QPixmap(signal_hss_st3_xpm);
            dlg->addVariant(tr("Hp0, Hp&1, Hp2 and Sh1"), pm);
            /*
             * index Subtype
             * -------------
             *   0     1
             *   1     7
             *   2     5
             * -------------
             */
            if (iSoldSubType == 1)
                dlg->setChoice(0);
            else if (iSoldSubType == 7)
                dlg->setChoice(1);
            else if (iSoldSubType == 5)
                dlg->setChoice(2);
            break;

        case siciHs1:
        case siciHs3:
            pm = QPixmap(signal_hs_st1_xpm);
            dlg->addVariant(tr("Hp&0 and Hp1"), pm);
            pm = QPixmap(signal_hs_st2_xpm);
            dlg->addVariant(tr("Hp0 and Hp&2"), pm);
            pm = QPixmap(signal_hs_st3_xpm);
            dlg->addVariant(tr("Hp0, Hp&1 and Hp2"), pm);
            /*
             * index Subtype
             * -------------
             *   0     0
             *   1     6
             *   2     4
             * -------------
             */
            if (iSoldSubType == 0)
                dlg->setChoice(0);
            else if (iSoldSubType == 6)
                dlg->setChoice(1);
            else if (iSoldSubType == 4)
                dlg->setChoice(2);
            break;

        case siciVs1:
        case siciVs3:
            pm = QPixmap(signal_vs_st1_xpm);
            dlg->addVariant(tr("Vr&0 and Vr1"), pm);
            pm = QPixmap(signal_vs_st2_xpm);
            dlg->addVariant(tr("Vr0 and Vr&2"), pm);
            pm = QPixmap(signal_vs_st3_xpm);
            dlg->addVariant(tr("Vr0, Vr&1 and Vr2"), pm);
            /*
             * index Subtype
             * -------------
             *   0     0
             *   1     6
             *   2     4
             * -------------
             */
            if (iSoldSubType == 0)
                dlg->setChoice(0);
            else if (iSoldSubType == 6)
                dlg->setChoice(1);
            else if (iSoldSubType == 4)
                dlg->setChoice(2);
            break;

        case siciDr1:
            pm = QPixmap(dkw_rechts_st2_xpm);
            dlg->addVariant(tr("&One drive (two switch positions)"), pm);
            pm = QPixmap(dkw_rechts_st3_xpm);
            dlg->addVariant(tr("&Two drives (four switch positions)"), pm);
            dlg->setChoice(iSoldSubType);
            break;

        case siciDl1:
            pm = QPixmap(dkw_links_st2_xpm);
            dlg->addVariant(tr("&One drive (two switch positions)"), pm);
            pm = QPixmap(dkw_links_st3_xpm);
            dlg->addVariant(tr("&Two drives (four switch positions)"), pm);
            /*
             * index Subtype
             * -------------
             *   0     0
             *   1     1
             * -------------
             */
            dlg->setChoice(iSoldSubType);
            break;

        case siciAdr:
            dlg->showImage(false);
            dlg->addVariant(tr("&EDiTS-Pro indicator"));
            dlg->addVariant(tr("&Train number tracing"));
            dlg->addVariant(tr("&BiDi locomotive address"));
            /*
             * index Inverted
             * --------------
             *   0     0       feedback address
             *   1     1       virtual address
             *   2     2       BiDi address
             * --------------
             */
            // fix -1 default value
            if (iSoldInvert < 0)
                iSoldInvert = 1;
            dlg->setChoice(iSoldInvert);
            break;

        case siciEnk:
            pm = QPixmap(entkoppler_st1_xpm);
            dlg->addVariant(tr("&Bistable coupler"), pm);
            pm = QPixmap(entkoppler_st2_xpm);
            dlg->addVariant(tr("Momentary coupler on &left connector"), pm);
            pm = QPixmap(entkoppler_st3_xpm);
            dlg->addVariant(tr("Momentary coupler on &right connector"), pm);
            /*
             * index Subtype
             * -------------
             *   0    -1
             *   1     0
             *   2     1
             * -------------
             */
            if (iSoldSubType == -1)
                dlg->setChoice(0);
            else if (iSoldSubType == 0)
                dlg->setChoice(1);
            else if (iSoldSubType == 1)
                dlg->setChoice(2);
            break;

        default:
            break;
    }

    if (dlg->exec() == QDialog::Accepted) {

        switch (classid) {
            case siciHss1:
            case siciHss3:
                iSoldSubType = dlg->getChoice();
                if (iSoldSubType == 0)
                    iSoldSubType = 1;
                else if (iSoldSubType == 1)
                    iSoldSubType = 7;
                else if (iSoldSubType == 2)
                    iSoldSubType = 5;
                break;

            case siciHs1:
            case siciHs3:
            case siciVs1:
            case siciVs3:
                iSoldSubType = dlg->getChoice();
                if (iSoldSubType == 1)
                    iSoldSubType = 6;
                else if (iSoldSubType == 2)
                    iSoldSubType = 4;
                break;

            case siciDr1:
            case siciDl1:
                iSoldSubType = dlg->getChoice();
                break;

            case siciAdr:
                iSoldInvert = dlg->getChoice();
                break;

            case siciEnk:
                iSoldSubType = dlg->getChoice();
                if (iSoldSubType == 0)
                    iSoldSubType = -1;
                else if (iSoldSubType == 1)
                    iSoldSubType = 0;
                else if (iSoldSubType == 2)
                    iSoldSubType = 1;
                break;

            default:
                break;
        }
        returnvalue = true;
        setupElementIcon();
    }

    delete dlg;
    return returnvalue;
}

/*show dialog at click position, position is also used to check if left
 * or right button is edited*/
bool element::showFeedbackTriggerDialog(const QPoint& p)
{
    QString buttontext;
    bool returnvalue = false;
    bool hastwo = false;

    // select Button names; FHT WGT, WHT UfGT,...
    bool left = p.x() < (width() / 2);

    switch (classid) {
        case siciHss1:
            hastwo = true;
            if (left)
                buttontext = tr("%1 Rear").arg(sSoldText);
            else
                buttontext = tr("%1 Front").arg(sSoldText);
            break;
        case siciHss3:
            hastwo = true;
            if (left)
                buttontext = tr("%1 Front").arg(sSoldText);
            else
                buttontext = tr("%1 Rear").arg(sSoldText);
            break;
        case siciSd1:
            hastwo = true;
            if (left)
                buttontext = tr("%1 Rear").arg(sSoldText);
            else
                buttontext = tr("%1 Front").arg(sSoldText);
            break;
        case siciSd3:
            hastwo = true;
            if (left)
                buttontext = tr("%1 Front").arg(sSoldText);
            else
                buttontext = tr("%1 Rear").arg(sSoldText);
            break;
        default:
            buttontext = sSoldText;
            break;
    }

    FeedbackTriggerDialog* dlg = new FeedbackTriggerDialog(this);
    if (dlg == NULL)
        return false;

    /*move dialog to mouse click point*/
    dlg->move(QCursor::pos());
    dlg->setWindowTitle(buttontext);
    dlg->setCheckBoxText(tr("&Enable feedback trigger"));
    if (left || !hastwo) {
        dlg->enableTrigger(enable1fbtrigger);
        dlg->setFBBus(button1fbbus);
        dlg->setFBContact(button1fbcontact);
    }
    else {
        dlg->enableTrigger(enable2fbtrigger);
        dlg->setFBBus(button2fbbus);
        dlg->setFBContact(button2fbcontact);
    }
    connect(dlg, SIGNAL(sigShowFBmodules()),
            this, SIGNAL(sigShowFBmodules()));

    if (dlg->exec() == QDialog::Accepted) {
        if (left || !hastwo) {
            enable1fbtrigger = dlg->isTriggerEnabled();
            button1fbbus = dlg->getFBBus();
            button1fbcontact = dlg->getFBContact();
        }
        else {
            enable2fbtrigger = dlg->isTriggerEnabled();
            button2fbbus = dlg->getFBBus();
            button2fbcontact = dlg->getFBContact();
        }
        returnvalue = true;
        setupElementIcon();
    }

    disconnect(dlg, SIGNAL(sigShowFBmodules()),
            this, SIGNAL(sigShowFBmodules()));

    delete dlg;
    return returnvalue;
}


bool element::showTriggerHaltDialog()
{
    bool returnvalue = false;
    QString title;

    TriggerAspectDialog* dlg = new TriggerAspectDialog(this);
    if (dlg == NULL)
        return false;

    title = tr("Trigger aspect of signal '%1'").arg(sSoldText);
    /*move dialog to mouse click point*/
    dlg->move(QCursor::pos());
    dlg->setWindowTitle(title);
    dlg->setOccupationCbText(tr("&Occupation triggers halt aspect"));
    dlg->setReleaseCbText(tr("&Release triggers clear aspect"));
    dlg->enableOccupationTrigger(octriggerhalt);
    dlg->enableReleaseTrigger(retriggerclear);
    dlg->setFBBus(fbtriggerhaltbus);
    dlg->setFBContact(fbtriggerhaltcontact);

    connect(dlg, SIGNAL(sigShowFBmodules()),
            this, SIGNAL(sigShowFBmodules()));

    if (dlg->exec() == QDialog::Accepted) {
        octriggerhalt = dlg->occupationTriggerEnabled();
        retriggerclear = dlg->releaseTriggerEnabled();
        fbtriggerhaltbus = dlg->getFBBus();
        fbtriggerhaltcontact = dlg->getFBContact();

        returnvalue = true;
        setupElementIcon();
    }

    disconnect(dlg, SIGNAL(sigShowFBmodules()),
            this, SIGNAL(sigShowFBmodules()));

    delete dlg;
    return returnvalue;
}


bool element::showTriggerClearDialog()
{
    bool returnvalue = false;
    QString title;

    TriggerAspectDialog* dlg = new TriggerAspectDialog(this);
    if (dlg == NULL)
        return false;

    title = tr("Trigger aspect of signal '%1'").arg(sSoldText);
    /*move dialog to mouse click point*/
    dlg->move(QCursor::pos());
    dlg->setWindowTitle(title);
    dlg->setOccupationCbText(tr("&Occupation triggers clear aspect"));
    dlg->setReleaseCbText(tr("&Release triggers halt aspect"));
    dlg->enableOccupationTrigger(octriggerclear);
    dlg->enableReleaseTrigger(retriggerhalt);
    dlg->setFBBus(fbtriggerclearbus);
    dlg->setFBContact(fbtriggerclearcontact);

    connect(dlg, SIGNAL(sigShowFBmodules()),
            this, SIGNAL(sigShowFBmodules()));

    if (dlg->exec() == QDialog::Accepted) {
        octriggerclear = dlg->occupationTriggerEnabled();
        retriggerhalt = dlg->releaseTriggerEnabled();
        fbtriggerclearbus = dlg->getFBBus();
        fbtriggerclearcontact = dlg->getFBContact();

        returnvalue = true;
        setupElementIcon();
    }

    disconnect(dlg, SIGNAL(sigShowFBmodules()),
            this, SIGNAL(sigShowFBmodules()));

    delete dlg;
    return returnvalue;
}


bool element::hasFeedbackTrigger()
{
    return (buttonCount() > 0) || (classid == siciRel);
}


/*setup property menu and keep click position */
void element::runPropertyMenue(const QPoint& p)
{
    QMenu* propmenu = new QMenu(this);
    propmenu->setObjectName("propertyMenu");
    int count = 0;

    if (hasVariants()) {
        QAction* variantaction = propmenu->addAction(tr("&Variant..."));
        variantaction->setData(1);
    }
    if (hasLabel()) {
        QAction* labelaction = propmenu->addAction(tr("&Label..."));
        labelaction->setData(2);
    }
    count = driveCount();
    if (count == 1) {
        QAction* driveaction = propmenu->addAction(tr("&Drive..."));
        driveaction->setData(3);
    }
    else if (count > 1) {
        QAction* drivesaction = propmenu->addAction(tr("&Drives..."));
        drivesaction->setData(4);
    }
    if (hasVirtualAddress()) {
        QAction* virtaction = propmenu->addAction(tr("Virtual &address..."));
        virtaction->setData(5);
    }
    if (hasTrackIndicator()) {
        QAction* trackaction = propmenu->addAction(tr("&Track indicator..."));
        trackaction->setData(6);
    }
    if (hasFeedbackTrigger()) {
        QAction* triggerbuttonaction = propmenu->addAction(
                tr("Trigger &button..."));
        triggerbuttonaction->setData(7);
    }
    if (isSignal() && !(classid == siciRt1 || classid == siciRt3 ||
                classid == siciZt1 || classid == siciZt3)) {
        QAction* triggerhaltaction = propmenu->addAction(
                tr("Trigger &halt aspect..."));
        triggerhaltaction->setData(8);
        QAction* triggerclearaction = propmenu->addAction(
                tr("Trigger &clear aspect..."));
        triggerclearaction->setData(9);
    }
    if (!propmenu->isEmpty()) {
        QAction* ac = propmenu->exec(QCursor::pos());
        if (ac != NULL) {
            bool elchanged = false;
            switch(ac->data().toInt()) {
                case 1:
                    elchanged = showVariantDialog();
                    break;
                case 2:
                    elchanged = showLabelDialog();
                    break;
                case 3:
                    elchanged = showDriveDialog();
                    break;
                case 4:
                    elchanged = showDualDriveDialog();
                    break;
                case 5:
                    elchanged = showVirtualAddressDialog();
                    break;
                case 6:
                    elchanged = showTrackIndicatorDialog();
                    break;
                case 7:
                    elchanged = showFeedbackTriggerDialog(p);
                    break;
                case 8:
                    elchanged = showTriggerHaltDialog();
                    break;
                case 9:
                    elchanged = showTriggerClearDialog();
                    break;
                case -1: //fall through
                default:
                    break;
            }
            if (elchanged)
                modified = true;
        }
    }
    delete propmenu;
}


void element::dragEnterEvent(QDragEnterEvent* e)
{
    if (visualMode != kvmEditLayout)
        return;

    if (e->mimeData()->hasFormat(MIME_FBC)) {

        if (canReceiveFbcDrop()) {
            setDropTargetView(true);
            e->acceptProposedAction();
        }
    }
}


void element::dragMoveEvent(QDragMoveEvent *e)
{
    e->acceptProposedAction();
}


void element::dragLeaveEvent(QDragLeaveEvent* e)
{
    if (visualMode != kvmEditLayout)
        return;

    setDropTargetView(false);
}

void element::dropEvent(QDropEvent *e)
{
    if (visualMode != kvmEditLayout)
        return;

    if (e->mimeData()->hasFormat(MIME_FBC)) {
        if (canReceiveFbcDrop()) {
            QByteArray data = e->mimeData()->data(MIME_FBC);
            setDroppedFbContact(data);
            modified = true;
            e->acceptProposedAction();
        }
    }
}
